/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
                          generator.cpp  -  description
                             -------------------
    begin                : sam jan 17 2004
    copyright            : (C) 2004 by avila
    email                : avila@deimos
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "generator.h"
#include <time.h>
#include <string>
#include <fstream>
#include <math.h>
#include <getopt.h>

/***************************************************************************
 *             GENERATORSALLE                                              *
 ***************************************************************************/

GeneratorSalle::GeneratorSalle (int nbS, TypeSalle ts, TypeEnvironnement te)
{
  nbSalle = nbS;
  type = ts;
  environnement = te;
  placementEffectue = false;
  for (int i = 0; i != MAX_ARME; i++)
    nbArme[i] = 0;
  for (int i = 0; i != MAX_ARMURE; i++)
    nbArmure[i] = 0;
  for (int i = 0; i != MAX_VIE; i++)
    nbVie[i] = 0;
  for (int i = 0; i != MAX_MUNITION; i++)
    nbMunition[i] = 0;
  for (int i = 0; i != MAX_NAIN; i++)
    nbNain[i] = 0;
  for (int i = 0; i != MAX_SECRET; i++)
    nbSecret[i] = 0;
  for (int i = 0; i != MAX_TRESOR; i++)
    nbTresor[i] = 0;
  for (int i = 0; i != MAX_CLE; i++)
    nbCle[i] = 0;
  for (int i = 0; i != MAX_PORTE; i++)
    nbPorte[i] = 0;
};

/***************************************************************************
 *             GENERATORLIEN                                               *
 ***************************************************************************/

GeneratorLien::GeneratorLien (GeneratorSalle * s1, GeneratorSalle * s2,
			      bool b1, bool b2)
{
  BlocLien bl, bl2;
  bl.connecte = b1;
  bl.salle = s1;
  bl2.connecte = b2;
  bl2.salle = s2;
  salles[0] = bl;
  salles[1] = bl2;
};

GeneratorSalle *
GeneratorLien::getSuivant (GeneratorSalle * courant)
{
  if (salles[0].salle == courant)
    return salles[1].salle;
  return salles[0].salle;
};

int
GeneratorLien::getSuivant (GeneratorSalle * salleCourant, BlocLien ** courant,
			   BlocLien ** suivant)
{
  if (salles[0].salle == salleCourant)
    {
      *courant = &salles[0];
      *suivant = &salles[1];
      return 1;
    };
  *courant = &salles[1];
  *suivant = &salles[0];
  return 0;
};

/***************************************************************************
 *             GENERATOR                                                   *
 ***************************************************************************/

GeneratorSalle *
Generator::subGenerationDivise (GeneratorSalle * sub, TypeSalle T, int N)
{
  GeneratorSalle *s = new GeneratorSalle (N, T, sub->environnement);
  listSalle.insert (listSalle.end (), s);
  repartition (sub, s, N, 50);
  return s;
};

void
Generator::subGenerationRegleGrandeSalle (GeneratorSalle * sub)
{
  // regle 0 : Sub -> (Sub1-Sub-Sub2) , Sub1,Sub2,Sub.n. grande salle
  // nombre de salles (1->4)
  int n = random () % 4 + 1;
  int nSalle = (sub->nbSalle - 1) / (n);
  if (nSalle == 0)
    nSalle = 1;
  for (int i = 0; i != n; i++)
    {
      GeneratorSalle *s =
	subGenerationDivise (sub, SALLE_INTERMEDIAIRE, nSalle);
      addLien (sub, s);
      // appel recursif....
      subGeneration (s);
    };
  sub->nbSalle = 1;
  sub->type = SALLE_GRANDE;
}

GeneratorSalle *
Generator::subGenerationCreerSalle (GeneratorSalle * sub, TypeSalle T)
{
  ListIndex li = trouveLien (sub);
  if (li.size () == 0)
    {
      printf ("Probleme : aucun lien\n");
      exit (0);
    }
  GeneratorLien & gl = getLien (li[0]);
  GeneratorSalle *suivant = gl.getSuivant (sub);
  GeneratorSalle *L = subGenerationDivise (sub, T, 1);
  remplaceLien (sub, suivant, L);
  addLien (sub, L);
  subGeneration (sub);
};

void
Generator::subGeneration (GeneratorSalle * sub)
{
  // Choix d'une regle...
  if (sub->nbSalle <= 1)	// Finir en grande salle (avec le risque de finir en depotoir...)
    {
      sub->type = SALLE_NORMALE;
      return;
    }
  int regle = random () % 100;
  switch (sub->environnement)
    {
    case ENV_MINE:
      printf ("MINE\n");
      if (regle < 70)
	{
	  printf ("encore un couloir...\n");
	  subGenerationCreerSalle (sub, SALLE_COULOIR);
	}
      else if (regle < 80)
	{
	  printf ("grande salle...\n");
	  subGenerationRegleGrandeSalle (sub);
	}
      else if (regle < 95)
	{
	  printf ("salle normale...\n");
	  subGenerationCreerSalle (sub, SALLE_NORMALE);
	}
      else if (regle < 100)
	{
	  printf ("exterieur...\n");
	  subGenerationCreerSalle (sub, SALLE_EXTERIEUR);
	}
      break;

    }
}

int
Generator::addLien (GeneratorSalle * s1, GeneratorSalle * s2)
{
  listLien.insert (listLien.end (), GeneratorLien (s1, s2, true, true));
  return listLien.size () - 1;
};

int
Generator::getLien (GeneratorSalle * s1, GeneratorSalle * s2)
{
  for (int i = 0; i != listLien.size (); i++)
    {
      if (((listLien[i].salles[0].salle == s1)
	   && (listLien[i].salles[1].salle == s2))
	  || ((listLien[i].salles[1].salle == s1)
	      && (listLien[i].salles[0].salle == s2)))
	return i;
    }
  return -1;
};

GeneratorLien & Generator::getLien (int index)
{
  if (index < 0)
    {
      printf ("invalid index\n");
      exit (-1);
    }
  return listLien[index];
};

void
Generator::lanceGeneration (int nbS, int dif, int comp)
{
  nbSalle = nbS;
  difficulte = dif;
  comp = complexite;

  // Creer 3  salles : debut / intermed / fin.
  GeneratorSalle *debut = new GeneratorSalle (1, SALLE_DEBUT, ENV_MINE);
  listSalle.insert (listSalle.end (), debut);
  GeneratorSalle *milieu =
    new GeneratorSalle (nbS, SALLE_INTERMEDIAIRE, ENV_MINE);
  listSalle.insert (listSalle.end (), milieu);
  int l1 = addLien (debut, milieu);

  // Rajouter des lments (armes, options, etc...) en fonction de la difficult et du nombre de salles.
  for (int i = 0; i != MAX_ARME; i++)
    milieu->nbArme[i] = int ((random () % nbS) * 0.5);
  for (int i = 0; i != MAX_ARMURE; i++)
    milieu->nbArmure[i] = random () % nbS;
  for (int i = 0; i != MAX_VIE; i++)
    milieu->nbVie[i] = random () % nbS;
  for (int i = 0; i != MAX_MUNITION; i++)
    milieu->nbMunition[i] = int ((random () % nbS) * 0.5);
  for (int i = 0; i != MAX_NAIN; i++)
    milieu->nbNain[i] = 0;
  milieu->nbNain[0] = (random () % nbS) * dif;
  for (int i = 0; i != MAX_SECRET; i++)
    milieu->nbSecret[i] = random () % nbS;
  for (int i = 0; i != MAX_TRESOR; i++)
    milieu->nbTresor[i] = random () % nbS;
  for (int i = 0; i != MAX_CLE; i++)
    milieu->nbCle[i] = random () % nbS;
  for (int i = 0; i != MAX_PORTE; i++)
    milieu->nbPorte[i] = random () % nbS;

  //return;
  GeneratorSalle *fin = new GeneratorSalle (1, SALLE_FIN, ENV_MINE);
  listSalle.insert (listSalle.end (), fin);
  int l2 = addLien (fin, milieu);

  subGeneration (milieu);
};

float
find_angle (position p1, position p2)
{
  float result = 0.0;
  int NB_SUBDIVISION = 100;
  position target = (p2 + (p1 * (-1.0))) * 1000.0;
  float last_dist = -1.0;
  for (int i = 0; i != NB_SUBDIVISION; i++)
    {
      float angle = 6.28 * float (i) / float (NB_SUBDIVISION);
      position cible (cos (angle), sin (angle), target.coord[2]);
      float dist = position ().distanceNorme1 (cible);
      if ((last_dist == -1.0) || (dist < last_dist))
	{
	  last_dist = dist;
	  result = angle;
	}
    }
  return result;
}

void
Generator::lanceGenerationContraintPlacementSpatial (int nbS, int dif,
						     int comp)
{
  // Un algo compltement stupide  la limite de la dbilit profonde.
  // Dommage, j'aurais pu faire un truc compliqu qui ne fonctionne pas.
  position position_courante (0, 0, 0);
  int MAX_TAILLE = 10;
  int MIN_TAILLE = 5;
  int contrainte = 0;		
  // 0 : haut (position haut/gauche), 1 : gauche (position gauche/bas), 2 : droite (position droite/bas)
  vector < ListBloc > listConnection;
  for (int index_salle = 0; index_salle != nbS; index_salle++)
    {
      position taille (random () % MAX_TAILLE + MIN_TAILLE,
		       random () % MAX_TAILLE + MIN_TAILLE,
		       random () % 3 + 3);

      GeneratorSalle *debut =
	new GeneratorSalle (1, index_salle ? ( (index_salle == nbS-1) ? SALLE_FIN : SALLE_NORMALE ) : SALLE_DEBUT,
			    ENV_MINE);
      listSalle.insert (listSalle.end (), debut);

      // positionner la salle en fonction de la contrainte de position.
      position min, max;
      switch (contrainte)
	{
	case 0:		// haut
	  {
	    float start_x =
	      position_courante.coord[0] -
	      (random () % (int (taille.coord[0]) - 2));
	    min = position (start_x, position_courante.coord[1], 0);
	    max =
	      position (start_x + taille.coord[0],
			position_courante.coord[1] + taille.coord[1],
			taille.coord[2]);
	  }
	  break;
	case 1:		// gauche
	  {
	    float start_y =
	      position_courante.coord[1] -
	      (random () % (int (taille.coord[1]) - 2));
	    min = position (position_courante.coord[0], start_y, 0);
	    max =
	      position (position_courante.coord[0] + taille.coord[0],
			start_y + taille.coord[1], taille.coord[2]);
	  }
	  break;
	case 2:		// droite
	  {
	    float start_y =
	      position_courante.coord[1] -
	      (random () % (int (taille.coord[1]) - 2));
	    min =
	      position (position_courante.coord[0] - taille.coord[0], start_y,
			0);
	    max =
	      position (position_courante.coord[0], start_y + taille.coord[1],
			taille.coord[2]);
	  }
	  break;
	}
      // Remplir le volume.
      debut->placementBloc = listBloc (min, max);
      // Ajouter les contraintes si c'est le premier lment. Sinon, c'est le couloir qui dtermine ca.
      if (index_salle == 0)
	{
	  listCheckpoint.insert (listCheckpoint.end (), position_courante);
	  listRotation.insert (listRotation.end (),
			       find_angle (position_courante, position ()));
	}
      debut->nbNain[0] = (random () % nbS) * dif;
      // couloir, sauf si c'est la dernire salle.
      if (index_salle != nbS - 1)
	{
	  GeneratorSalle *couloir =
	    new GeneratorSalle (1, SALLE_COULOIR, ENV_MINE);
	  listSalle.insert (listSalle.end (), couloir);
	  // Choisir la nouvelle direction dans laquelle on veut aller, sauf si c'est la derniere.
	  // (contrainte==0) contrainte=random()%3;
	  //else 
	  contrainte = 2;
	  // Choisir la longueur du couloir.
	  int longueur = random () % 10 + 1;
	  // Rajouter un couloir pour cette direction, sauf si c'est la derniere salle.
	  position cmin, cmax;
	  // choisir la position du couloir.
	  ListBloc connection;
	  switch (contrainte)
	    {
	    case 0:		// haut
	      {
		float start =
		  min.coord[0] + random () % int (max.coord[0] -
						  min.coord[0] - 2);
		cmin = position (start, max.coord[1], 0);
		cmax = position (start + 2, max.coord[1] + longueur, 2);
		position_courante = position (start, cmax.coord[1], 0);

		connection =
		  ListBloc (position
			    (cmin.coord[0], cmin.coord[1], cmin.coord[2]),
			    position (cmax.coord[0], cmin.coord[1] + 1,
				      cmax.coord[2]));
		listConnection.insert (listConnection.end (), connection);

		connection =
		  ListBloc (position
			    (cmin.coord[0], cmax.coord[1] - 1, cmin.coord[2]),
			    position (cmax.coord[0], cmax.coord[1],
				      cmax.coord[2]));
		listConnection.insert (listConnection.end (), connection);

		// Entre couloir
		listCheckpoint.insert (listCheckpoint.end (),
				       position (cmin.coord[0] + 1,
						 cmin.coord[1] - 1,
						 cmin.coord[2] + 1));
		listRotation.insert (listRotation.end (),
				     find_angle (position_courante,
						 position ()));

		// Nouvelle salle
		listCheckpoint.insert (listCheckpoint.end (),
				       position (cmin.coord[0] + 1,
						 cmax.coord[1] + 1,
						 cmin.coord[2] + 1));
		listRotation.insert (listRotation.end (),
				     find_angle (position_courante,
						 position ()));

	      }
	      break;
	    case 1:		// gauche
	      {
		float start =
		  min.coord[1] + random () % int (max.coord[1] -
						  min.coord[1] - 2);
		cmin = position (max.coord[0], start, 0);
		cmax = position (max.coord[0] + longueur, start + 2, 2);
		position_courante = position (cmax.coord[0], start, 0);
		connection =
		  ListBloc (position
			    (cmin.coord[0], cmin.coord[1], cmin.coord[2]),
			    position (cmin.coord[0] + 1, cmax.coord[1],
				      cmax.coord[2]));
		listConnection.insert (listConnection.end (), connection);
		connection =
		  ListBloc (position
			    (cmax.coord[0] - 1, cmin.coord[1], cmin.coord[2]),
			    position (cmax.coord[0], cmax.coord[1],
				      cmax.coord[2]));
		listConnection.insert (listConnection.end (), connection);

		// Entre couloir
		listCheckpoint.insert (listCheckpoint.end (),
				       position (cmin.coord[0] - 1,
						 cmin.coord[1] + 1,
						 cmin.coord[2] + 1));
		listRotation.insert (listRotation.end (),
				     find_angle (position_courante,
						 position ()));

		// Nouvelle salle
		listCheckpoint.insert (listCheckpoint.end (),
				       position (cmax.coord[0] + 1,
						 cmin.coord[1] + 1,
						 cmin.coord[2] + 1));
		listRotation.insert (listRotation.end (),
				     find_angle (position_courante,
						 position ()));

	      }
	      break;
	    case 2:		// droite
	      {
		float start =
		  min.coord[1] + random () % int (max.coord[1] -
						  min.coord[1] - 2);
		cmin = position (min.coord[0] - longueur, start, 0);
		cmax = position (min.coord[0], start + 2, 2);
		position_courante = position (cmin.coord[0], start, 0);
		connection =
		  ListBloc (position
			    (cmax.coord[0] - 1, cmin.coord[1], cmin.coord[2]),
			    position (cmax.coord[0], cmax.coord[1],
				      cmax.coord[2]));
		listConnection.insert (listConnection.end (), connection);
		connection =
		  ListBloc (position
			    (cmin.coord[0], cmin.coord[1], cmin.coord[2]),
			    position (cmin.coord[0] + 1, cmax.coord[1],
				      cmax.coord[2]));
		listConnection.insert (listConnection.end (), connection);

		// Entre couloir
		listCheckpoint.insert (listCheckpoint.end (),
				       position (cmax.coord[0] + 1,
						 cmin.coord[1] + 1,
						 cmin.coord[2] + 1));
		listRotation.insert (listRotation.end (),
				     find_angle (position_courante,
						 position ()));

		// Nouvelle salle
		listCheckpoint.insert (listCheckpoint.end (),
				       position (cmin.coord[0] - 1,
						 cmin.coord[1] + 1,
						 cmin.coord[2] + 1));
		listRotation.insert (listRotation.end (),
				     find_angle (position_courante,
						 position ()));

	      }
	      break;
	    }
	  couloir->placementBloc = listBloc (cmin, cmax);
	  // Rajouter la liste des blocs auquels il ne faut pas toucher !
	}
    }
  for (int i = 0; i < listSalle.size () - 1; i++)
    {
      int index = addLien (listSalle[i], listSalle[i + 1]);
      //if (i<listConnection.size())
      listLien[index].connection = listConnection[i];
      //else printf("Erreur avec nombre de connection...\n");
    }
};


Generator::Generator ()
{
};

Generator::~Generator ()
{
};

void
Generator::print (int level)
{
  int min[5000][5000];
  int max[5000][5000];
  int s[5000][5000];
  char li[5000][5000];
  for (int i = 0; i != 5000; i++)
    for (int j = 0; j != 5000; j++)
      {
	min[i][j] = 1000;
	max[i][j] = -1000;
	s[i][j] = -1;
	li[i][j] = ' ';
      }
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sg = listSalle[i];
      ListBloc & l = sg->placementBloc;
      for (int j = 0; j != l.size (); j++)
	{
	  position p = l[j];
	  int x = int (p.coord[0]) + 2500;
	  int y = int (p.coord[1]) + 2500;
	  int z = int (p.coord[2]);
	  s[x][y] = i;
	  if (z < min[x][y])
	    min[x][y] = z;
	  if (z > max[x][y])
	    max[x][y] = z;
	};

      string typeS;
      switch (sg->type)
	{
	case SALLE_COULOIR:
	  typeS = string ("couloir");
	  break;
	case SALLE_GRANDE:
	  typeS = string ("grande");
	  break;
	case SALLE_SECRET:
	  typeS = string ("secret");
	  break;
	case SALLE_DEBUT:
	  typeS = string ("debut");
	  break;
	case SALLE_FIN:
	  typeS = string ("fin");
	  break;
	case SALLE_NORMALE:
	  typeS = string ("normale");
	  break;
	case SALLE_EXTERIEUR:
	  typeS = string ("exterieur");
	  break;
	case SALLE_INTERMEDIAIRE:
	  typeS = string ("intermediaire");
	  break;
	}
      printf
	("SALLE %i<BR/>\nENV %i<BR/>\nTYPE %s<BR/>\nSUBTYPE <BR/>\nTEXTURE <BR/>\n",
	 i, sg->environnement, typeS.c_str (), sg->nbSalle);

      for (int j = 0; j != listLien.size (); j++)
	{
	  GeneratorLien & gl = listLien[j];
	  int i1 = getIndexSalle (gl.salles[0].salle);
	  int i2 = getIndexSalle (gl.salles[1].salle);
	  if ((i1 == i) || (i2 == i))
	    {
	      /*printf("          (%i %i)\n",i1,i2);
	         if (i1==i)
	         printf("          %i \n",i2);
	         if (i2==i)
	         printf("          %i \n",i1); */
	      if (gl.connection.size ())
		{
		  //  printf("           ");gl.connection.getMin().print();
		  //  printf("           ");gl.connection.getMax().print();
		};
	      ListBloc & l = gl.connection;
	      for (int j = 0; j != l.size (); j++)
		{
		  position p = l[j];
		  int x = int (p.coord[0]) + 2500;
		  int y = int (p.coord[1]) + 2500;
		  int z = int (p.coord[2]);
		  s[x][y] = i;
		  li[x][y] = 'L';
		  if (z < min[x][y])
		    min[x][y] = z;
		  if (z > max[x][y])
		    max[x][y] = z;
		};
	    }
	};
    };

  printf ("</TD><TD><TABLE>");
  for (int i = 200; i != 400; i++)
    {
      printf ("<TR>\n");
      for (int j = 200; j != 400; j++)
	if (s[i][j] != -1)
	  {
	    char tab[1000];
	    sprintf (tab, "%i %c<br/>", s[i][j], li[i][j]);
	    printf ("<TD>%s</TD>\n", tab);
	  }
	else
	  {
	    char pre[1000];
	    pre[0] = 0;
	    if (i == 200)
	      sprintf (pre, "%i", j - 200);
	    if (j == 200)
	      sprintf (pre, "%i", i - 200);
	    printf ("<TD>%s</TD>\n", pre);
	  }
      printf ("</TR>\n");
    }
  printf ("</TABLE></TD>");
};

int
Generator::getIndexSalle (GeneratorSalle * s)
{
  for (int i = 0; i != listSalle.size (); i++)
    if (listSalle[i] == s)
      return i;
  printf ("salle pas trouvee. pasd'bol, on fera mieux la prochaine fois.\n");
  exit (0);
};

int
Generator::remplaceLien (GeneratorSalle * s, GeneratorSalle * s1,
			 GeneratorSalle * s2)
{
  for (int j = 0; j != listLien.size (); j++)
    {
      GeneratorLien & gl = listLien[j];
      if ((gl.salles[0].salle == s) && (gl.salles[1].salle == s1))
	{
	  gl.salles[0].salle = s2;
	}
      else if ((gl.salles[1].salle == s) && (gl.salles[0].salle == s1))
	{
	  gl.salles[1].salle = s2;
	}
    }
};

ListIndex
Generator::trouveLien (GeneratorSalle * s1)
{
  ListIndex ret;
  for (int j = 0; j != listLien.size (); j++)
    {
      GeneratorLien & gl = listLien[j];
      if ((gl.salles[0].salle == s1) || (gl.salles[1].salle == s1))
	ret.insert (ret.end (), j);
    };
  return ret;
};

void
repartitionInt (int &i1, int &i2, int hasard, float pourcentage)
{
  float has =
    float (float (hasard) * float (rand ()) / float (RAND_MAX) -
	   hasard / 2.0) / 100.0;
  int valeur = int (i1 * pourcentage + i1 * has);
  if (valeur > i1)
    valeur = i1;
  if (valeur < 0)
    valeur = 0;
  i1 -= valeur;
  i2 += valeur;
  //printf("i1 %i i2 %i valeur %i has %f pourcentage %f hazard %i\n",i1,i2,valeur,has,pourcentage,hasard);
};

void
Generator::repartition (GeneratorSalle * orig, GeneratorSalle * nouv,
			int nouveauNbSalle, int hasard)
{
  if (orig->nbSalle == 0)
    orig->nbSalle = 1;
  float pourcentage = float (nouveauNbSalle) / float (orig->nbSalle);
  orig->nbSalle -= nouveauNbSalle;
  nouv->nbSalle = nouveauNbSalle;
  for (int i = 0; i != MAX_ARME; i++)
    repartitionInt (orig->nbArme[i], nouv->nbArme[i], hasard, pourcentage);
  //exit(0);
  for (int i = 0; i != MAX_VIE; i++)
    repartitionInt (orig->nbVie[i], nouv->nbVie[i], hasard, pourcentage);
  for (int i = 0; i != MAX_MUNITION; i++)
    repartitionInt (orig->nbMunition[i], nouv->nbMunition[i], hasard,
		    pourcentage);
  for (int i = 0; i != MAX_ARMURE; i++)
    repartitionInt (orig->nbArmure[i], nouv->nbArmure[i], hasard,
		    pourcentage);
  for (int i = 0; i != MAX_NAIN; i++)
    repartitionInt (orig->nbNain[i], nouv->nbNain[i], hasard, pourcentage);
  for (int i = 0; i != MAX_SECRET; i++)
    repartitionInt (orig->nbSecret[i], nouv->nbSecret[i], hasard,
		    pourcentage);
  for (int i = 0; i != MAX_TRESOR; i++)
    repartitionInt (orig->nbTresor[i], nouv->nbTresor[i], hasard,
		    pourcentage);
  for (int i = 0; i != MAX_CLE; i++)
    repartitionInt (orig->nbCle[i], nouv->nbCle[i], hasard, pourcentage);
  for (int i = 0; i != MAX_PORTE; i++)
    repartitionInt (orig->nbPorte[i], nouv->nbPorte[i], hasard, pourcentage);

};

void
Generator::lancePlacementSpatial ()
{
  ListBloc Monde;		// Et Dieu crea un monde vide, et il vit que cela etait Bien.
  for (int i = 0; i != listSalle.size (); i++)
    subPlacementSpatial (listSalle[i], Monde);
};

void
Generator::lanceGenerationParPatternEtPlacementObjet ()
{
}

;

/***************************************************************************
 *            MAIN                                                         *
 ***************************************************************************/

void
Generator::outputPattern (ostream & os)
{
  os << "InfoNiveau { NomNiveau=\"Mine\"; }" << endl
    << "Salle { " << endl
    << "        FichierAnimation { " << endl
    << "        Nom=\"mouvement\";" << endl
    << "        Objet {" << endl
    << "          Nom=\"camera\"; " << endl
    << "          Fichier=\"d_commun/d_warrior/corps.3do\";" << endl
    << "        }" << endl
    << "        Animation {" << endl
    << "          Nom=\"mouvement\"; " << endl;
  // On traverse gentiment chaque salle
  float current_time = 0.0;
  char s_temps_fin[100];
  for (int index_checkpoint = 0;
       index_checkpoint < listCheckpoint.size () - 1; index_checkpoint++)
    {
      // origin
      position origin =
	listCheckpoint[index_checkpoint] + position (0, 0, -0.5);
      float angle_origin = listRotation[index_checkpoint];
      // end
      position end =
	listCheckpoint[index_checkpoint + 1] + position (0, 0, -0.5);
      float angle_end = listRotation[index_checkpoint + 1];

      // distance = temps pour se dplacer,  1m/s
      float distance = origin.distanceEuclidienne (end);
      float TEMPS_SALLE = 10.0;
      if (index_checkpoint % 2 == 0)
	TEMPS_SALLE = 0.0;

      float temps_deplacement = current_time + distance;
      float temps_fin = temps_deplacement + TEMPS_SALLE;

      char s_current_time[100];
      char s_temps_deplacement[100];

      sprintf (s_current_time, "%f", current_time);
      sprintf (s_temps_deplacement, "%f", temps_deplacement);
      sprintf (s_temps_fin, "%f", temps_fin);

      char s_angle1[100];
      char s_angle2[100];

      sprintf (s_angle1, "%f", angle_origin);
      sprintf (s_angle2, "%f", angle_end);

      os << "                    Action { Objet=\"camera\"; T1="
	<< string (s_current_time) << "; T2=" << string (s_temps_fin) << ";"
	<< endl << "                             Rotation {" << endl <<
	"                                 AxeX;" << endl <<
	"                                 Angle1 = 1.7;" << endl <<
	"                                 Angle2 = 1.7;" << endl <<
	"                             }" << endl <<
	"                           }" << endl;
      os << "                    Action { Objet=\"camera\"; T1=" <<
	string (s_current_time) << "; T2=" << string (s_temps_fin) << ";" <<
	endl << "                             Rotation {" << endl <<
	"                                 AxeZ;" << endl <<
	"                                 Angle1 = " << s_angle1 << ";" <<
	endl << "                                 Angle2 = " << s_angle2 <<
	";" << endl << "                             }" << endl <<
	"                           }" << endl;

      os << "                    Action { Objet=\"camera\"; T1="
	<< string (s_current_time) << "; T2=" << string (s_temps_deplacement)
	<< ";" << endl << "                             Translation {" << endl
	<< "                                 V1 {" << origin.
	printString () << "}" << endl <<
	"                                 V2 {" << end.
	printString () << "}" << endl << "                             }" <<
	endl << "                           }" << endl;
      if (TEMPS_SALLE != 0.0)
	{
	  os << "                    Action { Objet=\"camera\"; T1="
	    << string (s_temps_deplacement) << "; T2=" << string (s_temps_fin)
	    << ";" << endl << "                             Translation {" <<
	    endl << "                                 V1 {" << end.
	    printString () << "}" << endl <<
	    "                                 V2 {" << end.
	    printString () << "}" << endl << "                             }"
	    << endl << "                           }" << endl;
	}
      current_time = temps_fin;
    }
  os << "        } } " << endl
    <<
    "      FichierSalle { NouvelleAnimation { NomAnimation=\"mouvement\"; AllerRetourInfini; T1=0; T2="
    + string (s_temps_fin) + "; } }" << endl << "      } " << endl;

  os << "Salle { " << endl
    << "      FichierSalle { }" << endl << "      FichierAnimation {" << endl;
  // PRE
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sub = listSalle[i];
      sub->numero = i;
      patternSallePre (sub, os);
    }
  for (int j = 0; j != listLien.size (); j++)
    {
      GeneratorLien *gl = &listLien[j];
      patternLienPre (gl, os);
    };

  // Compute
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sub = listSalle[i];
      sub->numero = i;
      patternSalleCompute (sub, os);
    }
  for (int j = 0; j != listLien.size (); j++)
    {
      GeneratorLien *gl = &listLien[j];
      patternLienCompute (gl, os);
    };

  // Final
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sub = listSalle[i];
      sub->numero = i;
      patternSalleFinal (sub, os);
    }
  for (int j = 0; j != listLien.size (); j++)
    {
      GeneratorLien *gl = &listLien[j];
      patternLienFinal (gl, os);
    };
  os << "     }" << endl << "}" << endl;


  os << "Salle { " << endl
    << "      FichierAnimation {" << endl
    << "        Objet {" << endl
    << "          Nom=\"lobj\"; " << endl
    << "          Source { Lumiere { Ambiante; ZoneLumiereScene; }  } " <<
    endl << "        }" << endl << "        Animation { Nom=\"lum\";" << endl
    << "                    Action { Objet=\"lobj\"; T1=0; T2=100;" << endl <<
    "                             ActionLumiere {" << endl <<
    "                                 Lumiere1{R=1.0;G=1.0;B=1.0; }" << endl
    << "                                 Lumiere2{R=1.0;G=1.0;B=1.0; }" <<
    endl << "                             }" << endl <<
    "                           }" << endl << "        }" << endl << "      }"
    << endl <<
    "     FichierSalle { NouvelleAnimation { NomAnimation=\"lum\"; AllerRetourInfini; T1=0; T2=10; } "
    << endl << "     }" << endl << "   }" << endl << "Jeu {" << endl;
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sub = listSalle[i];
      patternObjetPre (sub, os);
    }
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sub = listSalle[i];
      patternObjetCompute (sub, os);
    }
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sub = listSalle[i];
      patternObjetFinal (sub, os);
    }
  os << "}" << endl;
}

void
outputTable (ostream & os, int table[], int nb)
{
  for (int i = 0; i != nb; i++)
    {
      os << int2string (table[i]) << endl;
    }
}

void
inputTable (istream & os, int table[], int nb)
{
  for (int i = 0; i != nb; i++)
    {
      int v;
      os >> v;
      table[i] = v;
    }
}

void
Generator::save (ostream & os)
{
  os << int2string (listSalle.size ()) << endl;
  for (int i = 0; i != listSalle.size (); i++)
    {
      GeneratorSalle *sub = listSalle[i];
      sub->numero = i;
      os << int2string (sub->type) << endl;
      os << int2string (sub->environnement) << endl;
      os << int2string (sub->nbSalle) << endl;
      outputTable (os, sub->nbArme, MAX_ARME);
      outputTable (os, sub->nbVie, MAX_VIE);
      outputTable (os, sub->nbMunition, MAX_MUNITION);
      outputTable (os, sub->nbArmure, MAX_ARMURE);
      outputTable (os, sub->nbNain, MAX_NAIN);
      outputTable (os, sub->nbSecret, MAX_SECRET);
      outputTable (os, sub->nbTresor, MAX_TRESOR);
      outputTable (os, sub->nbCle, MAX_CLE);
      outputTable (os, sub->nbPorte, MAX_PORTE);

      sub->placementBloc.save (os);
      sub->connectionPossible.save (os);
    }
  os << int2string (listLien.size ()) << endl;
  for (int i = 0; i != listLien.size (); i++)
    {
      GeneratorLien *gl = &listLien[i];
      for (int j = 0; j != 2; j++)
	{
	  os << int2string (gl->salles[j].connecte) << endl;
	  os << int2string (gl->salles[j].cle) << endl;
	  os << int2string (gl->salles[j].porte) << endl;
	  os << int2string (gl->salles[j].salle->numero) << endl;
	}
      gl->connection.save (os);
    };
  printf ("Sauve ... %i %i\n", listSalle.size (), listLien.size ());
};

void
Generator::load (istream & os)
{
  listSalle.clear ();
  listLien.clear ();

  int nbSalle;
  os >> nbSalle;
  for (int i = 0; i != nbSalle; i++)
    {
      int env, nbS, t;
      os >> t;
      os >> env;
      os >> nbS;
      GeneratorSalle *sub =
	new GeneratorSalle (nbS, (TypeSalle) t, (TypeEnvironnement) env);

      listSalle.insert (listSalle.end (), sub);
      inputTable (os, sub->nbArme, MAX_ARME);
      inputTable (os, sub->nbVie, MAX_VIE);
      inputTable (os, sub->nbMunition, MAX_MUNITION);
      inputTable (os, sub->nbArmure, MAX_ARMURE);
      inputTable (os, sub->nbNain, MAX_NAIN);
      inputTable (os, sub->nbSecret, MAX_SECRET);
      inputTable (os, sub->nbTresor, MAX_TRESOR);
      inputTable (os, sub->nbCle, MAX_CLE);
      inputTable (os, sub->nbPorte, MAX_PORTE);

      sub->placementBloc.load (os);
      sub->connectionPossible.load (os);
      sub->numero = i;
    }
  int nbLien;
  os >> nbLien;
  for (int i = 0; i != nbLien; i++)
    {
      BlocLien bl[2];
      for (int j = 0; j != 2; j++)
	{
	  os >> bl[j].connecte;
	  os >> bl[j].cle;
	  os >> bl[j].porte;
	  int numero;
	  os >> numero;
	  bl[j].salle = listSalle[numero];
	}
      GeneratorLien lien (NULL, NULL, 0, 0);
      lien.salles[0] = bl[0];
      lien.salles[1] = bl[1];
      lien.connection.load (os);
      listLien.insert (listLien.end (), lien);
    };
  printf ("Charge ... %i %i\n", listSalle.size (), listLien.size ());
};

int print_usage()
{
  printf("Usage:\n");
  printf("./generator\n");
  printf("    --nbsalle,n=NBSALLE\n");
  printf("    --difficulte,d=[0-100]\n");
  printf("    --complexite,c=[0-100]\n");
  printf("    --repertoire,r=REPERTOIRE DE RESULTAT\n");
  printf("    --charge-fichier,f CHARGER A PARTIR D'UN FICHIER (NE PAS UTILISER)\n");
  printf("    --libre,l GENERER UN PARCOURS LIBRE (COMPLEXE,S'OPPOSE A UN PARCOURS IMPOSE, LINEAIRE) \n");
  printf("    --verbose,v : ce message.\n");
  exit(0);
}

int
main (int argc, char *argv[])
{
  time_t seed;
  time (&seed);
  srandom ((long) seed);
  Generator gene;

  int char_opt;
  int digit_optind = 0;

  int nbSalle=-1;
  int difficulte=-1; 
  int complexite=-1;
  string repertoire="";
  int load_from_file=0;
  int parcours_libre=0;

  while (1) 
    {
      int this_option_optind = optind ? optind : 1;
      int option_index = 0;
      static struct option long_options[] = 
	{
	  {"nbsalle", 1, 0, 'n'},
	  {"difficulte", 1, 0, 'd'},
	  {"complexite", 1, 0, 'c'},
	  {"repertoire", 1, 0, 'r'},
	  {"verbose", 0, 0, 'v'},
	  {"charge-fichier",0 ,0, 'f'},
	  {"libre",0 ,0, 'l'},
	  {0, 0, 0, 0}
	};
      char_opt = getopt_long (argc, argv, "n:d:c:r:vfl",
			      long_options, &option_index);
      if (char_opt == -1)
	break;
      
      switch (char_opt) 
	{
	case 'n':
	  printf ("nombre de salles: %s", optarg);
	  nbSalle=atoi(optarg);
	  printf ("\n");
	  break;

	case 'd':
	  printf ("difficulte: %s", optarg);
	  difficulte=atoi(optarg);
	  printf ("\n");
	  break;

	case 'c':
	  printf ("complexite: %s", optarg);
	  complexite=atoi(optarg);
	  printf ("\n");
	  break;
	  
	case 'r':
	  printf ("repertoire: %s", optarg);
	  repertoire=optarg;
	  system((string("mkdir GENERATION/")+repertoire).c_str());
	  printf ("\n");
	  break;
	  
	case 'v':
	  print_usage();
	  break;	  
	  
	case 'l':
	  parcours_libre=1;
	  break;

	case 'f':
	  load_from_file=1;
	  break;

	default:
	  printf ("?? getopt returned character code 0%o ??\n", char_opt);
	}
      
    }
  if ( (repertoire=="")
       ||
       (difficulte==-1)
       ||
       (complexite==-1)
       ||
       (nbSalle==-1)
       )
    {
      print_usage();
    }
  
  /*
     ListBloc t1(position(-10,-10,-10),position(2,3,4));
     ListBloc t2(position(0,0,0),position(3,3,4));
     printf("<PRE>%i</PRE>",t2.inclus(t1));
     return 1;
   */
  printf ("<TABLE><TR><TD><PRE>\n");

  if (load_from_file == 0)
    {
      if (parcours_libre)
	{
	  gene.lanceGeneration (nbSalle, difficulte, complexite);
	  gene.lancePlacementSpatial ();
	}
      else
	{
	  gene.lanceGenerationContraintPlacementSpatial (nbSalle, difficulte, complexite);
	}
      ofstream output ((string("GENERATION/")+repertoire+string("/savefile.niveau")).c_str(), ofstream::out);
      gene.save (output);
      output.close ();
    }
  else
    {
      printf ("loading from file...\n");

      ifstream input ((string("GENERATION/")+repertoire+"/savefile.niveau").c_str(), ofstream::in);
      gene.load (input);
      input.close ();
    };

  printf ("</PRE></TD><TD>\n");
  //gene.print (0);
  printf ("</TD></TR></TABLE></BODY></HTML>\n");

  ofstream sortie ((string("GENERATION/")+repertoire+"/niveau.niv.script").c_str(), ofstream::out);
  gene.outputPattern (sortie);
  sortie.close ();
};
