/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
                          generator.h  -  description
                             -------------------
    begin                : sam jan 17 2004
    copyright            : (C) 2004 by avila
    email                : avila@deimos
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GENERATOR_FILE
#define GENERATOR_FILE

#include <vector>
#include <ostream>
#include <istream>
using namespace std;

string int2string (int);

class GeneratorSalle;
typedef vector < GeneratorSalle * >ListSalle;

typedef struct position
{
  float coord[3];
  position operator+ (position);
  position operator- (position);
    position (float, float, float);
    position ();
  bool operator== (position pl);
  bool operator>= (position pl);
  bool operator<= (position pl);
  position operator* (float x);
  string printString ();
  string printString (int subdivision, bool x, bool y, bool z);
  position min (position);
  position max (position);
  void print ();
  float distanceEuclidienne (position & p);
  float distanceNorme1 (position & p);
  void save (ostream & os);
  void load (istream & os);
};

typedef vector < position > ListPosition;

typedef struct listBloc:vector < position >
{
  void translation (position);
  bool testConflit (listBloc &);
  bool inclus (listBloc &);
  // Etaler d'1 bloc dans tous les sens.
  listBloc etendUn ();
  position getMin ();
  position getMax ();

    listBloc (position min, position max);
    listBloc ();
  listBloc operator+ (listBloc &);
  void insertPosition (position);
  float distanceMin (listBloc &);
  bool testInclus (position p);
  void save (ostream & os);
  void load (istream & os);
};

typedef listBloc ListBloc;

struct BlocLien
{
  GeneratorSalle *salle;
  bool connecte;		// si on peut aller de celui-ci a l'autre
  int cle;			// la valeur designe la cle a posseder pour ouvrir la porte. -1 si aucune
  int porte;			// la valeur designe le numero de porte ferme (a utiliser avec le no de cle). -1 si aucune
};

typedef struct GeneratorLien
{
  BlocLien salles[2];
    GeneratorLien (GeneratorSalle *, GeneratorSalle *, bool, bool);
  GeneratorSalle *getSuivant (GeneratorSalle *);
  int getSuivant (GeneratorSalle * courant, BlocLien ** courant,
		  BlocLien ** suivant);
  // Les bloc associes a la connection, pour la porte !!!
  listBloc connection;
};

typedef vector < GeneratorLien > ListLien;

// Definition du nombre max d'elements de chaque categorie.
#define MAX_ARME        10
#define MAX_MUNITION    10

// petite, grande
#define MAX_ARMURE      2

// petite, grande
#define MAX_VIE      2

// Vitesse, invuln, pigeon,
#define MAX_OPTION      5

#define MAX_NAIN        1
#define MAX_SECRET      1
#define MAX_TRESOR      4

// Des cles et des portes associees
#define MAX_CLE        50
#define MAX_PORTE       50

typedef enum TypeEnvironnement
{
  ENV_EGOUTS,
  ENV_MINE,
  ENV_BUREAU,
  ENV_CATHEDRALE,
  ENV_LUNAIRE
};

typedef enum TypeSalle
{
  SALLE_COULOIR,
  SALLE_GRANDE,
  SALLE_SECRET,
  SALLE_DEBUT,
  SALLE_FIN,
  SALLE_NORMALE,
  SALLE_EXTERIEUR,
  SALLE_INTERMEDIAIRE
};

typedef vector < int >ListIndex;

// Pour trouver les positions satisfaisant une contrainte
typedef struct PositionPossible
{
  ListBloc position;
  ListIndex insatisfait;	// les liens non-satisfaits (moins il y en a, meilleure est la solution)
  ListIndex satisfait;		// les liens satisfaits
  ListIndex indifferent;	// les liens non-encore definis.
};

typedef vector < PositionPossible > ListPositionPossible;

class EnvData;


class GeneratorSalle
{

public:

  int nbArme[MAX_ARME];
  int nbVie[MAX_VIE];
  int nbMunition[MAX_MUNITION];
  int nbArmure[MAX_ARMURE];
  int nbNain[MAX_NAIN];
  int nbSecret[MAX_SECRET];
  int nbTresor[MAX_TRESOR];
  int nbCle[MAX_CLE];
  int nbPorte[MAX_PORTE];

  TypeSalle type;
  TypeEnvironnement environnement;
  int nbSalle;

  int numero;

  bool placementEffectue;
  listBloc placementBloc, connectionPossible;

    GeneratorSalle (int nbS, TypeSalle, TypeEnvironnement);
   ~GeneratorSalle ();

  // Liste de blocs  laisser libres, issus du placement stupide.
  // Cette liste donne galement la manire de se dpacer pour passer d'une salle  l'autre.
  ListBloc laisseLibre;

  // Pour chaque environnement.
  EnvData *data;

};

class Generator
{
  int nbSalle, difficulte, complexite;

  ListSalle listSalle;
  ListLien listLien;

  // Utilis par la generation contrainte pour avoir une liste de position de passage.
  ListPosition listCheckpoint;
  // Utilis par la generation contrainte pour avoir une liste de rotation de passage cible.
    vector < float >listRotation;

  // fonction pratique : diviser en 2, mettre le type T, le nombre de salle N (pas de lien)
  GeneratorSalle *subGenerationDivise (GeneratorSalle * sub, TypeSalle T,
				       int N);
  // fonction pratique : transofmer un lien 1->2, diviser en 2, mettre le type T, rajouter les liens.
  GeneratorSalle *subGenerationCreerSalle (GeneratorSalle * sub, TypeSalle T);
  // regle 1 : mettre une grande salle
  void subGenerationRegleGrandeSalle (GeneratorSalle * sub);
  // point d'entree
  void subGeneration (GeneratorSalle *);

  int addLien (GeneratorSalle * s1, GeneratorSalle * s2);
  int getLien (GeneratorSalle * s1, GeneratorSalle * s2);
    GeneratorLien & getLien (int index);
  int getIndexSalle (GeneratorSalle * s);

  // Tous les liens s-s1 doivent devenir des s2-s1
  // (ajouter dans s2, enlever dans s, remplacer dans la liste des liens)
  int remplaceLien (GeneratorSalle * s, GeneratorSalle * s1,
		    GeneratorSalle * s2);
  // Tous les liens avec la salle s1
  ListIndex trouveLien (GeneratorSalle * s1);

  void repartition (GeneratorSalle * orig, GeneratorSalle * nouv,
		    int nouveauNbSalle, int hasard);

  /*
     bool Generator::creerMinMaxPorte(
     GeneratorSalle* sub,

     position& infMinPos,position& setInfMin,
     position& infMaxPos,position& setInfMax,

     position& supMinPos,position& setSupMin,
     position& supMaxPos,position& setSupMax,

     listBloc& remplissageCourant,
     ListIndex& insatisfait);
   */
  void subPlacementSpatial (GeneratorSalle *, ListBloc & monde);
  void creerCouloirArtificiel (GeneratorLien &, listBloc & P1, listBloc & P2,
			       ListBloc & monde);
  ListPositionPossible testConfig (ListBloc & salle,
				   ListBloc & connectionPossible,
				   GeneratorSalle * sub, ListBloc & monde);
  void creationSalleCommune (GeneratorSalle * sub, position tailleMin,
			     position tailleMax, int minHauteurSolPorte,
			     int maxHauteurSolPorte, ListBloc & monde);
  void creationSalleCouloir (GeneratorSalle * sub, ListBloc & monde);
  void completeLien (GeneratorSalle * sub, ListBloc & monde,
		     ListBloc & connectionPossible);

  void patternSallePre (GeneratorSalle * sub, ostream & os);
  void patternSalleCompute (GeneratorSalle * sub, ostream & os);
  void patternSalleFinal (GeneratorSalle * sub, ostream & os);

  void patternLienPre (GeneratorLien * lien, ostream & os);
  void patternLienCompute (GeneratorLien * lien, ostream & os);
  void patternLienFinal (GeneratorLien * lien, ostream & os);

  void patternObjetPre (GeneratorSalle * sub, ostream & os);
  void patternObjetCompute (GeneratorSalle * sub, ostream & os);
  void patternObjetFinal (GeneratorSalle * sub, ostream & os);

  void patternMineSalle (GeneratorSalle *, ostream &);
  void patternMineSalleFinal (GeneratorSalle * sub, ostream & os);
  void patternMineSalleExterieur (GeneratorSalle *, ostream &);
  void patternMineCouloir (GeneratorSalle *, ostream &);

public:

  /* Mode de jeu libre */
  void lanceGeneration (int nbS, int dif, int comp);
  void lancePlacementSpatial ();

  /* Mode de jeu contraint */
  void lanceGenerationContraintPlacementSpatial (int nbS, int dif, int comp);

  void lanceGenerationParPatternEtPlacementObjet ();
  void print (int level);

  void save (ostream & os);
  void load (istream & os);

  void outputPattern (ostream & os);
    Generator ();
   ~Generator ();
};

#endif
