/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: RWvsNDJ_game.cpp,v 1.6 2006/12/28 10:50:43 atila Exp $

#include "RWvsNDJ_game.h"
#include "SDL/SDL.h"
#include "banque.h"
#include "def_texture.h"
#include "controle.h"
#include "animation.h"
#include "jeu.h"
#include "affiche_texte.h"
#include "jeu_joueur.h"
#include "connexion.h"
#include "snd_mixer.h"
#include "option.h"
#include "systeme.h"
#include "dico.h"

int *check_int = NULL;
char *check_string = NULL;
bool *check_bool = NULL;

void
set_check_data (bool * b, int *i, char *s)
{
  if (i)
    check_int = i;
  if (s)
    check_string = s;
  if (b)
    check_bool = b;
};

void
print_checking_up ()
{
  printf ("Checking UP");
  if (check_int)
    printf ("CHECKING UP INT.... %i\n", *check_int);
  if (check_string)
    printf ("CHECKING UP STRING.... %s\n", check_string);
  if (check_bool)
    printf ("CHECKING UP BOOL.... %i\n", check_bool);
};

void init_video (bool, int);

double
gettime ()
{
  return ((float) SDL_GetTicks ()) / 1000.0;
};

void
RW_GAME::gestion_temps ()
{
  float lt = blocinfo_commun.T;
  float t = gettime ();
  float diff = t - lt;

  blocinfo_commun.T = t;
  blocinfo_commun.LT = lt;
  blocinfo_commun.diff = diff;
  blocinfo_commun.real_T = t;
  blocinfo_commun.real_LT = lt;
  blocinfo_commun.real_diff = diff;

  blocinfo_client.T = t;
  blocinfo_client.LT = lt;
  blocinfo_client.diff = diff;
  blocinfo_client.real_T = t;
  blocinfo_client.real_LT = lt;
  blocinfo_client.real_diff = diff;

  blocinfo_serveur.T = t;
  blocinfo_serveur.LT = lt;
  blocinfo_serveur.diff = diff;
  blocinfo_serveur.real_T = t;
  blocinfo_serveur.real_LT = lt;
  blocinfo_serveur.real_diff = diff;

};

RW_GAME::RW_GAME (bool server)
{
  double lt_serveur = 0;
  double diff_serveur = 0;

  FPS = 0;
  lFPS = 0;
  tFPS = 0;

  blocinfo_commun.serveur = server;
  blocinfo_commun.client = !server;

  printf ("Debut d'init commun...\n");

  blocinfo_commun.lpreseau = NULL;
  blocinfo_commun.lpjeu = NULL;
  blocinfo_commun.lpniveau = NULL;
  blocinfo_commun.lpbanque_texture = new BANQUE_TEXTURE ();
  printf ("banque_texture ... OK\n");
  blocinfo_commun.lpbdd = new BDD ();
  LISTE_MOT fichier_bdd = get_liste_bdd ();
  for (int i = 0; i != fichier_bdd.size (); i++)
    {
      printf ("Loading data file : %s\n", fichier_bdd[i].info);
      blocinfo_commun.lpbdd->ajoute_bdd (&fichier_bdd[i]);
    }
  if (fichier_bdd.size () == 0)
    {
      printf ("No data file found (*.zip) !!! CHECK YOUR INSTALLATION !!\n");
      exit (1);
    }
  printf ("bdd ... OK\n");
  blocinfo_commun.lpbanque_animation = new BANQUE_ANIMATION ();
  printf ("banque animation ... OK\n");
  blocinfo_commun.lpbanque_animation_texture =
    new BANQUE_ANIMATION_TEXTURE ();
  printf ("banque animation texture... OK\n");
  blocinfo_commun.lpbanque_objet = new BANQUE_OBJET ();
  printf ("banque objet ... OK\n");

  blocinfo_commun.lpaffiche_texte =
    new AFFICHE_TEXTE (blocinfo_commun.lpbanque_texture,
		       blocinfo_commun.lpbdd);
  printf ("affiche texte ... OK\n");

  blocinfo_commun.lpcontrole = new CONTROLE ();
  printf ("controle ... OK\n");
  blocinfo_commun.lpmoteur = new MOTEUR (&blocinfo_commun);
  printf ("moteur 3D ... OK\n");
  blocinfo_commun.lpdictionnaire = new DICO (blocinfo_commun.lpbdd);
  printf ("Dictionnaire ... OK\n");
  blocinfo_commun.lpmixer = new MIXER (blocinfo_commun.lpbdd);
  printf ("mixer ... OK\n");
  if (!server)
    {
      if (blocinfo_commun.lpaffiche_texte)
	blocinfo_commun.lpaffiche_texte->init_client (blocinfo_commun.
						      lpbanque_texture,
						      blocinfo_commun.lpbdd,
						      true);
    }
  MOT nom_fichier_option ("config.cfg");
  blocinfo_commun.lpfichier_option =
    new FICHIER_OPTION (get_nom_fichier_config ());
  blocinfo_commun.lpfichier_option->read_option ();
  blocinfo_commun.lpetat_texture = new ETAT_TEXTURE ();
  printf ("etat_texture ... OK\n");
  blocinfo_commun.T = 0;
  blocinfo_commun.LT = 0;
  blocinfo_commun.diff = 0;

  blocinfo_commun.real_T = 0;
  blocinfo_commun.real_LT = 0;
  blocinfo_commun.real_diff = 0;

  blocinfo_commun.blocoption.typeVue = TypeVueInterne;
  blocinfo_commun.blocoption.vueDistance = 2.0;

  blocinfo_commun.blocecran.lumiere = TypeLumiereComplete;
  blocinfo_commun.blocecran.resolution = RESOLUTION_800_600;
  blocinfo_commun.blocecran.fullscreen = true;

  printf ("Fin d'init commun...\n");

  gestion_temps ();

  son_mort = 0;
};

void
RW_GAME::fill_NULL_blocinfo (BLOCINFO & blocinfo, bool server, bool client)
{
  blocinfo.serveur = server;
  blocinfo.client = client;
  blocinfo.LT = 0;
  blocinfo.T = 0;
  blocinfo.diff = 0;
  blocinfo.real_LT = 0;
  blocinfo.real_T = 0;
  blocinfo.real_diff = 0;
  blocinfo.lpbanque_texture = NULL;
  blocinfo.lpbdd = NULL;
  blocinfo.lpjeu = NULL;
  blocinfo.lpniveau = NULL;
  blocinfo.lpbanque_animation = NULL;
  blocinfo.lpbanque_objet = NULL;
  blocinfo.lpbanque_animation_texture = NULL;
  blocinfo.lpreseau = NULL;
  blocinfo.lpaffiche_texte = NULL;
  blocinfo.lpcontrole = NULL;
  blocinfo.lpmoteur = NULL;
  blocinfo.blocoption = blocinfo_commun.blocoption;
  blocinfo.lpdictionnaire = blocinfo_commun.lpdictionnaire;
  blocinfo.lpmixer = NULL;
  blocinfo.lpfichier_option = NULL;
  blocinfo.lpetat_texture = NULL;
  blocinfo.texture_halo = 0;
};


void
RW_GAME::gestion_blocinfo (BLOCINFO & blocinfo)
{
  double diff = blocinfo_commun.diff;
  double t = blocinfo_commun.T;
  double lt = blocinfo_commun.LT;

  blocinfo.T = t;
  blocinfo.LT = lt;
  blocinfo.diff = diff;

  blocinfo.real_T = t;
  blocinfo.real_LT = lt;
  blocinfo.real_diff = diff;

  RWVECTOR position;
  JEU_JOUEUR *joueur = blocinfo.lpjeu->get_joueur ();
  if (joueur)
    position = joueur->get_position ();

  if (blocinfo.lpaffiche_texte)
    blocinfo.lpaffiche_texte->gestion_globale (&blocinfo);
//  printf("Gestion texte \n");
  if (blocinfo.lpreseau)
    blocinfo.lpreseau->gestion_globale (&blocinfo);
//  printf("Gestion reseau\n");
  if (blocinfo.lpniveau)
    blocinfo.lpniveau->gestion_globale (&blocinfo);
//  printf("Gestion niveau\n");

  if (blocinfo.lpjeu)
    blocinfo.lpjeu->gestion_globale (&blocinfo);
//  printf("Gestion jeu\n");

  if (blocinfo.lpbanque_animation_texture)
    blocinfo.lpbanque_animation_texture->gestion_globale (&blocinfo);
//  printf("Gestion banque animation\n");
  if (blocinfo.lpmixer)
    blocinfo.lpmixer->gestion (position);

}


void
RW_GAME::affichage_client (BLOCINFO & blocinfo)
{

  float rotx = 0;
  float roty = 0;
  RWVECTOR decal, decal_rot, decal_effet;
  RWVECTOR pos;
  int vie = 0;
  int armure = 0;
  int points = 0;
  int frags = 0;
  int arme_active = 0;
  float effet_mort = 0;
  float effet_degat = 0;
  float effet_vie = 0;
  float effet_option = 0;
  DONNEE_ARME tab[NB_ARME];
  float option[MAX_OPTION];
  for (int i = 0; i != MAX_OPTION; i++)
    option[i] = 0;
  bool gagne = false;

  if (blocinfo.lpjeu)
    {
      RWVECTOR position;
      JEU_JOUEUR *joueur = blocinfo.lpjeu->get_joueur ();
      if (joueur)
	{
	  position = joueur->get_position ();
	  rotx = joueur->get_vuex_joueur ();
	  roty = joueur->get_vuez_joueur ();
	  joueur->get_decal (decal, decal_rot, decal_effet);
	  pos = joueur->get_position ();
	  arme_active = joueur->get_munitions (tab);
	  vie = joueur->get_vie ();
	  armure = joueur->get_armure ();
	  points = joueur->get_points ();
	  frags = joueur->get_frags ();
	  effet_mort = joueur->get_effet_mort ();
	  gagne = joueur->get_gagne ();
	  joueur->get_option (option);
	  effet_degat = joueur->get_effet_degat ();
	  effet_vie = joueur->get_effet_vie ();
	  effet_option = joueur->get_effet_option ();
	  pos = pos + RWVECTOR (0, 0, -effet_mort * 1.0) + decal_effet;
	  if ((effet_mort == 0.5) && (son_mort == 0) && (!gagne))
	    {
	      MOT fin ("d_commun/d_ecran/gameover.ogg");
	      blocinfo.lpmixer->lance_son (fin, -1);
	      son_mort = 1;
	    }
	}
    }
  if (blocinfo.lpmoteur)
    {
      blocinfo.lpmoteur->effet (&blocinfo, gagne, effet_vie, effet_option,
				effet_degat, effet_mort);
      blocinfo.lpmoteur->swap_end ();
      blocinfo.lpmoteur->init_affichage (&blocinfo,
					 pos,
					 decal, decal_rot, roty, rotx, 0.7);

    }
  if (!jeu_engage_client)
    {
      MOT nom_map;
      int max_joueur;
      if (blocinfo.lpreseau)
	if (blocinfo.lpreseau->get_info_jeu (max_joueur, nom_map))
	  {
	    jeu_engage_client = true;
	    printf ("Chargement de la carte : %s \n", nom_map.info);
	    if (blocinfo.lpniveau)
	      blocinfo.lpniveau->charge_niveau (&nom_map, &blocinfo);
	  }
    }
  else
    {
      if (blocinfo.lpniveau)
	blocinfo.lpniveau->affichage (&blocinfo);
      if (blocinfo.lpjeu)
	blocinfo.lpjeu->affichage (&blocinfo);
    }
  if (blocinfo.lpmoteur)
    blocinfo.lpmoteur->fin_affiche (&blocinfo);
  FPS++;
  if (tFPS != (int) blocinfo.T)
    {
      tFPS = (int) blocinfo.T;
      lFPS = FPS;
      FPS = 0;
    };

  if (!((blocinfo.lpaffiche_texte) &&
	(blocinfo.lpcontrole) &&
	(blocinfo.lpjeu) && (blocinfo.lpmoteur) && (blocinfo.lpniveau)))
    {
      //return;
    }

  MOT m ("CONNEXION AU SERVEUR EN COURS...");
  char te[100];
  sprintf (te, "%i FPS", lFPS);
  MOT m2 (te);

  sprintf (te, "VIE %i ARMURE %i POINTS %i FRAGS %i", vie, armure, points,
	   frags);
  MOT m3 (te);

  if (blocinfo.lpaffiche_texte)
    {
      if (!jeu_engage_client)
	{
	  blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, texte, -23, 10,
						   30, 0);
	  blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m, -20, 0, 30,
						   0);
	}
      blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m2, 15, 14, 30, 0);
      blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m3, -20, 15, 30, 0);
    }
  if (option[OPTION_PIGEON] != 0)
    {
      int opt = (int) option[OPTION_PIGEON];
      char buff[1000];
      sprintf (buff, "PIGEON %i", opt);
      MOT m = MOT (buff);
      blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m, -23, -7, 30, 0);
    };
  if (option[OPTION_INVULNERABILITE] != 0)
    {
      int opt = (int) option[OPTION_INVULNERABILITE];
      char buff[1000];
      sprintf (buff, "INVULNERABILITE %i", opt);
      MOT m = MOT (buff);
      blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m, -23, -6, 30, 0);
    };
  if (option[OPTION_ANNIHILATRON] != 0)
    {
      int opt = (int) option[OPTION_ANNIHILATRON];
      char buff[1000];
      sprintf (buff, "ANNIHILATRON %i", opt);
      MOT m = MOT (buff);
      blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m, -23, -5, 30, 0);
    };
  if (option[OPTION_RAPIDITE] != 0)
    {
      int opt = (int) option[OPTION_RAPIDITE];
      char buff[1000];
      sprintf (buff, "RAPIDITE %i", opt);
      MOT m = MOT (buff);
      blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m, -23, -4, 30, 0);
    };
  if (option[OPTION_SUPERPUISSANCE] != 0)
    {
      int opt = (int) option[OPTION_SUPERPUISSANCE];
      char buff[1000];
      sprintf (buff, "SUPERPUISSANCE %i", opt);
      MOT m = MOT (buff);
      blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m, -23, -3, 30, 0);
    };
  if (blocinfo.mode_jeu == MODE_JEU_ARCADE)
    {
      MOT cible ("d_commun/d_ecran/crossair.pcx");
      blocinfo.lpmoteur->affiche_cible (cible, &blocinfo);
      if (arme_active != -1)
	for (int i = 0; i != NB_ARME; i++)
	  {
	    MOT ar;
	    int munition = -1;
	    switch (i)
	      {
	      case 0:
		ar = MOT ("POING");
		break;
	      case 1:
		ar = MOT ("MITRAILLE");
		munition = tab[0].munition;
		break;
	      case 2:
		ar = MOT ("TROMBLON");
		munition = tab[0].munition;
		break;
	      case 3:
		ar = MOT ("TRIPLE MITRAILLE");
		munition = tab[0].munition;
		break;
	      case 4:
		ar = MOT ("LANCE-MINE");
		munition = tab[1].munition;
		break;
	      case 5:
		ar = MOT ("LANCE-ROQUETTE");
		munition = tab[2].munition;
		break;
	      case 6:
		ar = MOT ("FUSIL PLASMA");
		munition = tab[3].munition;
		break;
	      };
	    if (munition != -1)
	      sprintf (te, "%i %s", munition, ar.info);
	    else
	      sprintf (te, "%s", ar.info);
	    float R = 1.0, G = 1.0, B = 1.0, alpha = 0;
	    if (i == arme_active)
	      {
		R = 1.0;
		B = 1.0;
		alpha = 0.0;
	      }
	    else
	      {
		alpha = 0.5;
		if (tab[i].possede)
		  {
		    R = 1.0;
		  };
	      }
	    m = MOT (te);
	    blocinfo.lpaffiche_texte->affiche_texte (&blocinfo, m, -25, i + 10,
						     35, R, G, B, alpha);
	  }
      blocinfo.lpaffiche_texte->affiche_tout (&blocinfo);
    }

  blocinfo.lpcontrole->gestion_globale ();
  // Detecter le passage en mode texte...
  if (mode_texte)
    {
      char c = blocinfo.lpcontrole->lettre_appuyee ();
      if (c)
	texte.ajoute_caractere (c);
      if (blocinfo.lpcontrole->entree ())
	{
	  mode_texte = false;
	  blocinfo.lpjeu->texte (texte, blocinfo_commun.T);
	  texte = MOT ("");
	}
      if (blocinfo.lpcontrole->efface_appuyee ())
	texte.dec ();
    }
  else
    {
      if (blocinfo.lpcontrole->vue ())
	{
	  switch (blocinfo.blocoption.typeVue)
	    {
	    case TypeVueExterneDos:
	      blocinfo.blocoption.typeVue = TypeVueExterneFixe;
	      break;
	    case TypeVueExterneFixe:
	      blocinfo.blocoption.typeVue = TypeVueInterne;
	      break;
	    case TypeVueInterne:
	      blocinfo.blocoption.typeVue = TypeVueExterneDos;
	      break;
	    }
	};
      if (blocinfo.lpcontrole->texte ())
	{
	  mode_texte = true;
	};
      if (blocinfo.lpcontrole->fullscreen ())
	{
	  blocinfo.blocecran.fullscreen = !blocinfo.blocecran.fullscreen;
	  init_video (!blocinfo.blocecran.fullscreen,
		      blocinfo.blocecran.resolution);
	};
    }
  glFlush ();
};

void
init_blocinfo_classique (BLOCINFO & blocinfo, BLOCINFO & blocinfo_commun)
{
  blocinfo.lpreseau = new RESEAU ();
  blocinfo.lpjeu = new JEU ();
  blocinfo.lpniveau = new NIVEAU ();
  blocinfo.lpbanque_animation = new BANQUE_ANIMATION ();
  blocinfo.lpbanque_animation_texture = new BANQUE_ANIMATION_TEXTURE ();
  blocinfo.lpbanque_objet = new BANQUE_OBJET ();
  blocinfo.lpbanque_texture = new BANQUE_TEXTURE ();
  blocinfo.lpbdd = blocinfo_commun.lpbdd;
  blocinfo.lpmoteur = blocinfo_commun.lpmoteur;
  blocinfo.lpmixer = blocinfo_commun.lpmixer;
  blocinfo.lpetat_texture = blocinfo_commun.lpetat_texture;
  blocinfo.lpcontrole = blocinfo_commun.lpcontrole;
  blocinfo.blocecran = blocinfo_commun.blocecran;
  blocinfo.blocoption = blocinfo_commun.blocoption;
  blocinfo.mode_mouvement = blocinfo_commun.mode_mouvement;
  blocinfo.mode_jeu = blocinfo_commun.mode_jeu;
  printf ("Fin d'init classique\n");
};

void
RW_GAME::debut_game (GAME_TYPE gt, MOT srv, MOT map, MOT nom)
{
  game_type = gt;
  switch (gt)
    {
    case CLIENT_SERVER_MELTED:
      fill_NULL_blocinfo (blocinfo_serveur, false, false);
      fill_NULL_blocinfo (blocinfo_client, true, true);
      init_blocinfo_classique (blocinfo_client, blocinfo_commun);
      blocinfo_client.lpaffiche_texte =
	new AFFICHE_TEXTE (blocinfo_client.lpbanque_texture,
			   blocinfo_client.lpbdd);
      blocinfo_client.lpaffiche_texte->init_client (blocinfo_client.
						    lpbanque_texture,
						    blocinfo_client.lpbdd,
						    true);

      blocinfo_client.lpniveau->charge_niveau (&map, &blocinfo_client);
      blocinfo_client.lpreseau->set_info_jeu (-1, &map);
      blocinfo_client.lpreseau->init_serveur (&blocinfo_client);
      mode_texte = false;
      jeu_engage_client = true;
      printf ("Fin d'init jeu...\n");
      gestion_temps ();
      gestion_temps ();
      break;
    case STANDALONE_SERVER:
    case STANDALONE_CLIENT:
    case CLIENT_SERVER_FULL:
      // Je rappelle que CLIENT_SERVER_FULL c'est du FULL parallele,
      // ce qui explique les blocs identiques.
      fill_NULL_blocinfo (blocinfo_serveur, true, false);
      fill_NULL_blocinfo (blocinfo_client, false, true);
      init_blocinfo_classique (blocinfo_serveur, blocinfo_commun);
      if (gt != STANDALONE_CLIENT)
	{

	  blocinfo_serveur.lpaffiche_texte =
	    new AFFICHE_TEXTE (blocinfo_serveur.lpbanque_texture,
			       blocinfo_serveur.lpbdd);
	  blocinfo_serveur.blocoption = blocinfo_commun.blocoption;
	  blocinfo_serveur.lpniveau->charge_niveau (&map, &blocinfo_serveur);
	  blocinfo_serveur.lpreseau->set_info_jeu (-1, &map);
	  blocinfo_serveur.lpreseau->init_serveur (&blocinfo_serveur);
	  gestion_temps ();
	  gestion_temps ();
	};
      if (gt != STANDALONE_SERVER)
	{
	  blocinfo_client.lpreseau = new RESEAU ();
	  blocinfo_client.lpjeu = new JEU ();
	  blocinfo_client.lpniveau = new NIVEAU ();
	  blocinfo_client.lpbanque_animation = new BANQUE_ANIMATION ();
	  blocinfo_client.lpbanque_animation_texture =
	    new BANQUE_ANIMATION_TEXTURE ();
	  blocinfo_client.lpbanque_objet = new BANQUE_OBJET ();
	  blocinfo_client.lpbanque_texture = new BANQUE_TEXTURE ();
	  blocinfo_client.lpbdd = blocinfo_commun.lpbdd;
	  blocinfo_client.lpmoteur = blocinfo_commun.lpmoteur;
	  blocinfo_client.lpmixer = blocinfo_commun.lpmixer;
	  blocinfo_client.lpetat_texture = blocinfo_commun.lpetat_texture;
	  blocinfo_client.lpcontrole = blocinfo_commun.lpcontrole;
	  blocinfo_client.lpaffiche_texte =
	    new AFFICHE_TEXTE (blocinfo_client.lpbanque_texture,
			       blocinfo_client.lpbdd);
	  blocinfo_client.lpaffiche_texte->init_client (blocinfo_client.
							lpbanque_texture,
							blocinfo_client.
							lpbdd, true);
	  blocinfo_client.lpreseau->connecte_client (&blocinfo_client, srv);
	};
      mode_texte = false;
      jeu_engage_client = false;
      printf ("Fin d'init jeu...\n");
      gestion_temps ();
      gestion_temps ();
      break;
    };
}


int
RW_GAME::gestion_jeu ()
{
  SDL_Delay (10);
  if (game_type == CLIENT_SERVER_MELTED)
    {
      affichage_client (blocinfo_client);
      if (blocinfo_client.lpjeu)
	{	  
	  JEU_JOUEUR *joueur = blocinfo_client.lpjeu->get_joueur ();
	  if (joueur)
	    {
	      if (joueur->get_fini())
		{
		return true;
		};
	    };
	}
      gestion_temps ();
      gestion_blocinfo (blocinfo_client);
      return false;
    }
  else
    {
      if (game_type != STANDALONE_SERVER)
	affichage_client (blocinfo_client);
      gestion_temps ();
      gestion_blocinfo (blocinfo_serveur);
      gestion_blocinfo (blocinfo_client);
      return false;
    }
};

#define DELETE_AND_SET_NULL(x) if ( x ) { delete x ; x = NULL; }

void nettoie_bloc(BLOCINFO blocinfo)
{
  
}

void
RW_GAME::fin_jeu ()
{
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_texture)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbdd)
  DELETE_AND_SET_NULL(blocinfo_commun.lpjeu)
  DELETE_AND_SET_NULL(blocinfo_commun.lpniveau)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_animation)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_objet)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_animation_texture)
  DELETE_AND_SET_NULL(blocinfo_commun.lpreseau)
  DELETE_AND_SET_NULL(blocinfo_commun.lpaffiche_texte)
  DELETE_AND_SET_NULL(blocinfo_commun.lpcontrole)
  DELETE_AND_SET_NULL(blocinfo_commun.lpmoteur)
  DELETE_AND_SET_NULL(blocinfo_commun.lpdictionnaire)
  DELETE_AND_SET_NULL(blocinfo_commun.lpmixer)
  DELETE_AND_SET_NULL(blocinfo_commun.lpfichier_option)
  DELETE_AND_SET_NULL(blocinfo_commun.lpetat_texture)
};

void
RW_GAME::set_info_joueur (MOT nom, MOT skin)
{
};

RW_GAME::~RW_GAME ()
{
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_texture)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbdd)
  DELETE_AND_SET_NULL(blocinfo_commun.lpjeu)
  DELETE_AND_SET_NULL(blocinfo_commun.lpniveau)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_animation)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_objet)
  DELETE_AND_SET_NULL(blocinfo_commun.lpbanque_animation_texture)
  DELETE_AND_SET_NULL(blocinfo_commun.lpreseau)
  DELETE_AND_SET_NULL(blocinfo_commun.lpaffiche_texte)
  DELETE_AND_SET_NULL(blocinfo_commun.lpcontrole)
  DELETE_AND_SET_NULL(blocinfo_commun.lpmoteur)
  DELETE_AND_SET_NULL(blocinfo_commun.lpdictionnaire)
  DELETE_AND_SET_NULL(blocinfo_commun.lpmixer)
  DELETE_AND_SET_NULL(blocinfo_commun.lpfichier_option)
  DELETE_AND_SET_NULL(blocinfo_commun.lpetat_texture)
};
