/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: TCP.cpp,v 1.4 2006/12/21 18:18:14 atila Exp $

#include "TCP.h"

#ifdef linux

int
  TCP::maxfd;

TCP::TCP ():mode (INACTIF)
{
  buffer = NULL;
};

TCP::~TCP ()
{
  close (sock);
};

int
TCP::gestion_globale ()
{
  int nfound;
  fd_set mask;
  FD_ZERO (&mask);
  FD_SET (sock, &mask);
  int bytesread;
  struct timeval timeout = { 0, 0 };	// 0 sec de time-out...
//printf("ETAT SOCK %i %i LECTURE=%i ECRITURE=%i INACTIF=%i ECHEC=%i\n", 
//              sock, mode,LECTURE,ECRITURE,INACTIF,ECHEC);
  switch (mode)
    {
    case ECHEC:
      return -1;
      break;
    case FIN_IO:
      return 0;
      break;
    case LECTURE_PASSIVE:
      // S'activer en lecture, s'il y a qqch
      // qui arrive
      {
	nfound = select (FD_SETSIZE,
			 &mask, (fd_set *) 0, (fd_set *) 0, &timeout);
	if ((FD_ISSET (sock, &mask)) && (nfound))
	  // Houston, on a qqch !!!
	  {
	    mode = INACTIF;
	    mise_en_lecture ();
	  };
      };
      break;
    case ECRITURE:
      {
	void *buff;
	unsigned int taille_a_ecrire;
	unsigned int taille_ecrite;
	bool dans_entete;
	if (position_buffer < sizeof (entete_TCP))
	  {
	    dans_entete = true;
	    buff = (char *) &entete + position_buffer;
	    taille_a_ecrire = sizeof (entete_TCP) - position_buffer;
	  }
	else
	  {
	    dans_entete = false;
	    buff = (char *) buffer + position_buffer - sizeof (entete_TCP);
	    taille_a_ecrire =
	      taille_buffer - position_buffer + sizeof (entete_TCP);
	    if (taille_a_ecrire == 0)
	      {
		mode = FIN_IO;
		return 0;
	      };
	  };
	if (taille_a_ecrire > 1024)
	  taille_a_ecrire = 1024;
	// On tente l'écriture d'un bloc !
	taille_ecrite = write (sock, buff, taille_a_ecrire);
	//printf("ECRIT taille_a_ecrire %i taille_ecrite %i numsocket %i\n",taille_a_ecrire,taille_ecrite,sock);
	if (taille_ecrite == -1)
	  {
	    mode = ECHEC;
	    if (buffer)
	      free (buffer);
	    buffer = NULL;
	    return 0;
	  };
	position_buffer = position_buffer + taille_ecrite;
	if (position_buffer == taille_buffer + sizeof (entete_TCP))
	  {
	    mode = FIN_IO;
	    if (buffer)
	      free (buffer);
	    buffer = NULL;
	    return 0;
	  };
      };
      break;
    case LECTURE:
      {
	// On tente de lire qqch !!!
	// Taille à lire et ou le ranger.
	void *buff;
	int taille_a_lire;
	int taille_lue;
	bool dans_entete;
	if (position_buffer < sizeof (entete_TCP))
	  {
	    dans_entete = true;
	    buff = (char *) &entete + position_buffer;
	    taille_a_lire = sizeof (entete_TCP) - position_buffer;
	  }
	else
	  {
	    dans_entete = false;
	    buff = (char *) buffer + position_buffer - sizeof (entete_TCP);
	    taille_a_lire =
	      taille_buffer - position_buffer + sizeof (entete_TCP);
	  };
	nfound = select (FD_SETSIZE,
			 &mask, (fd_set *) 0, (fd_set *) 0, &timeout);
	if ((FD_ISSET (sock, &mask)) && (nfound))
	  // On a qqch !!! on récupère...
	  {
	    time (&dernier_acces_ok);
	    taille_lue = read (sock, (char *) buff, taille_a_lire);
	    if (taille_lue == -1)
	      {
		// Echec GRAVE. 

		mode = ECHEC;
		//printf("Echec GRAVE lecture Ici, %i %i\n",taille_a_lire,taille_lue);
		if (buffer)
		  free (buffer);
		buffer = NULL;
		return -1;
	      }
	    else
	      {
		position_buffer = position_buffer + taille_lue;
		if (taille_lue == 0)
		  {
		    mode = ECHEC;
		    if (buffer)
		      free (buffer);
		    buffer = NULL;
		    return -1;
		  }

		if ((dans_entete) && (position_buffer == sizeof (entete_TCP)))
		  // On alloue et on passe à la suite.
		  {
		    //printf("CREATION\n");
		    if (entete.taille_info != 0)
		      {
			taille_buffer = entete.taille_info;
			buffer = malloc (entete.taille_info);
		      }
		    else	// pas besoin d'allouer.
		      {
			buffer = NULL;
			// Test Paquet d'attente ...
			if (entete.type_requete == 0)
			  {
			    // Yes... All right. Wait NE><T ..
			    mise_en_lecture ();
			  }
			else
			  {
			    // Non. Info disponible.
			    mode = FIN_IO;
			    taille_buffer = 0;
			    buffer = NULL;
			  };
		      };
		  }
		if ((!dans_entete)
		    && (position_buffer ==
			taille_buffer + sizeof (entete_TCP)))
		  {
		    mode = FIN_IO;
		  };
	      }
	  };
      };
      break;
    };
};

MODE_SOCKET
TCP::etat ()
{
  return mode;
};

int
TCP::recup_info (unsigned char *msg_type,
		 void **buffer_recup, int &taille_info)
{
  if (mode != FIN_IO)
    return -1;
  *msg_type = entete.type_requete;
  *buffer_recup = buffer;
  taille_info = taille_buffer;
  buffer = NULL;
  mode = INACTIF;
  return 0;
};

int
TCP::mise_en_lecture ()
{
  if (mode == ECHEC)
    return -1;
  if (buffer)
    free (buffer);
  buffer = NULL;
  taille_buffer = 0;
  mode = LECTURE;
  position_buffer = 0;
  time (&dernier_acces_ok);
};

int
TCP::mise_en_lecture_passive ()
{
  if (mode == ECHEC)
    return -1;
  buffer = NULL;
  taille_buffer = 0;
  mode = LECTURE_PASSIVE;
  position_buffer = 0;
  time (&dernier_acces_ok);
};

int
TCP::ecriture (unsigned char msg_type,
	       void *buffer_ecr, unsigned int taille_buffer_ecr)
{
  if ((mode != INACTIF) && (mode != FIN_IO))
    return -1;
  buffer = buffer_ecr;
  taille_buffer = taille_buffer_ecr;
  mode = ECRITURE;
  position_buffer = 0;
  memcpy (entete.signature, SIGNATURE_REBEL_WARRIOR, 10);
  entete.type_requete = msg_type;
  entete.taille_info = taille_buffer_ecr;
  time (&dernier_acces_ok);
  return 0;
};

int
TCP::ecoute_connection (TCP * c2)
{
  struct sockaddr_in remote;
  int addrlen;

  struct timeval timeout = { 0, 0 };
  fd_set mask;
  FD_ZERO (&mask);
  FD_SET (sock, &mask);
  int nfound = 0;
  nfound = select (maxfd + 1, &mask, (fd_set *) 0, (fd_set *) 0, &timeout);
  if (FD_ISSET (sock, &mask))
    {
      char str[500];
      addrlen = sizeof (remote);
      c2->sock = accept (sock, (struct sockaddr *) &remote,
			 (socklen_t *) & addrlen);
      if (c2->sock < 0)
	return -1;
      sprintf (str, "CONNECTION DE L'HOTE %s, SUR PORT %d, SOCKET %d\n",
	       inet_ntoa (remote.sin_addr),
	       ntohs (remote.sin_port), c2->sock);
      c2->nom_distant = MOT (inet_ntoa (remote.sin_addr));
      c2->port_distant = port_distant;
      printf (str);
      if (c2->sock > maxfd)
	maxfd = c2->sock;
      return 1;
    }
  return 0;
};


int
TCP::connection_serveur (char *machine, int port)
{
  struct hostent *hostp;
  struct sockaddr_in server, client;
  struct servent s;
  if ((sock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
    {
      printf ("MODULE COMM: ERREUR DE CREATION DE SOCKET.");
      return -1;
    };
  if (((hostp = gethostbyname (machine)) == 0))
    {
      printf ("MODULE COMM : ERREUR A LA RESOLUTION DU NOM");
      return -2;
    };
  memset ((void *) &server, 0, sizeof server);
  if (port == 0)
    port = PORT_STANDARD;
  s.s_port = htons (ushort (port));
  server.sin_family = AF_INET;
  memcpy ((void *) &server.sin_addr, hostp->h_addr, hostp->h_length);
  server.sin_port = s.s_port;
  int erreur = 0;
  if ((erreur =
       connect (sock, (struct sockaddr *) &server, sizeof server)) < 0)
    {
      printf ("MODULE COMM : CONNECTION IMPOSSIBLE AU SERVEUR");
      return -3;
    };
  return 0;
};

int
TCP::ouverture_socket_ecoute (int port)
{
  port_distant = port;
  struct servent *servp;
  static struct servent s;
  struct sockaddr_in server;
  if ((sock = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
    {
      printf ("MODULE COMM: ERREUR DE CREATION DE SOCKET.");
      return -3;
    };
  if (port == 0)
    port = PORT_STANDARD;
  servp = &s;
  s.s_port = htons (ushort (port));
  memset ((void *) &server, 0, sizeof server);
  server.sin_family = AF_INET;
  server.sin_addr.s_addr = INADDR_ANY;
  server.sin_port = servp->s_port;
  if (bind (sock, (struct sockaddr *) &server, sizeof server) < 0)
    {
      printf ("MODULE COMM: ERREUR DE NOMMAGE DE SOCKET !!!");
      return -1;
    };
  if ((listen (sock, SOMAXCONN)) < 0)
    {
      printf ("MODULE COMM: ERREUR DE MISE EN ECOUTE DE LA SOCKET");
      return -2;
    };
  maxfd = sock;
  return 0;
};

int
TCP::get_nom_machine (MOT & nom)
{
  if (mode == ECHEC)
    return -1;
  nom = nom_distant;
  return 0;
};

int
TCP::get_port_machine (unsigned int &port)
{
  if (mode == ECHEC)
    return -1;
  port = port_distant;
  return 0;
};

#else

int
TCP::get_nom_machine (MOT & nom)
{
}

int
TCP::get_port_machine (unsigned int &port)
{
}

int
TCP::ouverture_socket_ecoute (int port)
{
}

int
TCP::connection_serveur (char *machine, int port)
{
}

int
TCP::ecoute_connection (TCP * c2)
{
}

int
TCP::ecriture (unsigned char msg_type,
	       void *buffer_ecr, unsigned int taille_buffer_ecr)
{
}

TCP::TCP ():mode (INACTIF)
{
  buffer = NULL;
};

TCP::~TCP ()
{

};

int
TCP::gestion_globale ()
{
  return 0;
};

MODE_SOCKET
TCP::etat ()
{
  return mode;
};

int
TCP::recup_info (unsigned char *msg_type,
		 void **buffer_recup, int &taille_info)
{

  return 0;
};

int
TCP::mise_en_lecture ()
{
  return 0;
}

int
TCP::mise_en_lecture_passive ()
{
  return 0;
};

#endif
