/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: UDP.cpp,v 1.4 2006/12/21 18:18:14 atila Exp $

#include "UDP.h"
#include <unistd.h>

#ifdef linux

int
UDP::connexion (int port_distant, int port_local, char *hote_distant)
{
  valide_connexion = ERREUR;
  struct hostent *hostp;
  struct sockaddr_in remote;
  if (((hostp = gethostbyname (hote_distant)) == 0))
    {
      printf ("MODULE COMM : ERREUR A LA RESOLUTION DU NOM");
      return -2;
    };
  memset ((void *) &remote, 0, sizeof remote);
  remote.sin_family = AF_INET;
  remote.sin_addr.s_addr = INADDR_ANY;
  memcpy ((void *) &remote.sin_addr, hostp->h_addr, hostp->h_length);
  remote.sin_port = htons (port_distant);
  struct sockaddr_in local;
  if ((sock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
    {
      printf ("MODULE COMM: ERREUR DE CREATION DE SOCKET.");
      return -2;
    };
  int vrai = true;
//printf("SO %i\n",
  setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &vrai, sizeof (int));
//);  
  memset ((void *) &local, 0, sizeof local);
  local.sin_family = AF_INET;
  local.sin_addr.s_addr = INADDR_ANY;
  local.sin_port = htons (port_local);
  if (bind (sock, (struct sockaddr *) &local, sizeof local) < 0)
    {
      printf ("MODULE COMM: ERREUR DE NOMMAGE DE SOCKET !!!");
      return -1;
    };
//printf("SO %i\n",
  setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &vrai, sizeof (int));
//);
  if (connect (sock, (struct sockaddr *) &remote, sizeof remote) < 0)
    {
      printf ("MODULE COMM: ERREUR DE CONNEXION UDP !!! BIZARRE ...");
      return -1;
    };
  valide_connexion = VEILLE;
  return 0;
};

UDP::UDP (bool asservi)
{
// Initialiser les canaux...
  for (int i = 0; i < NB_CANNAUX; i++)
    {
      canaux_UDP_emission[i].buffer = NULL;
      canaux_UDP_reception[i].buffer = NULL;

      canaux_UDP_emission[i].taille = 0;
      canaux_UDP_reception[i].taille = 0;
    };
  notour = 0;
  dernier_tour_recu = 0;
  asservissement = asservi;
  valide_connexion = NONPRETE;

  exclusion = SDL_CreateMutex ();
};

int
UDP::change_info_canal (bool typecanal, int canal, unsigned int taille,
			void *buffer)
{
  SDL_mutexP (exclusion);
  prepare_canal_UDP *can;
  if (typecanal)
    {
      can = &(canaux_UDP_emission[canal]);
    }
  else
    {
      can = &(canaux_UDP_reception[canal]);
    };
  if (can->buffer)
    free (can->buffer);
  can->buffer = NULL;
  can->taille = 0;
  if (buffer)
    {
      can->buffer = malloc (taille);
      can->taille = taille;
      memcpy (can->buffer, buffer, taille);
    };
  SDL_mutexV (exclusion);
};

int
UDP::change_info_canal_reception (int canal, unsigned int taille,
				  void *buffer)
{
  change_info_canal (false, canal, taille, buffer);
};

int
UDP::change_info_canal_emission (int canal, unsigned int taille, void *buffer)
{
  change_info_canal (true, canal, taille, buffer);
};

UDP::~UDP ()
{
  SDL_DestroyMutex (exclusion);
  close (sock);
};

int
UDP::envoyer (void *buffer, unsigned int longueur)
{
  if ((valide_connexion == EMISSION_RECEPTION)
      || (valide_connexion == VEILLE))
    {
      int nfound;
      fd_set mask;
      FD_ZERO (&mask);
      FD_SET (sock, &mask);
      struct timeval timeout = { 0, 0 };	// 0 sec de time-out... NON MAIS !!!
      nfound = select (FD_SETSIZE,
		       (fd_set *) 0, &mask, (fd_set *) 0, &timeout);
      if ((FD_ISSET (sock, &mask)) && (nfound > 0))
	return (send (sock, buffer, longueur, 0) < 0);
    }
  else
    return -1;
};

int
UDP::enveille ()
{
  if ((valide_connexion != ERREUR) && (valide_connexion != NONPRETE))
    {
      valide_connexion = VEILLE;
    }
  else
    return -1;
};

int
UDP::enfonction ()
{
  if ((valide_connexion != ERREUR) && (valide_connexion != NONPRETE))
    {
      valide_connexion = EMISSION_RECEPTION;
    }
  else
    return -1;
};

int
UDP::remplit_canal_emission (int nocanal, void *buffer, unsigned int taille)
{
  change_info_canal_emission (nocanal, taille, buffer);
};

int
UDP::recupere_canal_reception (int nocanal, void **buffer,
			       unsigned int &taille)
{

  SDL_mutexP (exclusion);

  *buffer = canaux_UDP_reception[nocanal].buffer;
  canaux_UDP_reception[nocanal].buffer = NULL;

  taille = canaux_UDP_reception[nocanal].taille;
  canaux_UDP_reception[nocanal].taille = 0;

  SDL_mutexV (exclusion);

  return 0;
};

int
UDP::gestion_globale ()
{

  unsigned char buf[100000];
  if ((valide_connexion != ERREUR) && (valide_connexion != NONPRETE))
    {
/* Emettre */
// Construction du buffer d'mission ...

      SDL_mutexP (exclusion);

      int NbBuff = 0;
      for (int i = 0; i < NB_CANNAUX; i++)
	if (canaux_UDP_emission[i].buffer)
	  NbBuff++;
      notour++;
      memcpy ((char *) &((struct entete_UDP *) buf)->signature, SIGNATURE_UDP,
	      T_SIGNATURE);
      if (asservissement)
	((struct entete_UDP *) buf)->notour = dernier_tour_recu;
      else
	((struct entete_UDP *) buf)->notour = notour;
      ((struct entete_UDP *) buf)->nb_bloc_cannaux = NbBuff;
// Remplir avec le contenu des cannaux...
      int position = sizeof (struct entete_UDP);

      for (int i = 0; i < NB_CANNAUX; i++)
	if (canaux_UDP_emission[i].buffer)
	  {
	    ((struct info_bloc_UDP *) &(buf[position]))->canal = i;
	    ((struct info_bloc_UDP *) &(buf[position]))->taille =
	      canaux_UDP_emission[i].taille;
	    position += sizeof (struct info_bloc_UDP);
	  };


      for (int i = 0; i < NB_CANNAUX; i++)
	if (canaux_UDP_emission[i].buffer)
	  {
	    memcpy (((char *) &buf[position]), canaux_UDP_emission[i].buffer,
		    canaux_UDP_emission[i].taille);
	    position += canaux_UDP_emission[i].taille;
	  };

      SDL_mutexV (exclusion);


      envoyer (buf, position);
/* Vider les buffers en Emission */
      for (int i = 0; i < NB_CANNAUX; i++)
	change_info_canal_emission (i, 0, NULL);

/* Vider les buffers en rception */
      for (int i = 0; i < NB_CANNAUX; i++)
	change_info_canal_reception (i, 0, NULL);

/*   Reception */
      bool trouve = true;
      while (trouve)
	{
	  int nfound;
	  fd_set mask;
	  FD_ZERO (&mask);
	  FD_SET (sock, &mask);
	  struct timeval timeout = { 0, 0 };	// 0 sec de time-out...
	  nfound = select (FD_SETSIZE,
			   &mask, (fd_set *) 0, (fd_set *) 0, &timeout);
	  if ((FD_ISSET (sock, &mask)) && (nfound > 0))
	    // Houston, on a qqch !!!
	    {
	      nfound = recv (sock, (void *) buf, 100000, 0);
	      // On analyse l'en-tete.  ( A la gueule du client, quoi.) ...(Mais,C'est pas drole!  :)).
	      if (!bcmp
		  ((char *) &(((struct entete_UDP *) buf)->signature),
		   SIGNATURE_UDP, T_SIGNATURE))
		{
		  // Bonne signature... Go'n Enjoy !
		  if (((struct entete_UDP *) buf)->notour > dernier_tour_recu)
		    {
		      dernier_tour_recu = ((struct entete_UDP *) buf)->notour;
		      // Dcomposition du paquet.
		      int NbBuff =
			((struct entete_UDP *) buf)->nb_bloc_cannaux;
		      int position_buffer =
			sizeof (struct entete_UDP) +
			NbBuff * sizeof (struct info_bloc_UDP);
		      struct info_bloc_UDP *bloc =
			(struct info_bloc_UDP *) ((char *) buf +
						  sizeof (struct entete_UDP));
		      for (int i = 0; i < NbBuff; i++)
			{
			  change_info_canal_reception (bloc[i].canal,
						       bloc[i].taille,
						       &(buf
							 [position_buffer]));
			  position_buffer += bloc[i].taille;
			};
		      //printf("OK %i \n",NbBuff);
		    }
		  else
		    {

		    }
		}
	      else
		{
		  printf
		    ("MODULE COMM: MAUVAIS PAQUET UDP ; MAUVAIS PROTO !!!");
		  return -1;
		};
	    }
	  else
	    {
	      trouve = false;
	    };
	};
    };
};



#else



int
UDP::connexion (int port_distant, int port_local, char *hote_distant)
{

  return 0;
};

UDP::UDP (bool asservi)
{

};

int
UDP::change_info_canal (bool typecanal, int canal, unsigned int taille,
			void *buffer)
{
  return 0;
};

int
UDP::change_info_canal_reception (int canal, unsigned int taille,
				  void *buffer)
{
  return 0;
};

int
UDP::change_info_canal_emission (int canal, unsigned int taille, void *buffer)
{
  return 0;
};

UDP::~UDP ()
{

};

int
UDP::envoyer (void *buffer, unsigned int longueur)
{
  return 0;
};

int
UDP::enveille ()
{
  return 0;
};

int
UDP::enfonction ()
{
  return 0;
};

int
UDP::remplit_canal_emission (int nocanal, void *buffer, unsigned int taille)
{
  return 0;
};

int
UDP::recupere_canal_reception (int nocanal, void **buffer,
			       unsigned int &taille)
{
  return 0;
};

int
UDP::gestion_globale ()
{
  return 0;
};

#endif
