/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*
Ce module implmente les routines attaches
 la gestion UDP de RWvsNDJ
*/
// $Id: UDP.h,v 1.4 2006/12/21 18:18:14 atila Exp $

#ifndef FICHIER_UDP
#define FICHIER_UDP


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

#ifdef linux
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#endif

#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <SDL/SDL.h>

#define PACKET_SIZE_MAX	20000

#define NB_CANNAUX  		32
// 32 canaux  mux.
#define PROFONDEUR_CANNAUX 	5
// Pour la gestion des packets mlangs.

#define T_SIGNATURE 6
#define SIGNATURE_UDP "RW_R1"

/****************************************************
Structures utilises pour le paquet UDP.
****************************************************/

struct entete_UDP
{
  char signature[T_SIGNATURE];
  unsigned int notour;
  unsigned char nb_bloc_cannaux;
};

struct info_bloc_UDP
{
  unsigned int taille;
  unsigned int canal;
};

/******************************************************
Gestion d'une connexion UDP
*******************************************************/

struct prepare_canal_UDP
{
  void *buffer;
  unsigned int taille;
};

typedef enum ETAT_CONNEXION_UDP
{
  VEILLE,
  NONPRETE,
  EMISSION_RECEPTION,
  ERREUR
};

class UDP
{
  int sock;
  int port_local;
  ETAT_CONNEXION_UDP valide_connexion;
  time_t derniere_heure;
  prepare_canal_UDP canaux_UDP_emission[NB_CANNAUX];
  prepare_canal_UDP canaux_UDP_reception[NB_CANNAUX];
  int dernier_tour_recu;
  int notour;
  bool asservissement;

  SDL_mutex *exclusion;

/* Les 3 fonctions internes suivantes manipulent les buffers... RECOPIE le buffer !!! */
  int change_info_canal (bool typecanal, int canal, unsigned int taille,
			 void *buffer);
  int change_info_canal_reception (int canal, unsigned int taille,
				   void *buffer);
  int change_info_canal_emission (int canal, unsigned int taille,
				  void *buffer);
  int envoyer (void *buffer, unsigned int longueur);

public:

    UDP (bool asservi);
  int connexion (int port_distant, int port_local, char *hote_distant);
   ~UDP ();

  int remplit_canal_emission (int nocanal, void *buffer, unsigned int taille);

/* File le buffer  dtruire ensuite !! */
  int recupere_canal_reception (int nocanal, void **buffer,
				unsigned int &taille);

/* Emet et recoit; Gre le Time-Out ; Dtruit les prcdents buffers */
  int gestion_globale ();
  int enveille ();
  int enfonction ();

// Pour simuler une emission.
  void simule_emission (float taux);	// taux reprsente le taux d'chec (alatoire)
};

#endif
