/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Id: animation.cpp,v 1.4 2006/12/28 20:29:00 atila Exp $

#include "animation.h"
#include "snd_mixer.h"

/****************INFO_ANIMATION */

void
INFO_ANIMATION::applique_animation (LPBLOCINFO lpblocinfo, LPANIMATION lpanim)
{
  LISTE_ELEMENT_ACTION::iterator parcours;
  for (parcours = liste_element_action.begin ();
       parcours != liste_element_action.end (); parcours++)
    {
      (*parcours)->applique_action (lpblocinfo, lpanim);
    };
};

INFO_ANIMATION::INFO_ANIMATION (BUFFER_NOEUD * BF, buffer_noeud * bn,
				LPANIMATION lpanim)
{
  buffer_noeud *bn2 = BF->trouve_suivant (bn, CONSTANTE_SCRIPT_NOM);
  nom = BF->get_MOT (bn2);
  while (bn)
    {
      if (BF->get_nom (bn) == CONSTANTE_SCRIPT_ACTION)
	liste_element_action.insert (liste_element_action.begin (),
				     new ELEMENT_ACTION (BF,
							 BF->get_fils (bn),
							 lpanim));
      bn = BF->trouve_suivant (bn);
    }
};

INFO_ANIMATION::~INFO_ANIMATION ()
{
  LISTE_ELEMENT_ACTION::iterator parcours;
  for (parcours = liste_element_action.begin ();
       parcours != liste_element_action.end (); parcours++)
    {
      delete (*parcours);
    }
};

bool
INFO_ANIMATION::operator== (MOT * m)
{
  return (nom == *m);
};

bool
INFO_ANIMATION::fusion (INFO_ANIMATION * ia)
{
  if (nom == ia->nom)
    {
      // Inserer les elements de ia. Supprimer la liste de ia.
      LISTE_ELEMENT_ACTION::iterator parcours;
      for (parcours = ia->liste_element_action.begin ();
	   parcours != ia->liste_element_action.end (); parcours++)
	{
	  liste_element_action.insert (liste_element_action.end (),
				       *parcours);
	}
      ia->liste_element_action.erase (ia->liste_element_action.begin (),
				      ia->liste_element_action.end ());
      return true;
    };
  return false;
};


/****************LISTE_INFO_ANIMATION */

BASE_ANIMATION::BASE_ANIMATION ()
{
};

BASE_ANIMATION::~BASE_ANIMATION ()
{
  LISTE_LPINFO_ANIMATION::iterator i;
  for (i = liste_lpinfo_animation.begin ();
       i != liste_lpinfo_animation.end (); i++)
    delete (*i);
};

void
BASE_ANIMATION::insert (BUFFER_NOEUD * BF, buffer_noeud * bn,
			LPANIMATION lpanim)
{
  liste_lpinfo_animation.insert (liste_lpinfo_animation.begin (),
				 new INFO_ANIMATION (BF, bn, lpanim));
}

void
BASE_ANIMATION::applique_animation (MOT * nom, LPBLOCINFO lpblocinfo,
				    LPANIMATION lpanim)
{
  LISTE_LPINFO_ANIMATION::iterator i;
  for (i = liste_lpinfo_animation.begin ();
       i != liste_lpinfo_animation.end (); i++)
    if ((*(*i)) == nom)
      break;
  if (i != liste_lpinfo_animation.end ())
    (*i)->applique_animation (lpblocinfo, lpanim);
  // on applique l'animation seulement
  //si on a une animation qui a ce nom-la
};

void
BASE_ANIMATION::grouper_animations ()
{
  LISTE_LPINFO_ANIMATION::iterator i, j;
  for (i = liste_lpinfo_animation.begin ();
       i != liste_lpinfo_animation.end (); i++)
    {
      for (j = i + 1; j != liste_lpinfo_animation.end (); j++)
	{
	  if (i != j)
	    if ((*i)->fusion (*j))
	      {
		// Dtruire j.
		delete *j;
		liste_lpinfo_animation.erase (j, j + 1);
		j = i;
	      };
	};
    }
}

/****************ANIMATION */

void
ANIMATION::trouve_parents ()
{
  // trouver les parents pour chaque objet;
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      (lpliste_objet[i])->
	set_index_pere (trouve_objet ((lpliste_objet[i])->get_nom_pere ()));
    };
};

void
ANIMATION::connecte_pointeurs ()
{
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      int index = lpliste_objet[i]->get_index_pere ();
      LPOBJET lpo = get_objet (index);
      (lpliste_objet[i])->set_lpparent (lpo);
    };
}


int
ANIMATION::trouve_objet (MOT * nom_trouve)
{
  LPOBJET obj_courant;
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      if ((*((lpliste_objet[i])->get_nom ())) == (*nom_trouve))
	return i;
    };
  return -1;			//objet non_trouv
};

void
ANIMATION::charge_animation (MOT * nom_fichier, LPBLOCINFO lpblocinfo)
{
  BUFFER_NOEUD BF (lpblocinfo->lpbdd, nom_fichier);
  buffer_noeud *bf;
  bf = BF.ROOT ();
  MOT vide;
  if ((!bf) && (!(*nom_fichier == vide)))
    printf ("Charge animation : %s NON-TROUVE \n", nom_fichier->info);
  else
    charge_animation (&BF, bf, lpblocinfo);
};


void
ANIMATION::charge_animation (BUFFER_NOEUD * BF, buffer_noeud * bf,
			     LPBLOCINFO lpblocinfo)
{
  buffer_noeud *bf_orig = bf;
  copie = false;
  rayon = 0;
  ensemble = false;
  centre = RWVECTOR (0, 0, 0);
  centre_fin = RWVECTOR (0, 0, 0);
  while (bf)
    {
      if (BF->get_nom (bf) == CONSTANTE_SCRIPT_RAYON)
	{
	  rayon = BF->get_float (bf);
	  ensemble = true;
	}
      if (BF->get_nom (bf) == CONSTANTE_SCRIPT_CENTRE)
	{
	  centre = VERTEX (BF, BF->get_fils (bf)).get_point ().T_VECTOR ();
	}
      if (BF->get_nom (bf) == CONSTANTE_SCRIPT_OBJET)
	{
	  buffer_noeud *bf2, *bf3;
	  buffer_noeud *bf_source = NULL;
	  bf2 = BF->get_fils (bf);

	  MOT nom_fichier_objet;
	  MOT nom_objet;
	  MOT nom_objet_pere;

	  bf3 = BF->trouve_suivant (bf2, CONSTANTE_SCRIPT_FICHIER);
	  nom_fichier_objet = BF->get_MOT (bf3);

	  bf_source = BF->trouve_suivant (bf2, CONSTANTE_SCRIPT_SOURCE);

	  bf3 = BF->trouve_suivant (bf2, CONSTANTE_SCRIPT_PERE);
	  nom_objet_pere = BF->get_MOT (bf3);

	  bf3 = BF->trouve_suivant (bf2, CONSTANTE_SCRIPT_NOM);
	  nom_objet = BF->get_MOT (bf3);

	  LPOBJET fils = NULL;
	  if (bf_source)
	    {
	      // Charger directement l'objet.
	      buffer_noeud *bf_source2 = BF->get_fils (bf_source);
	      fils = new OBJET (*BF, bf_source2, lpblocinfo);
	      //fils = (LPOBJET)fils->duplique_element();
	    }
	  else
	    {

	      int err = lpblocinfo->lpbanque_objet->trouve_element ((void *)
								    &nom_fichier_objet,
								    lpblocinfo,
								    &fils);
	    }
	  fils->set_nom (&nom_objet);
	  fils->set_nom_pere (&nom_objet_pere);
	  lpliste_objet.insert (lpliste_objet.end (), fils);
	}
      bf = BF->trouve_suivant (bf);
    }

  trouve_parents ();

  lpbase_animation = new BASE_ANIMATION ();
  bf = bf_orig;
  while (bf)
    {
      if (BF->get_nom (bf) == CONSTANTE_SCRIPT_ANIMATION)
	{
	  lpbase_animation->insert (BF, BF->get_fils (bf), this);
	}
      bf = BF->trouve_suivant (bf);
    }
  // Grouper les animations avec des noms similaires.
  lpbase_animation->grouper_animations ();
  // Joindre tous les objets au final.
  connecte_pointeurs ();
};

LPANIMATION
ANIMATION::copie_animation ()
{
  // On ralise une copie brute ...
  LPANIMATION lpa = new ANIMATION ();
  lpa->lpbase_animation = lpbase_animation;
  lpa->copie = true;
  lpa->centre = centre;
  lpa->rayon = rayon;
  lpa->ensemble = ensemble;
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      LPOBJET lpno = (lpliste_objet[i])->copie_objet ();
      lpa->lpliste_objet.insert (lpa->lpliste_objet.end (), lpno);
    };
  lpa->connecte_pointeurs ();
  return lpa;
};

ANIMATION::ANIMATION ()
{
  ensemble = false;
  lpbase_animation = NULL;
  copie = true;
  rayon = -1;
  premier_trace = true;
  moteur_callback = NULL;
};

LPOBJET
ANIMATION::get_objet (int indice)
{
  if (indice != -1)
    return lpliste_objet[indice];
  return NULL;
};

void
ANIMATION::pas_affichage ()
{
  if (moteur_callback)
    {
      moteur_callback->fin_animation (this);
    }
  moteur_callback = NULL;
  premier_trace = true;
}

ANIMATION::~ANIMATION ()
{
  pas_affichage ();
  // Destruction de la liste des objets.
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      LPOBJET objcour = lpliste_objet[i];
      delete (objcour);
    }
  if (!copie)
    {
      // Destruction des lments chargs en copie simple.
      delete (lpbase_animation);
    }
};

void
ANIMATION::init_animation ()
{
  int size = lpliste_objet.size ();
  MATRIX id;
  LPLISTE_OBJET::iterator fin = lpliste_objet.end ();
  for (LPLISTE_OBJET::iterator i = lpliste_objet.begin (); i != fin; i++)
    {
      LPOBJET objcour = *i;
      objcour->lrepere_global = objcour->repere_global;
      objcour->repere_local = id;
      objcour->repere_global = id;
      objcour->activation = 0;
      objcour->init_lumiere ();
    };
};

void
ANIMATION::applique_animation (MOT * nom, LPBLOCINFO lpblocinfo)
{
  if (lpbase_animation)
    lpbase_animation->applique_animation (nom, lpblocinfo, this);
};


void
ANIMATION::fin_animation (LPBLOCINFO lpblocinfo, LPMATRIX lpmatrice_finale)
{
  // calcul de reperes ...
  LPOBJET lpcourant;
  int taille = lpliste_objet.size ();
  if (!taille)
    return;
  for (int i = 0; i < taille; i++)
    {
      lpcourant = lpliste_objet[i];
      lpcourant->calcul = false;
      lpcourant->calcul_inverse = false;
      lpcourant->repere_global = lpcourant->repere_local;
    };
  BOOL modif = true;
  while (modif)
    {
      modif = false;
      for (int i = 0; i < lpliste_objet.size (); i++)
	{
	  lpcourant = lpliste_objet[i];
	  if (!(lpcourant->calcul))
	    {
	      if (lpcourant->lpparent)
		{
		  if (lpcourant->lpparent->calcul)	// le parent a-t-il calcul son repre ?
		    {
		      modif = true;
		      lpcourant->calcul = true;
		      // calcul correspondant;
		      lpcourant->repere_global =
			lpcourant->lpparent->repere_global *
			lpcourant->repere_local;
		    };
		}
	      else
		{
		  modif = true;
		  lpcourant->calcul = true;
		}
	    }
	}
    };
  // Mise en place des Matrices.
  if (lpmatrice_finale)
    {
      if (ensemble)
	matrice_fin_affiche = *lpmatrice_finale;
      for (int i = 0; i < taille; i++)
	{
	  // Mise en place des Matrices.
	  lpcourant = lpliste_objet[i];
	  lpcourant->repere_global =
	    (*lpmatrice_finale) * lpcourant->repere_global;
	};
    }
  for (int i = 0; i < taille; i++)
    {
      lpcourant = lpliste_objet[i];
      lpcourant->fin_animation (lpblocinfo);
      // Si c'est la camera, on met  jour dans lpblocinfo.
      if (lpcourant->is_camera)
	{
	  lpblocinfo->position_camera = lpcourant->repere_global;
	}
    };
  // Mise en place de la position des sons.
  RWVECTOR zero;
  for (int i = 0; i != liste_son_actif.size (); i++)
    {
      ACTION_ANIMATION_SON & aas = liste_son_actif[i];
      LPOBJET lpobjet = aas.lpobjet;
      RWVECTOR pos = lpobjet->repere_global.transforme (zero);
      lpblocinfo->lpmixer->controle_position_son (aas.ID_son, pos);
    }
  // mise en place du centre final si besoin.
  if (rayon != 0)
    centre_fin = lpmatrice_finale->transforme (centre);
};

void
ANIMATION::trace_animation (LPBLOCINFO lpblocinfo)
{
  if (premier_trace)
    {
      moteur_callback = lpblocinfo->lpmoteur;
      premier_trace = false;
      lpblocinfo->lpmoteur->nouvelle_animation (this);
    }
};

void
ANIMATION::ajoute_facettes_proches (RWVECTOR point,
				    float distance,
				    LPLISTE_LPFACETTE_PROCHE liste)
{
  int taille = lpliste_objet.size ();
  for (int i = 0; i < taille; i++)
    {
      lpliste_objet[i]->ajoute_facettes_proches (point, distance, liste,
						 this);
    }
};


ELEMENT_ACTION::~ELEMENT_ACTION ()
{
};

ELEMENT_ACTION::ELEMENT_ACTION (BUFFER_NOEUD * BN, buffer_noeud * bn,
				ANIMATION * lpanim)
{
  // Rcuprer le nom de l'objet ainsi que T1 et T2...
  buffer_noeud *bn2 = BN->trouve_suivant (bn, CONSTANTE_SCRIPT_OBJET);
  MOT nom_objet = BN->get_MOT (bn2);
  numero_objet = lpanim->trouve_objet (&nom_objet);

  bn2 = BN->trouve_suivant (bn, CONSTANTE_SCRIPT_T1);
  TEMPS1 = BN->get_float (bn2);
  bn2 = BN->trouve_suivant (bn, CONSTANTE_SCRIPT_T2);
  TEMPS2 = BN->get_float (bn2);

  sens_positif = true;
  sens_negatif = true;
  longueur_son = -1;

  while (bn)
    {
      bn2 = BN->get_fils (bn);
      switch (BN->get_nom (bn))
	{
	case CONSTANTE_SCRIPT_TRANSLATION:
	  type_action = TRANSLATION;
	  deb_pos = VERTEX (BN,
			    BN->get_fils (BN->
					  trouve_suivant (bn2,
							  CONSTANTE_SCRIPT_V1))).
	    get_point ().T_VECTOR ();
	  fin_pos =
	    VERTEX (BN,
		    BN->get_fils (BN->
				  trouve_suivant (bn2,
						  CONSTANTE_SCRIPT_V2))).
	    get_point ().T_VECTOR ();
	  break;
	case CONSTANTE_SCRIPT_ROTATION:
	  type_action = ROTATIONX;
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_AXEX))
	    type_action = ROTATIONX;
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_AXEY))
	    type_action = ROTATIONY;
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_AXEZ))
	    type_action = ROTATIONZ;
	  ANGLE1 =
	    BN->get_float (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_ANGLE1));
	  ANGLE2 =
	    BN->get_float (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_ANGLE2));
	  break;
	case CONSTANTE_SCRIPT_ECHELLE:
	  type_action = ECHELLE;
	  fin_echelle = VERTEX (BN,
				BN->get_fils (BN->
					      trouve_suivant (bn2,
							      CONSTANTE_SCRIPT_V2))).
	    get_point ().T_VECTOR ();
	  deb_echelle =
	    VERTEX (BN,
		    BN->get_fils (BN->
				  trouve_suivant (bn2,
						  CONSTANTE_SCRIPT_V1))).
	    get_point ().T_VECTOR ();
	  break;
	case CONSTANTE_SCRIPT_ACTIVEOBJET:
	  type_action = ACTIVEOBJET;
	  break;
	case CONSTANTE_SCRIPT_DESACTIVEOBJET:
	  type_action = DESACTIVEOBJET;
	  break;
	case CONSTANTE_SCRIPT_ACTION_LUMIERE:
	  type_action = ACTION_LUMIERE;
	  deb_valeur_lumiere = VALEUR_LUMIERE (BN,
					       BN->get_fils (BN->
							     trouve_suivant
							     (bn2,
							      CONSTANTE_SCRIPT_LUMIERE1)));
	  fin_valeur_lumiere =
	    VALEUR_LUMIERE (BN,
			    BN->get_fils (BN->
					  trouve_suivant (bn2,
							  CONSTANTE_SCRIPT_LUMIERE2)));
	  break;
	case CONSTANTE_SCRIPT_ACTIVE_SON:
	  type_action = ACTION_SON;
	  buffer_noeud *bn3 =
	    BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_FICHIER);
	  while (bn3)
	    {
	      MOT nom_son = BN->get_MOT (bn3);
	      liste_nom_son.insert (liste_nom_son.begin (), nom_son);
	      bn3 = BN->trouve_suivant (bn3);
	      bn3 = BN->trouve_suivant (bn3, CONSTANTE_SCRIPT_FICHIER);
	    };
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_SENSPOSITIF))
	    {
	      sens_negatif = false;
	    }
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_SENSNEGATIF))
	    {
	      sens_positif = false;
	    }
	  if (bn3 = BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_LONGUEURSON))
	    {
	      longueur_son = BN->get_float (bn3);
	    }
	  break;
	}
      bn = BN->trouve_suivant (bn);
    }
};


void
ELEMENT_ACTION::applique_action (LPBLOCINFO lpblocinfo, LPANIMATION lpanim)
{
  float T = lpblocinfo->T;
  float LT = lpblocinfo->LT;
  LPOBJET lpobjet = lpanim->get_objet (numero_objet);
  if ((T < TEMPS2) && (T >= TEMPS1) && lpobjet)	// validit temporelle et pointeur
    {
      switch (type_action)
	{
	case ACTIVEOBJET:
	  {
	    lpobjet->activation++;
	  };
	case DESACTIVEOBJET:
	  {
	    lpobjet->activation--;
	  };
	case TRANSLATION:
	  {
	    RWVECTOR trans_pos;
	    MATRIX translation;
	    trans_pos =
	      deb_pos + (fin_pos -
			 deb_pos) * ((T - TEMPS1) / (TEMPS2 - TEMPS1));
	    translation.translation (trans_pos);
	    if (lpobjet)
	      lpobjet->repere_local = lpobjet->repere_local * translation;
	    break;
	  };
	case ECHELLE:
	  {
	    RWVECTOR trans_pos;
	    MATRIX echelle;
	    trans_pos =
	      deb_echelle + (fin_echelle -
			     deb_echelle) * ((T - TEMPS1) / (TEMPS2 -
							     TEMPS1));
	    echelle.echelle (trans_pos);
	    if (lpobjet)
	      lpobjet->repere_local = lpobjet->repere_local * echelle;
	    break;
	  };
	case ROTATIONX:
	  {
	    float trans;
	    MATRIX rotation;
	    trans =
	      ANGLE1 + (T - TEMPS1) * (ANGLE2 - ANGLE1) / (TEMPS2 - TEMPS1);
	    rotation.rotationx (trans);
	    if (lpobjet)
	      lpobjet->repere_local = lpobjet->repere_local * rotation;
	    break;
	  };
	case ROTATIONY:
	  {
	    float trans;
	    MATRIX rotation;
	    trans =
	      ANGLE1 + (T - TEMPS1) * (ANGLE2 - ANGLE1) / (TEMPS2 - TEMPS1);
	    rotation.rotationy (trans);
	    if (lpobjet)
	      lpobjet->repere_local = lpobjet->repere_local * rotation;
	    break;
	  };
	case ROTATIONZ:
	  {
	    float trans;
	    MATRIX rotation;
	    trans =
	      ANGLE1 + (T - TEMPS1) * (ANGLE2 - ANGLE1) / (TEMPS2 - TEMPS1);
	    rotation.rotationz (trans);
	    if (lpobjet)
	      lpobjet->repere_local = lpobjet->repere_local * rotation;
	    break;
	  };
	case ACTION_LUMIERE:
	  {
	    VALEUR_LUMIERE val;
	    val.mix (deb_valeur_lumiere, fin_valeur_lumiere, TEMPS1, TEMPS2,
		     T);
	    if (lpobjet)
	      lpobjet->valeur_lumiere = val;
	  }
	  break;
	case ACTION_SON:
	  {
	    if (lpblocinfo->client)
	      {
		if ((!sens_positif) && (T > LT))
		  break;
		if ((!sens_negatif) && (T < LT))
		  break;
		// Regarder s'il est enclench.
		ACTION_ANIMATION_SON & aas = lpanim->get_son (this);
		bool reactive = false;
		if ((longueur_son != -1)
		    && (longueur_son < lpblocinfo->real_T - aas.temps))
		  {
		    reactive = true;
		  }
		if (((lpblocinfo->lpmixer->fin_son (aas.ID_son))
		     && (liste_nom_son.size () > 0)) || reactive)
		  {
		    //printf(" Relanc SP %i SN %i T %f LT %f \n",sens_positif,sens_negatif,T,LT);
		    int num_son = rand () % liste_nom_son.size ();
		    if ((num_son >= 0) && (num_son < liste_nom_son.size ()))
		      {
			MOT nom_son = liste_nom_son[num_son];
			int ID_son =
			  lpblocinfo->lpmixer->lance_son (nom_son, 128);
			aas.ID_son = ID_son;
			aas.lpobjet = lpobjet;
			aas.temps = lpblocinfo->real_T;
			//printf("Fini - Relanc SP %i SN %i T %f LT %f %s - TEMPS %f\n",
			//       sens_positif,sens_negatif,T,LT,nom_son.info,aas.temps);
		      };
		  };
	      };
	    break;
	  }
	};
    };
};


ACTION_ANIMATION_SON & ANIMATION::get_son (LPELEMENT_ACTION lpea)
{
  // Essaie de trouver le son
  for (int i = 0; i != liste_son_actif.size (); i++)
    {
      if (liste_son_actif[i].ID_action == lpea)
	return liste_son_actif[i];
    }
  // Insrer un nouveau son.
  struct ACTION_ANIMATION_SON
    anim_son;
  anim_son.ID_son = -1;
  anim_son.ID_action = lpea;
  liste_son_actif.insert (liste_son_actif.begin (), anim_son);
  return liste_son_actif[0];
};


void
ANIMATION::pre_mix ()
{
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      lpliste_objet[i]->pre_mix ();
    }
}

void
ANIMATION::mix (MOT & nom_racine, float t)
{
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      lpliste_objet[i]->mix (t);
    }
}

LPMATRIX
ANIMATION::get_ref_objet (MOT & nom)
{
  LPOBJET lpo;
  int alp = trouve_objet (&nom);
  if (alp != -1)
    lpo = get_objet (alp);
  else
    return NULL;
  return &(lpo->repere_global);
};

bool
  ANIMATION::impact (RWVECTOR & a, RWVECTOR & b, LPFACETTE * touche,
		     LPOBJET * lpo)
{
  bool retour = false;
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      if (lpliste_objet[i]->impact (a, b, touche))
	{
	  retour = true;
	  *lpo = lpliste_objet[i];
	};
    }
  return retour;
}

void
ANIMATION::set_texture_animation (GLuint no_texture)
{
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      lpliste_objet[i]->set_texture_objet (no_texture);
    };
}

//*************************************************
//      Pour implmenter classe_modele
//*************************************************

int
ANIMATION::charge_avec_fichier (void *data, LPBLOCINFO lpblocinfo)
{
  CLASSE_MODELE::charge_avec_fichier (data, lpblocinfo);
  charge_animation ((MOT *) data, lpblocinfo);
};

CLASSE_MODELE *
ANIMATION::duplique_element ()
{
  return copie_animation ();
};


//*************************************************
//      Pour faire des impacts sphriques.
//*************************************************

void
ANIMATION::impact_sphere (RWVECTOR & point, float distance,
			  float distance_facette, int ID,
			  LPLISTE_OBJET & liste, float temps)
{
  for (int i = 0; i < lpliste_objet.size (); i++)
    {
      if (lpliste_objet[i]->
	  impact_sphere (point, distance, distance_facette, ID, temps))
	{
	  liste.insert (liste.begin (), lpliste_objet[i]);
	}
    }
}
