/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: animation.h,v 1.3 2006/12/21 18:18:14 atila Exp $

#ifndef ANIMATION_FICHIER_H
#define ANIMATION_FICHIER_H

#include "lecture.h"
#include <GL/gl.h>
#include "def_3d.h"
#include "banque.h"
#include <vector>
#include "moteur3d.h"

//extern LPLISTE_SON lpliste_son;
class ANIMATION;
typedef ANIMATION *LPANIMATION;

/*********************************************************/
/* Creation d'une classe d'animation d'objet 3D          */
/*********************************************************/

// constantes predefinies pour les types d'actions

enum TYPE_ACTION
{ TRANSLATION,
  ROTATIONX, ROTATIONY, ROTATIONZ,
  ECHELLE,
  ACTIVEOBJET, DESACTIVEOBJET,
  ACTION_SON,
  ACTION_LUMIERE,
};
typedef vector < MOT > LISTE_NOM_SON;
// structure qui sert a stocker les actions


class ELEMENT_ACTION
{
  int numero_objet;

  // le type d'action a gerer
  TYPE_ACTION type_action;

  // des structures relatives a chaque type d'actions

  // Pour les oprateurs matriciels;
  RWVECTOR deb_pos, fin_pos;
  RWVECTOR deb_echelle, fin_echelle;
  float ANGLE1, ANGLE2;

  // Pour les loupiottes.
  VALEUR_LUMIERE deb_valeur_lumiere, fin_valeur_lumiere;

  // Pour les sons.
  LISTE_NOM_SON liste_nom_son;
  // Pour avoir le sens du son, A priori, on le joue dans les 2 sens.
  bool sens_positif, sens_negatif;
  // Pour overrider la taille du son;
  // on met  -1 pour forcer le son  tre rejou (si ncessaire)  la fin de la dure indique.
  // ATTENTION : un temps trop court risque d'entrainer une boucle infinie !!!
  float longueur_son;

  float TEMPS1, TEMPS2;		// les temps de depart et de fin sont globaux
  // pour chaque type d'action : le calcul des diffrents
  // paramètres au temps T se font linairement
public:  ELEMENT_ACTION (BUFFER_NOEUD * BF, buffer_noeud * bn,
		    LPANIMATION lpanim);
   ~ELEMENT_ACTION ();
  MOT *lecture_action ();
  void applique_action (LPBLOCINFO lpblocinfo, LPANIMATION lpanim);

};

typedef ELEMENT_ACTION *LPELEMENT_ACTION;
typedef vector < LPELEMENT_ACTION > LISTE_ELEMENT_ACTION;

// structure qui sert a stocker les informations d'animations

class INFO_ANIMATION
{
  LISTE_ELEMENT_ACTION liste_element_action;
  MOT nom;
public:
  void applique_animation (LPBLOCINFO lpblocinfo, LPANIMATION lpanim);
    INFO_ANIMATION (BUFFER_NOEUD * BF, buffer_noeud * bn, LPANIMATION lpanim);
   ~INFO_ANIMATION ();
  bool operator== (MOT * m);
  bool fusion (INFO_ANIMATION *);
};

typedef INFO_ANIMATION *LPINFO_ANIMATION;
typedef vector < LPINFO_ANIMATION > LISTE_LPINFO_ANIMATION;

// classe de stockage des diffrentes info_animation

class BASE_ANIMATION
{
  LISTE_LPINFO_ANIMATION liste_lpinfo_animation;

public:

  void grouper_animations ();
  void applique_animation (MOT * nom, LPBLOCINFO lpblocinfo,
			   LPANIMATION lpanim);
  void insert (BUFFER_NOEUD * BF, buffer_noeud * bn, LPANIMATION lpanim);
    BASE_ANIMATION ();
   ~BASE_ANIMATION ();
};

typedef BASE_ANIMATION *LPBASE_ANIMATION;

typedef vector < LPOBJET > LPLISTE_OBJET;

// Cet structure est utilise pour savoir si un son lanc
// par une action est encore actif.
struct ACTION_ANIMATION_SON
{
  // Le son, identifi par son identificateur.
  int ID_son;
  // L'action, identifie par son pointeur unique.
  LPELEMENT_ACTION ID_action;
  // L'objet considr, par un pointeur.
  LPOBJET lpobjet;
  // Le temps  partir duquel on a commenc  jouer le son. Ca sert  overrider
  // le principe d'attente de fin du son, en spcifiant un temps fixe.
  float temps;
};

typedef vector < struct ACTION_ANIMATION_SON >LISTE_SON_ACTIF;

class MOTEUR;

class ANIMATION:public CLASSE_MODELE
{

protected:
  // Centre et rayon pour le moteur 3D.
  // Le centre est deplace par rapport a la matrice finale.
  RWVECTOR centre, centre_fin;
  bool ensemble;
  float rayon;


  // Pour savoir si c'est ralis  partir d'une copie.
  // Dans ce cas, on ne dtruit pas TOUTES les infos.
  bool copie;


  LPBASE_ANIMATION lpbase_animation;	// chaine a ne pas dupliquer
  LPLISTE_OBJET lpliste_objet;

  // La liste des portals associes si c'est un groupe (ensemble=true)
  LISTE_INDICE_PORTAL indice_portal;
  // La liste des lumieres qui s'applique a toute l'animation.
  LPLISTE_OBJET lpliste_objet_lumiere_animation;
  LISTE_SON_ACTIF liste_son_actif;

    ACTION_ANIMATION_SON & get_son (LPELEMENT_ACTION);

  // Si il faut s'enregistrer aupres du moteur.
  bool premier_trace;
  // pour se desenregistrer a la destruction.
  LPMOTEUR moteur_callback;

  // La matrice passee pour tous les objets au moment de tracer.
  MATRIX matrice_fin_affiche;


public:

  // On arrete l'affichage au moteur 3D.
  void pas_affichage ();


  void trouve_parents ();
  void connecte_pointeurs ();

  int trouve_objet (MOT * nom_trouve);
  void charge_animation (MOT * nom_fichier, LPBLOCINFO lpblocinfo);
  void charge_animation (BUFFER_NOEUD * BF, buffer_noeud * bn,
			 LPBLOCINFO lpblocinfo);
  LPANIMATION copie_animation ();
  void init_animation ();
  void applique_animation (MOT * nom, LPBLOCINFO lpblocinfo);
  void fin_animation (LPBLOCINFO, LPMATRIX);
  void trace_animation (LPBLOCINFO lpblocinfo);
  LPMATRIX get_ref_objet (MOT & nom);
  LPOBJET get_objet (int i);
    ANIMATION ();
   ~ANIMATION ();
  void ajoute_facettes_proches (RWVECTOR point,
				float distance,
				LPLISTE_LPFACETTE_PROCHE liste);

  void pre_mix ();
  void mix (MOT & nom_racine, float t);

  bool impact (RWVECTOR & a, RWVECTOR & b, LPFACETTE * touche, LPOBJET * lpo);

  // Pour la reconnaissance et la recopie. VOIR classe_modele.h
  int charge_avec_fichier (void *data, LPBLOCINFO lpblocinfo);
  CLASSE_MODELE *duplique_element ();

  // Pour les impacts sphriques de tous poils. voir def.cpp / facette.cpp pour les dtails
  // de l'implmentation.
  // A savoir si ca doit aller ANIMATION ou dans SALLE : la rponse
  // est : ici ca risque de planter si on appelle le callback sur un objet limin,
  // dans SALLE on se prive d'effets de lumière/sang/... sur les objets.
  //
  // Parametre supplmentaire liste pour stocker les objets supplmentaires impliqus
  // dans l'impact.
  // Rappel : Ceux-ci peuvent tre rpartis sur plusieurs animations !!!!
  void impact_sphere (RWVECTOR & point, float distance,
		      float distance_facette, int ID, LPLISTE_OBJET & liste,
		      float temps);
  void set_T (float diff);
  void set_texture_animation (GLuint no_texture);

  friend class ELEMENT_ACTION;
  friend class MOTEUR;

};

typedef vector < LPANIMATION > LISTE_ANIMATION;

#endif
