/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "animation_texture.h"


/**********************************************************
 *
 *		ACTION_ANIMATION_TEXTURE
 *
 **********************************************************/


void
ACTION_ANIMATION_TEXTURE::pre_applique_action (LPBLOCINFO lpblocinfo,
					       MATRIX & m, GLuint & desc_text)
{
  float T = lpblocinfo->T;
  float LT = lpblocinfo->LT;
  if ((LT <= TEMPS2) && (LT >= TEMPS1))	// validit temporelle 
    {
      switch (type_action_texture)
	{
	case TEXTURE_SAUT:
	  {
	    lpblocinfo->T = SautT;
	    lpblocinfo->LT = SautT;
	    break;
	  }
	case TEXTURE_SAUT_ALEATOIRE:
	  {
	    lpblocinfo->T =
	      rand () / (float) RAND_MAX *(SautAleatoireT2 -
					   SautAleatoireT1) + SautAleatoireT1;
	    break;
	  }
	}
    }
}

void
ACTION_ANIMATION_TEXTURE::post_applique_action (LPBLOCINFO lpblocinfo,
						MATRIX & m,
						GLuint & desc_text)
{
  float T = lpblocinfo->T;
  float LT = lpblocinfo->LT;

  if ((T <= TEMPS2) && (T >= TEMPS1))	// validit temporelle
    {
      switch (type_action_texture)
	{
	case TEXTURE_TRANSLATION:
	  {
	    RWVECTOR trans_pos;
	    MATRIX translation;
	    trans_pos =
	      ((fin_pos - deb_pos) * ((T - TEMPS1) / (TEMPS2 - TEMPS1))) +
	      deb_pos;
	    translation.translation (trans_pos);
	    m = m * translation;
	    break;
	  };
	case TEXTURE_ROTATIONX:
	  {
	    float trans;
	    MATRIX rotation;
	    trans =
	      ANGLE1 + (T - TEMPS1) * (ANGLE2 - ANGLE1) / (TEMPS2 - TEMPS1);
	    rotation.rotationx (trans);
	    m = m * rotation;
	    break;
	  };
	case TEXTURE_ROTATIONY:
	  {
	    float trans;
	    MATRIX rotation;
	    trans =
	      ANGLE1 + (T - TEMPS1) * (ANGLE2 - ANGLE1) / (TEMPS2 - TEMPS1);
	    rotation.rotationy (trans);
	    m = m * rotation;
	    break;
	  };
	case TEXTURE_ROTATIONZ:
	  {
	    float trans;
	    MATRIX rotation;
	    trans =
	      ANGLE1 + (T - TEMPS1) * (ANGLE2 - ANGLE1) / (TEMPS2 - TEMPS1);
	    rotation.rotationz (trans);
	    m = m * rotation;
	    break;
	  };
	case TEXTURE_ECHELLE:
	  {
	    RWVECTOR trans_pos;
	    MATRIX echelle;
	    trans_pos =
	      ((fin_echelle -
		deb_echelle) * ((T - TEMPS1) / (TEMPS2 - TEMPS1))) +
	      deb_echelle;
	    echelle.echelle (trans_pos);
	    m = m * echelle;
	    break;
	  };
	case TEXTURE_APPLIQUE_TEXTURE:
	  {
	    desc_text = desc_texture;
	  };
	};
    };

}

ACTION_ANIMATION_TEXTURE::~ACTION_ANIMATION_TEXTURE ()
{
};

bool
ACTION_ANIMATION_TEXTURE::get_blend ()
{
  return blend;
};

ACTION_ANIMATION_TEXTURE::ACTION_ANIMATION_TEXTURE (BUFFER_NOEUD * BN,
						    buffer_noeud * bn,
						    LPBLOCINFO lpblocinfo)
{
  // Rcuprer T1 et T2...
  buffer_noeud *bn2;
  bn2 = BN->trouve_suivant (bn, CONSTANTE_SCRIPT_T1);
  TEMPS1 = BN->get_float (bn2);
  bn2 = BN->trouve_suivant (bn, CONSTANTE_SCRIPT_T2);
  TEMPS2 = BN->get_float (bn2);
  blend = false;
  while (bn)
    {
      bn2 = BN->get_fils (bn);
      switch (BN->get_nom (bn))
	{
	case CONSTANTE_SCRIPT_ECHELLE:
	  type_action_texture = TEXTURE_ECHELLE;
	  fin_echelle =
	    VERTEX (BN,
		    BN->get_fils (BN->
				  trouve_suivant (bn2,
						  CONSTANTE_SCRIPT_V2))).
	    get_point ();
	  deb_echelle =
	    VERTEX (BN,
		    BN->get_fils (BN->
				  trouve_suivant (bn2,
						  CONSTANTE_SCRIPT_V1))).
	    get_point ();
	  break;
	case CONSTANTE_SCRIPT_TRANSLATION:
	  type_action_texture = TEXTURE_TRANSLATION;
	  deb_pos =
	    VERTEX (BN,
		    BN->get_fils (BN->
				  trouve_suivant (bn2,
						  CONSTANTE_SCRIPT_V1))).
	    get_point ();
	  fin_pos =
	    VERTEX (BN,
		    BN->get_fils (BN->
				  trouve_suivant (bn2,
						  CONSTANTE_SCRIPT_V2))).
	    get_point ();
	  break;
	case CONSTANTE_SCRIPT_ROTATION:
	  type_action_texture = TEXTURE_ROTATIONX;
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_AXEX))
	    type_action_texture = TEXTURE_ROTATIONX;
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_AXEY))
	    type_action_texture = TEXTURE_ROTATIONY;
	  if (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_AXEZ))
	    type_action_texture = TEXTURE_ROTATIONZ;
	  ANGLE1 =
	    BN->get_float (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_ANGLE1));
	  ANGLE2 =
	    BN->get_float (BN->trouve_suivant (bn2, CONSTANTE_SCRIPT_ANGLE2));
	  break;
	case CONSTANTE_SCRIPT_AT_SAUT:
	  type_action_texture = TEXTURE_SAUT;
	  SautT =
	    BN->get_float (BN->
			   trouve_suivant (bn2, CONSTANTE_SCRIPT_AT_TEMPS));
	  break;
	case CONSTANTE_SCRIPT_AT_SAUTALEATOIRE:
	  type_action_texture = TEXTURE_SAUT_ALEATOIRE;
	  SautAleatoireT1 =
	    BN->get_float (BN->
			   trouve_suivant (bn2,
					   CONSTANTE_SCRIPT_AT_TEMPSALEATOIRE1));
	  SautAleatoireT2 =
	    BN->get_float (BN->
			   trouve_suivant (bn2,
					   CONSTANTE_SCRIPT_AT_TEMPSALEATOIRE2));
	  break;
	case CONSTANTE_SCRIPT_AT_TEXTURE:
	  type_action_texture = TEXTURE_APPLIQUE_TEXTURE;
	  desc_texture =
	    lpblocinfo->lpbanque_texture->trouve_texture (lpblocinfo->lpbdd,
							  BN,
							  BN->get_fils (bn),
							  blend);
	  break;
	}
      bn = BN->trouve_suivant (bn);
    }
  //printf("ACTION %i -  %f %f %f  - %f %f %f - %f %f - %f %f \n",type_action_texture,
  //            deb_pos.T_VECTOR().x,deb_pos.T_VECTOR().y,deb_pos.T_VECTOR().z,
  //            fin_pos.T_VECTOR().x,fin_pos.T_VECTOR().y,fin_pos.T_VECTOR().z,
  //            ANGLE1,ANGLE2,TEMPS1,TEMPS2);
};


/**********************************************************
 *
 *		ANIMATION_TEXTURE
 *
 **********************************************************/

ANIMATION_TEXTURE::ANIMATION_TEXTURE ()
{
  // Rserver un slot OpenGL.
  GLdesc = glGenLists (1);
  glNewList (GLdesc, GL_COMPILE);
  glEndList ();
  T = 0;
  LT = 0;
}

ANIMATION_TEXTURE::~ANIMATION_TEXTURE ()
{
  glDeleteLists (GLdesc, 1);
  for (int i = 0; i != liste_action_animation_texture.size (); i++)
    {
      delete liste_action_animation_texture[i];
    };
}

int
ANIMATION_TEXTURE::charge_avec_fichier (void *data, LPBLOCINFO lpblocinfo)
{
  CLASSE_MODELE::charge_avec_fichier (data, lpblocinfo);
  MOT nom_fichier = *(MOT *) data;
  BUFFER_NOEUD BF (lpblocinfo->lpbdd, &nom_fichier);
  buffer_noeud *bf = BF.ROOT ();
  if (!bf)
    {
      printf ("Fichier d'animation texture non-trouv : %s\n",
	      nom_fichier.info);
      return 0;
    }
  while (bf)
    {
      switch (BF.get_nom (bf))
	{
	case CONSTANTE_SCRIPT_ACTION:
	  {
	    ACTION_ANIMATION_TEXTURE *aat;
	    aat =
	      new ACTION_ANIMATION_TEXTURE (&BF, BF.get_fils (bf),
					    lpblocinfo);
	    liste_action_animation_texture.
	      insert (liste_action_animation_texture.end (), aat);
	    break;
	  }
	}
      bf = BF.trouve_suivant (bf);
    }
  blend = false;
  for (int i = 0; i != liste_action_animation_texture.size (); i++)
    {
      if (liste_action_animation_texture[i]->get_blend ())
	blend = true;
    }
};


CLASSE_MODELE *
ANIMATION_TEXTURE::duplique_element ()
{
  // Pas de duplicata !!
  return this;
};

void
ANIMATION_TEXTURE::gestion_globale (LPBLOCINFO lpblocinfo)
{
  // Gestion : regnrer la liste d'affichage.
  BLOCINFO bi = *lpblocinfo;

  LT = T;
  T = T + lpblocinfo->diff;

  bi.LT = LT;
  bi.T = T;
  MATRIX m;
  GLuint desc_text = 0;
  deque < ACTION_ANIMATION_TEXTURE * >::iterator parcours;
  for (parcours = liste_action_animation_texture.begin ();
       parcours != liste_action_animation_texture.end (); parcours++)
    {
      (*parcours)->pre_applique_action (&bi, m, desc_text);
    }
  for (parcours = liste_action_animation_texture.begin ();
       parcours != liste_action_animation_texture.end (); parcours++)
    {
      (*parcours)->post_applique_action (&bi, m, desc_text);
    }

  T = bi.T;
  LT = bi.LT;

  // crer finalement la liste.
  MATRIX m2 = m.special_OpenGL ();
  glNewList (GLdesc, GL_COMPILE);
  glMatrixMode (GL_TEXTURE);
  glLoadMatrixf ((const GLfloat *) &m2);
  glBindTexture (GL_TEXTURE_2D, desc_text);
  glMatrixMode (GL_MODELVIEW);
  glEndList ();
};

void
ANIMATION_TEXTURE::gestion_locale (LPBLOCINFO lpblocinfo, MATRIX & m,
				   GLuint & desc_text)
{
  // Gestion locale: calculer les paramtres matrice/descripteur.
  lpblocinfo->LT = lpblocinfo->T;
  lpblocinfo->T = lpblocinfo->T + lpblocinfo->diff;

  deque < ACTION_ANIMATION_TEXTURE * >::iterator parcours;
  for (parcours = liste_action_animation_texture.begin ();
       parcours != liste_action_animation_texture.end (); parcours++)
    {
      (*parcours)->pre_applique_action (lpblocinfo, m, desc_text);
    }
  for (parcours = liste_action_animation_texture.begin ();
       parcours != liste_action_animation_texture.end (); parcours++)
    {
      (*parcours)->post_applique_action (lpblocinfo, m, desc_text);
    }
}

GLuint
ANIMATION_TEXTURE::get_GLdesc ()
{
  return GLdesc;
};

bool
ANIMATION_TEXTURE::get_blend ()
{
  return blend;
};


/**********************************************************
 *
 *		BANQUE_ANIMATION_TEXTURE
 *
 **********************************************************/

GLuint
  BANQUE_ANIMATION_TEXTURE::trouve_animation_texture (LPBLOCINFO lpblocinfo,
						      MOT & fichier,
						      bool & blend)
{
  ANIMATION_TEXTURE *animt;
  renvoie_element ((void *) &fichier, lpblocinfo, &animt, false);
  blend = animt->get_blend ();
  return animt->get_GLdesc ();
}


BANQUE_ANIMATION_TEXTURE::BANQUE_ANIMATION_TEXTURE ()
{
}

BANQUE_ANIMATION_TEXTURE::~BANQUE_ANIMATION_TEXTURE ()
{
}
void
BANQUE_ANIMATION_TEXTURE::gestion_globale (LPBLOCINFO lpblocinfo)
{
  deque < ANIMATION_TEXTURE * >::iterator parcours;
  for (parcours = liste_element.begin ();
       parcours != liste_element.end (); parcours++)
    {
      (*parcours)->gestion_globale (lpblocinfo);
    }
};

ANIMATION_TEXTURE *
BANQUE_ANIMATION_TEXTURE::
trouve_animation_texture_brutal (LPBLOCINFO lpblocinfo, MOT & fichier)
{
  ANIMATION_TEXTURE *animt;
  renvoie_element ((void *) &fichier, lpblocinfo, &animt, false);
  return animt;
}
