/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: banque.h,v 1.2 2004/06/02 17:14:29 atila Exp $

#ifndef FICHIER_BANQUE
#define FICHIER_BANQUE

using namespace std;
#include <deque>
#include "classe_modele.h"

struct BLOCINFO;
#define LPBLOCINFO BLOCINFO*

template < class T > class BANQUE
{
protected:

  deque < T * >liste_element;

  // Parcours de la liste à la recherche d'un objet 
  // dont le nom corresponde.

  T *retrouve_element (void *data);

public:

  // Copie si trouvé ! Charge sinon, copie et renvoie la copie.
  int trouve_element (void *data, LPBLOCINFO lpblocinfo, T ** resultat);

  // "copie" spécifie si il en faut une copie, ou une simple référence.
  int renvoie_element (void *data, LPBLOCINFO lpblocinfo, T ** resultat,
		       int copie);

  // Générer l'élément sans l'insérer dans la liste.
  int charge_element (void *data, LPBLOCINFO lpblocinfo, T ** resultat);

  // Destruction de l'ensemble des infos contenues.
  void nettoyage ();

  BANQUE ()
  {
  };

  ~BANQUE ()
  {
    nettoyage ();
  };

  typedef typename deque < T * >::iterator banque_iterator;

};

template < class T >
  int BANQUE < T >::charge_element (void *data, LPBLOCINFO lpblocinfo,
				    T ** resultat)
{
  T *lpt = new T ();
  ((CLASSE_MODELE *) lpt)->charge_avec_fichier (data, lpblocinfo);
  *resultat = lpt;
}

template < class T > void BANQUE < T >::nettoyage ()
{
  banque_iterator elem = liste_element.begin ();
  while (elem != liste_element.end ())
    {
      delete (*elem);
      elem++;
    };
  liste_element.clear ();
}

template < class T > T * BANQUE < T >::retrouve_element (void *data)
{
  banque_iterator elem = liste_element.begin ();
  while (elem != liste_element.end ())
    {
      if ((**elem) == data)
	return *elem;
      else
	elem++;
    };
  return NULL;
};

template < class T >
  int BANQUE < T >::renvoie_element (void *data, LPBLOCINFO lpblocinfo,
				     T ** resultat, int copie)
{
  T *trouve = retrouve_element (data);
  if (trouve)
    {
      if (copie)
	*resultat = (T *) (trouve->duplique_element ());
      else
	*resultat = trouve;
      return 0;
    }
  else
    {
      *resultat = NULL;
      charge_element (data, lpblocinfo, resultat);
      liste_element.insert (liste_element.begin (), *resultat);
      if (copie)
	*resultat = (T *) ((*resultat)->duplique_element ());
      return 0;
    };
};

template < class T >
  int BANQUE < T >::trouve_element (void *data, LPBLOCINFO lpblocinfo,
				    T ** resultat)
{
  return renvoie_element (data, lpblocinfo, resultat, true);
};

#endif
