/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: connexion.h,v 1.2 2004/06/02 17:14:29 atila Exp $

#ifndef FICHIER_CONNEXION
#define FICHIER_CONNEXION

#include "TCP.h"
#include "UDP.h"
#include "lecture.h"
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>

// Le temps attendu entre chaque "ping" de la connexion TCP.
#define TEMPS_LAG_MAXIMAL	1

/* Ceci implmente l'tat d'une connexion 
	machine/machine 
en groupant les 2 protocoles UDP et TCP. */

///=========================================================
/*
Voici les codes utiliss pour la connexion TCP
*/
//=========================================================
#define TCP_S_INFO_SERVEUR 	101	// La rponse du serveur : infos serveur.

#define TCP_C_CONN 		102	// Demande connexion au serveur
					// Donne les infos
#define TCP_S_CONN_OK		103	// Accepte
					// Donne les caractristiques
#define TCP_S_CONN_CANCEL	104	// Refuse

#define TCP_C_FIN_SERVEUR	150	// Dconnexion serveur.
#define TCP_S_FIN_CLIENT	151	// Dconnexion client

#define TCP_S_FIN_PARTIE	152	// Fin de partie OK.
#define TCP_C_FIN_PARTIE	153	// Fin de partie OK.

#define TCP_S_DEB_PARTIE	154	// Debut de partie OK.
#define TCP_C_DEB_PARTIE	155	// Debut de partie OK.

#define TCP_CS_DEMANDE_FICHIER	200
#define TCP_CS_FIN_DEMANDE	201
#define TCP_CS_FICHIER_OK	202
#define TCP_CS_FICHIER_CANCEL	203

#define TCP_CS_PING		1


enum ETAT_CONNEXION
{
  CONNEXION_AUCUNE,
  CONNEXION_ATTENTE,
  CONNEXION_CONNECTE,
  CONNEXION_JEU,
  CONNEXION_REFUSE,
  CONNEXION_ERREUR,
  CONNEXION_FIN
};

class CONNEXION;

int point_entree_thread (void *conn);

class CONNEXION
{
  ETAT_CONNEXION etat;

  double DeltaT_UDP;

  bool client;

  TCP *connexion_tcp;
  UDP *connexion_udp;


  bool fin_partie;
  bool tour_ecriture;
  time_t dernier_temps;


  SDL_Thread *thread_global;
  bool demande_fin_thread;
  bool thread_termine;

  friend int point_entree_thread (void *);
  void thread_gestion_globale (void);
  int thread_attente_fin_evenement ();
  void thread_initie_download (MOT * nom_fichier_princ, int taille);
  void thread_UPload ();
  void thread_DOWNload ();
  void thread_gestion_client ();
  void thread_gestion_serveur ();
  int thread_time_has_come ();
  void thread_passage_tour ();

public:int ID_connexion;

  // Ca, c'est pour savoir si on veut juste rcuprer 
  // les infos du serveur ou si on veux jouer !!!!!
  bool info_jeu;
  void set_info_jeu (bool);

  bool bloc_jeu_ok;
  bool get_bloc_jeu_ok ();
  int get_bloc_jeu (void **bloc, int *taille);
  int set_bloc_jeu (void *bloc, int taille);
  void *bloc_jeu;
  int taille_jeu;

  bool info_joueur_ok;
  bool get_info_joueur_ok ();
  int set_info_joueur (void *, int);
  int get_info_joueur (void **, int *);
  void *bloc_info_joueur;
  int taille_info_joueur;

  bool carac_joueur_ok;
  bool get_carac_joueur_ok ();
  int get_carac_joueur (void **, int *);
  int set_carac_joueur (void *, int);
  void *bloc_carac_joueur;
  int taille_carac_joueur;

  bool fichier_up_ok;
  bool get_fichier_up_ok ();
  int set_fichier_up (MOT fichier_skin, int tfu);
  int get_fichier_up (MOT * fichier_skin, int *tfu);
  MOT fichier_up;
  int taille_fichier_up;

  int ecrire_canal_UDP (int no_canal, unsigned int taille, void *buffer);
  int recuperer_canal_UDP (int no_canal, unsigned int *taille, void **buffer);

  // Pour le client/serveur, on a une gestion externe de l'UDP.
  // Cet appel prend nanmois en compte le temps (inutile d'emettre
  // plus que ce que la connexion supporte) et l'existence de la liaison
  // UDP.
  void gestion_tour (double DeltaT);

  // Une fonction client.
  int connexion (int port, char *hote);

  // Une fonction serveur.
  int recup_connexion (TCP * conn);

  ETAT_CONNEXION recup_etat ();

    CONNEXION (bool client_default);
   ~CONNEXION ();
};

#endif
