/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: controle.cpp,v 1.3 2006/12/21 18:18:14 atila Exp $

#include "controle.h"


CONTROLE::CONTROLE ()
{
  premier = 0;
  x_view = 0;
  z_view = 0;
  raw_x_view = 0;
  raw_y_view = 0;
  mouse_sensib = 1;
  for (int i = 0; i < MAX_TOUCHE; i++)
    touche[i] = false;
  for (int i = 0; i < MAX_TOUCHE; i++)
    touche_simple[i] = false;
  for (int i = 0; i < MAX_SOURIS; i++)
    souris[i] = false;

  // Touches par dfaut...

  touche_controle[TOUCHE_AVANCE] = SDLK_UP;
  touche_controle[TOUCHE_RECULE] = SDLK_DOWN;
  touche_controle[TOUCHE_STRAFE_GAUCHE] = SDLK_RIGHT;
  touche_controle[TOUCHE_STRAFE_DROITE] = SDLK_LEFT;
  touche_controle[TOUCHE_TIR] = -1;
  touche_controle[TOUCHE_SAUTE] = -3;
  touche_controle[TOUCHE_TEXTE] = SDLK_t;
  touche_controle[TOUCHE_ACTION] = SDLK_a;
  touche_controle[TOUCHE_ARME + 0] = SDLK_q;
  touche_controle[TOUCHE_ARME + 1] = SDLK_s;
  touche_controle[TOUCHE_ARME + 2] = SDLK_d;
  touche_controle[TOUCHE_ARME + 3] = SDLK_f;
  touche_controle[TOUCHE_ARME + 4] = SDLK_g;
  touche_controle[TOUCHE_ARME + 5] = SDLK_h;
  touche_controle[TOUCHE_ARME + 6] = SDLK_j;
  touche_controle[TOUCHE_ARME + 7] = SDLK_k;
  touche_controle[TOUCHE_ARME + 8] = SDLK_l;
  touche_controle[TOUCHE_ARME + 9] = SDLK_m;
  touche_controle[TOUCHE_VUE] = SDLK_v;
  touche_controle[TOUCHE_FULLSCREEN] = SDLK_F3;
  mouse_sensib = 1.0;
  mouse_inverse = true;
}

CONTROLE::~CONTROLE ()
{
}

void
CONTROLE::touche_handle (SDL_keysym * keysym, bool up)
{
  touche[keysym->sym] = up;
  touche_simple[keysym->sym] = up;

  // Sortie force...
  if (SDLK_ESCAPE == keysym->sym)
    {
      SDL_Quit ();
      exit (0);
    }

  // Petite pause caf...
  if (SDLK_F1 == keysym->sym)
    {
#ifdef LINUX
      sleep (2);
#endif
    }

};

void
CONTROLE::gestion_globale ()
{
  SDL_ShowCursor (false);

  for (int i = 0; i < MAX_TOUCHE; i++)
    touche_simple[i] = false;
  for (int i = 0; i < MAX_SOURIS; i++)
    souris_simple[i] = false;

  SDL_Event event;
  while (SDL_PollEvent (&event))
    {
      if (premier > 1)
	switch (event.type)
	  {
	  case SDL_KEYDOWN:
	    touche_handle (&event.key.keysym, true);
	    break;
	  case SDL_KEYUP:
	    touche_handle (&event.key.keysym, false);
	    break;
	  case SDL_MOUSEMOTION:
	    if (!((event.motion.x == 100) && (event.motion.y == 100)))
	      {
		raw_x_view += event.motion.xrel;
		raw_y_view -= event.motion.yrel;

		float zplus = event.motion.xrel * (-0.01) * mouse_sensib;
		float xplus = event.motion.yrel * (0.01) * mouse_sensib;
		if (mouse_inverse)
		  xplus *= -1;
		z_view += zplus;
		x_view += xplus;
		if (x_view < -1.5)
		  x_view = -1.5;
		if (x_view > 1.5)
		  x_view = 1.5;
	      }
	    break;
	  case SDL_MOUSEBUTTONDOWN:
	    //printf("Le bouton %d de la souris a t prss en (%d,%d)\n",
	    //       event.button.button, event.button.x, event.button.y);
	    souris[event.button.button] = true;
	    souris_simple[event.button.button] = true;
	    break;
	  case SDL_MOUSEBUTTONUP:
	    // printf("Le bouton %d de la souris a t relach en (%d,%d)\n",
	    //        event.button.button, event.button.x, event.button.y);
	    souris[event.button.button] = false;
	    break;
	  }
    }
  SDL_WarpMouse (100, 100);
#ifndef linux
  SDL_PollEvent (&event);
#endif
  premier++;
  if (premier == 3)
    premier = 2;
};

bool
CONTROLE::isToucheActif (int i)
{
  if (i < 0)
    {
      if (-i < MAX_SOURIS)
	return souris[-i] || souris_simple[-i];
      else
	return false;
    }
  if (i < MAX_TOUCHE)
    return touche[i];
};


bool
CONTROLE::isToucheActifComplexe (int i)
{
  if (i < 0)
    if (-i < MAX_SOURIS)
      return souris[-i] || souris_simple[-i];
    else
      return false;
  if (i < MAX_TOUCHE)
    {
      bool ret = touche_simple[i];
      touche_simple[i] = false;
      return ret;
    }
};

bool
CONTROLE::getControleActif (int &num)
{
  for (num = -MAX_SOURIS; num != MAX_TOUCHE; num++)
    if (isToucheActifComplexe (num))
      return true;
  return false;
}

bool
CONTROLE::avance ()
{
  return isToucheActif (touche_controle[TOUCHE_AVANCE]);
}

bool
CONTROLE::recule ()
{
  return isToucheActif (touche_controle[TOUCHE_RECULE]);
}

bool
CONTROLE::saute ()
{
  return isToucheActif (touche_controle[TOUCHE_SAUTE]);
}

bool
CONTROLE::strafe_gauche ()
{
  return isToucheActif (touche_controle[TOUCHE_STRAFE_GAUCHE]);
}

bool
CONTROLE::strafe_droite ()
{
  return isToucheActif (touche_controle[TOUCHE_STRAFE_DROITE]);
}

bool
CONTROLE::tir ()
{
  return isToucheActif (touche_controle[TOUCHE_TIR]);
}

bool
CONTROLE::arme (int i)
{
  return isToucheActif (touche_controle[TOUCHE_ARME + i]);
}

bool
CONTROLE::arme_plus ()
{
  return (souris_simple[5]);
}

bool
CONTROLE::arme_moins ()
{
  return (souris_simple[4]);
}

bool
CONTROLE::action ()
{
  return isToucheActif (touche_controle[TOUCHE_ACTION]);
}

bool
CONTROLE::texte ()
{
  return isToucheActif (touche_controle[TOUCHE_TEXTE]);
}

bool
CONTROLE::vue ()
{
  return isToucheActifComplexe (touche_controle[TOUCHE_VUE]);
}

bool
CONTROLE::fullscreen ()
{
  return isToucheActifComplexe (touche_controle[TOUCHE_FULLSCREEN]);
}

bool
CONTROLE::haut ()
{
  return touche_simple[SDLK_UP];
}

bool
CONTROLE::bas ()
{
  return touche_simple[SDLK_DOWN];
}

bool
CONTROLE::gauche ()
{
  return touche_simple[SDLK_LEFT];
}

bool
CONTROLE::droite ()
{
  return touche_simple[SDLK_RIGHT];
}

bool
CONTROLE::entree ()
{
  return touche_simple[SDLK_RETURN] || souris_simple[0] || souris_simple[1];
}

char
CONTROLE::lettre_appuyee ()
{
  int retour = 0;
  for (int i = SDLK_a; i <= SDLK_z; i++)
    if (touche_simple[i])
      retour = i - SDLK_a + 'A';
  if (touche_simple[SDLK_SPACE])
    retour = ' ';
  return retour;
}

char
CONTROLE::chiffre_appuyee ()
{
  int retour = 0;
  if (touche_simple[SDLK_KP0])
    retour = '0';
  if (touche_simple[SDLK_KP1])
    retour = '1';
  if (touche_simple[SDLK_KP2])
    retour = '2';
  if (touche_simple[SDLK_KP3])
    retour = '3';
  if (touche_simple[SDLK_KP4])
    retour = '4';
  if (touche_simple[SDLK_KP5])
    retour = '5';
  if (touche_simple[SDLK_KP6])
    retour = '6';
  if (touche_simple[SDLK_KP7])
    retour = '7';
  if (touche_simple[SDLK_KP8])
    retour = '8';
  if (touche_simple[SDLK_KP9])
    retour = '9';
  if (touche_simple[SDLK_KP_PERIOD])
    retour = '.';
  return retour;
}

bool
CONTROLE::efface_appuyee ()
{
  if (touche_simple[SDLK_BACKSPACE])
    return true;
  return false;
}

bool
CONTROLE::fin_jeu ()
{
  return touche_simple[SDLK_F2];
}

float
CONTROLE::get_z_view ()
{
  return z_view;
};

float
CONTROLE::get_x_view ()
{
  return x_view;
};

float
CONTROLE::get_raw_x_view ()
{
  return raw_x_view;
};

float
CONTROLE::get_raw_y_view ()
{
  return raw_y_view;
};

MOT
CONTROLE::get_name (int numero)
{
  MOT m;
  if (numero > 0)
    m = MOT (SDL_GetKeyName ((SDLKey) numero));
  if (numero == -1)
    m = MOT ("BGAUCHE");
  if (numero == -2)
    m = MOT ("BCENTRE");
  if (numero == -3)
    m = MOT ("BDROITE");
  if (numero == -4)
    m = MOT ("MHAUT");
  if (numero == -5)
    m = MOT ("MBAS");
  m.upper ();
  return m;
}

int
CONTROLE::get_touche (MOT m)
{
  //printf("get_touche %s\n",m.info);
  for (int i = 0; i != MAX_TOUCHE; i++)
    {
      //printf("%s %s\n",m.info,MOT(SDL_GetKeyName((SDLKey)i)).info);
      MOT t = MOT (SDL_GetKeyName ((SDLKey) i));
      t.upper ();
      if (m == t)
	{
	  return i;
	}
    }
  if (m == MOT ("BGAUCHE"))
    return -1;
  if (m == MOT ("BCENTRE"))
    return -2;
  if (m == MOT ("BDROITE"))
    return -3;
  if (m == MOT ("MHAUT"))
    return -4;
  if (m == MOT ("MBAS"))
    return -5;
  return 0;
}

MOT
CONTROLE::get_avance ()
{
  return get_name (touche_controle[TOUCHE_AVANCE]);
};

MOT
CONTROLE::get_recule ()
{
  return get_name (touche_controle[TOUCHE_RECULE]);
};

MOT
CONTROLE::get_saute ()
{
  return get_name (touche_controle[TOUCHE_SAUTE]);
};

MOT
CONTROLE::get_tir ()
{
  return get_name (touche_controle[TOUCHE_TIR]);
};

MOT
CONTROLE::get_texte ()
{
  return get_name (touche_controle[TOUCHE_TEXTE]);
};

MOT
CONTROLE::get_action ()
{
  return get_name (touche_controle[TOUCHE_ACTION]);
};

MOT
CONTROLE::get_strafe_gauche ()
{
  return get_name (touche_controle[TOUCHE_STRAFE_GAUCHE]);
};

MOT
CONTROLE::get_strafe_droite ()
{
  return get_name (touche_controle[TOUCHE_STRAFE_DROITE]);
};

MOT
CONTROLE::get_arme (int i)
{
  return get_name (touche_controle[TOUCHE_ARME + i]);
};

MOT
CONTROLE::get_vue ()
{
  return get_name (touche_controle[TOUCHE_VUE]);
};

void
CONTROLE::set_avance (int i)
{
  touche_controle[TOUCHE_AVANCE] = i;
};

void
CONTROLE::set_recule (int i)
{
  touche_controle[TOUCHE_RECULE] = i;
};

void
CONTROLE::set_saute (int i)
{
  touche_controle[TOUCHE_SAUTE] = i;
};

void
CONTROLE::set_tir (int i)
{
  touche_controle[TOUCHE_TIR] = i;
};

void
CONTROLE::set_texte (int i)
{
  touche_controle[TOUCHE_TEXTE] = i;
};

void
CONTROLE::set_action (int i)
{
  touche_controle[TOUCHE_ACTION] = i;
};

void
CONTROLE::set_strafe_gauche (int i)
{
  touche_controle[TOUCHE_STRAFE_GAUCHE] = i;
};

void
CONTROLE::set_strafe_droite (int i)
{
  touche_controle[TOUCHE_STRAFE_DROITE] = i;
};

void
CONTROLE::set_vue (int i)
{
  touche_controle[TOUCHE_VUE] = i;
};

void
CONTROLE::set_arme (int arme, int i)
{
  touche_controle[TOUCHE_ARME + arme] = i;
};

void
CONTROLE::set_sensibilite (float val)
{
  mouse_sensib = val;
};

float
CONTROLE::get_sensibilite ()
{
  return mouse_sensib;
};

void
CONTROLE::set_inverse (bool inv)
{
  mouse_inverse = inv;
};

bool
CONTROLE::get_inverse ()
{
  return mouse_inverse;
};
