/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: def_3d.h,v 1.3 2006/12/21 18:18:14 atila Exp $

#ifndef def_3d_fichier_const
#define def_3d_fichier_const

#include "bloc_info.h"
#include "classe_modele.h"
#include "facette.h"
#include "lumiere.h"
#include "def_texture.h"
#include "animation_texture.h"
#include <GL/gl.h>

typedef vector < int >LISTE_INDICE_PORTAL;

/********************************************
*	Pour grer une liste de portal
*
* On dfinit ici une nomenclature pour le nommage 
* des objets  l'intrieur des portals
* [ANIMATION:]OBJET
* ANIMATION peut tre:
*   - NOM, (qui doit tre enregistr ainsi dans
*              la salle).
*   - +/- pour la salle suivante ou prcdente.
* OBJET peut tre:
*   - NOM (qui doit tre enregistr ainsi dans
*              l'objet).
*   - +/- pour l'objet suivant ou prcdent.
*
* La rsolution des noms se fait  la fin du 
* chargement des niveaux,1 seule fois. On peut 
* donc rfrencer des objets qui seront chargs 
* plus loin.
********************************************/

class FACETTE_PORTAL;
typedef FACETTE_PORTAL *LPFACETTE_PORTAL;

#include "moteur3d.h"


// *************************************************
// La classe OBJET proprement dite...
// *************************************************

class MOTEUR;

class OBJET:public CLASSE_MODELE
{
  // Rfrence des calculs de lumire.
  // ces calculs sont effectus par Animation.
  // Une fois le calcul effectu, certains sont utiliss , d'autres pas.
  INFO_LUMIERE info_lumiere;
  VALEUR_LUMIERE valeur_lumiere;

  // Les paramtres de couleur finaux...
  // Ils sont appliqus sur chaque facette, en faisant un glColor4f local
  // avant appel de liste.
  VALEUR_LUMIERE valeur_lumiere_locale;

  // variables de calculs internes
  // pour les calculs d'animation
  BOOL aligne_camera;
  BOITE boite;

  MOT nom;
  MOT nom_pere;
  int index_pere;

  int activation, lactivation;

  LPOBJET lpparent;
  LPOBJET lpmodele;

  LISTE_FACETTE lf_statique;

  GLuint GL_statique;

  // La liste des facettes translucides  afficher en fin de passage.
  // NB : elles sont dans lf_statique, mais pas dans GL_statique. [ ne pas dsallouer ]
  //      elles ne portent qu'un clairage complet (pas de rond dessus).
  LISTE_FACETTE lf_translucide;

  // Une liste pour les facettes intermdiaire  temps de vie limit
  LISTE_FACETTE_IMPACT lf_dynamique;

  // Une autre liste pour les lumires dynamiques.
  LISTE_FACETTE_IMPACT lf_dynamique_lumiere;
  // Une autre liste pour les lumires dynamiques sur les facettes transparentes.
  LISTE_FACETTE_IMPACT lf_dynamique_lumiere_transparente;

  // La liste des portal
  LISTE_PORTAL l_portal;

  // texture  utiliser au dbut de l'affichage de l'objet.
  // utilis pour donner sa texture aux objets joueurs.
  GLuint texture_base;

  // Routine de trac rel.
  void trace_objet_base ();

public:

  // Dernire frame avec calcul des lumieres.
  // Utilis pour connaitre les objets rentrs dans le champs de vision depuis la dernire
  // frame.
  int frame_lumiere;

  void ajoute_facette_proches_liste (RWVECTOR point,
				     float distance,
				     LPLISTE_LPFACETTE_PROCHE liste,
				     LISTE_FACETTE * lpliste,
				     LPANIMATION lpanim);


  // Utilis pour la sauvegarde du contexte avec le mix.
  MATRIX sauve_local;


  // Variables utiles
  MATRIX lrepere_global;
  MATRIX repere_global;

  // La matrice OpenGL ki-va-bien.
  MATRIX repere_global_OpenGL;

  MATRIX lrepere_global_inverse;
  MATRIX repere_global_inverse;

  MATRIX repere_local;
  bool calcul;
  bool calcul_inverse;

  // Se rappeler si c'est un portal
  bool is_portal;

  // Se rappeler si c'est la camro pour le mode contraint.
  bool is_camera;

    OBJET ();
  void init ();
  void charge_fichier (MOT * fichier, LPBLOCINFO lpblocinfo);
  void charge_fichier (BUFFER_NOEUD & BF, buffer_noeud * bf,
		       LPBLOCINFO lpblocinfo);
    OBJET (MOT * fichier, LPBLOCINFO lpblocinfo);
    OBJET (BUFFER_NOEUD & BF, buffer_noeud * bf, LPBLOCINFO lpblocinfo);
  LPOBJET copie_objet ();

  // pour tracer. 
  // La premire enregistre les objets dans le moteur 3D,
  // la seconde trace les objets fixes
  // la 3ime s'occupe de la transparence.
  void fin_animation (LPBLOCINFO);
  void trace_objet_final (LPBLOCINFO);
  void trace_objet_final_transparent (LPBLOCINFO);

   ~OBJET ();

  void set_index_pere (int i);

  void set_nom (MOT * m);
  void set_nom_pere (MOT * m);

  MOT *get_nom ();
  MOT *get_nom_pere ();
  int get_index_pere ();

  void set_lpparent (LPOBJET);

  void calcul_parametres_objet ();
  void ajoute_facettes_proches (RWVECTOR point,
				float distance,
				LPLISTE_LPFACETTE_PROCHE liste,
				LPANIMATION lpanim);

  // Pour les mix.
  void pre_mix ();
  void mix (float t);

  // Pour les impacts de balle.
  // vrai/faux si impact. a et b pour le tir. normale pour la normale
  bool impact (RWVECTOR & a, RWVECTOR & b, LPFACETTE * touche);
  void calcul_matrices_inverse ();

  // Pour la reconnaissance et la recopie. VOIR classe_modele.h
  int charge_avec_fichier (void *data, LPBLOCINFO lpblocinfo);
  CLASSE_MODELE *duplique_element ();

  // Crer les facettes d'impacts sphriques.
  // bool : renvoie si il y a eu des impacts crs.
  // distance pour le rayon de la sphre.
  // distance_facette pour la distance de la facette_impact  la facette.
  // ID pour l'identifiant d'impact qui servira pour appeler les callbacks.
  // temps pour le temps de vie max des facette_impacts.
  bool impact_sphere (RWVECTOR & point, float distance,
		      float distance_facette, int ID, float temps);
  void set_T (float diff);
  void set_dynamique (int ID, MATRIX & m, GLuint text_desc);

  // Nettoyer les facettes d'impacts sphriques lumineux avant le calcul.
  void nettoie_impact_sphere_lumiere ();
  // appliquer la lumiere si besoin est.
  // source est utilis pour calculer la couleur, l'intensit et d'autres paramtres de lumiere...
  void lumiere (LPOBJET source, TypeLumiere type_lumiere);

  friend class ANIMATION;
  friend class ELEMENT_ACTION;

  ZONE_LUMIERE get_zone_lumiere ();
  bool valeur_lumiere_isNul ();

  // Appel par animation pour initialiser la valeur de la lumiere locale  0.
  void init_lumiere ();
  void fin_lumiere ();
  void set_texture_objet (GLuint no_texture);

  // Connexion au moteur 3D s'il possede des portals.
  void connecte_moteur3d (MOT nom_animation, LPBLOCINFO lpblocinfo);
  // utilise par le moteur 3D pour determiner si un portal est contenu.
    BOITE & get_boite ();

  void ajoute_lumiere_blanche ();

  // Stocke la liste des portals auxquels l'objet appartient. Cette structure est modifiee par
  // le moteur 3D directement. Cela permet une optimisation en temps en vitznt de
  // recalculer a chaque frame la liste.
  LISTE_INDICE_PORTAL indice_portal;
  // Le temps avant rafraichissement. (peut-tre modifi par l'objet en fonction de son deplacement.
  // Sinon il est plac par le moteur 3D une constante, une fois le refresh effectue.
  float refreshTime;

  friend class MOTEUR;

};

typedef vector < LPOBJET > LISTE_LPOBJET;

#endif
