/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: facette.cpp,v 1.4 2006/12/21 18:18:14 atila Exp $

#define GL_GLEXT_PROTOTYPES
#include "facette.h"

#include "glext.h"
#include "win3d.h"

/************************ This is a fast allocator/deallocator for VERTEXes. ********/

#define MAX_VERTEX_ALLOC 1000

vector < VERTEX * >fast_vertex_alloc;
int vertex_alloc_size = 0;

VERTEX *
newVERTEX (const VERTEX & v)
{
  if (vertex_alloc_size)
    {
      vertex_alloc_size--;
      *(fast_vertex_alloc[vertex_alloc_size]) = v;
      return fast_vertex_alloc[vertex_alloc_size];
    }
  else
    return new VERTEX (v);
};

void
deleteVERTEX (VERTEX * src)
{
  while (fast_vertex_alloc.size () < MAX_VERTEX_ALLOC)
    fast_vertex_alloc.insert (fast_vertex_alloc.end (), NULL);
  if (vertex_alloc_size != MAX_VERTEX_ALLOC)
    {
      fast_vertex_alloc[vertex_alloc_size] = src;
      vertex_alloc_size++;
    }
  else
    delete src;
}

/************************ GENCOORDTEX *******************************************/

GENCOORDTEX::GENCOORDTEX ()
{
};

GENCOORDTEX::~GENCOORDTEX ()
{
};

// C'est l qu'est toute l'intelligence.
void
GENCOORDTEX::calcul_param (VERTEX & a, VERTEX & b, VERTEX & c, float ScaleTX,
			   float ScaleTY)
{
  // Regarder le nombre de Coord donnes. 
  int coord = 0;
  if (a.get_isinfocoord ())
    coord++;
  if (b.get_isinfocoord ())
    coord++;
  if (c.get_isinfocoord ())
    coord++;
  RWVECTOR A = a.get_point ();
  RWVECTOR B = b.get_point ();
  RWVECTOR C = c.get_point ();
  PLAN P;
  P.calcul_PLAN (A, B, C);
  RWVECTOR N = P.u;
  // Si coord=3 alors se calquer sur les points, sinon calculer leur coord tex manquante
  // On procde incmentalement. On suppose que les premiers points sont donns.
  // ATTENTION : volontairement pas de break;


  switch (coord)
    {
    case 0:
      {
	a.set_infocoord (0, 0);
      };
    case 1:
      {
	float dist = (B - A).norme ();
	b.set_infocoord (dist * ScaleTX + a.get_TX (), a.get_TY ());
      };
    case 2:
      {
	RWVECTOR AB = B - A;
	RWVECTOR AC = C - A;
	float TXc = (AC.produit_scalaire (AB));
	TXc = TXc / AB.norme () / AB.norme ();
	TXc = TXc * (b.get_TX () - a.get_TX ());
	RWVECTOR AP =
	  AB * ((AB.produit_scalaire (AC) / (AB.norme () * AB.norme ())));
	RWVECTOR PC = AC - AP;
	float TYc = PC.norme () * ScaleTY + (a.get_TY ());
	c.set_infocoord (TXc, TYc);
      };
    };

  // Puis appliquer la mthode.
  RWVECTOR AB = B - A, AC = C - A, BNA = (B + N) - A;
  MATRIX M (AB, AC, BNA);
  MATRIX M2 = M.inverse ();

  float Txa = a.get_TX ();
  float Txb = b.get_TX ();
  float Txc = c.get_TX ();

  float Tya = a.get_TY ();
  float Tyb = b.get_TY ();
  float Tyc = c.get_TY ();

  RWVECTOR ax (Txb - Txa, Txc - Txa, Txb - Txa);
  RWVECTOR ay (Tyb - Tya, Tyc - Tya, Tyb - Tya);

  Px.u = M2.transforme (ax);
  Px.P = Px.u * (Txa - Px.u.produit_scalaire (A));
  Px.P = Px.P * (1 / (Px.u.produit_scalaire (Px.u)));

  Py.u = M2.transforme (ay);
  Py.P = Py.u * (Tya - Py.u.produit_scalaire (A));
  Py.P = Py.P * (1 / (Py.u.produit_scalaire (Py.u)));

  // Impressive, isn't it ?

}

void
GENCOORDTEX::autogencoord (VERTEX & a)
{
  RWVECTOR p = a.get_point ();
  a.set_infocoord (Px.CalcVal (p), Py.CalcVal (p));
  //printf("POINT %f %f %f ---- %f %f \n",p.x(),p.y(),p.z(),a.get_TX(),a.get_TY());
};



/************************ PROPRIETE_FACETTE *******************************************/

PROPRIETE_FACETTE::PROPRIETE_FACETTE ()
{
  type_matiere_facette = MATIERE_PIERRE;
  k_facette = 500;
  k_adherence = 0.8;
  k_amortissement_x = 0.9;
  k_amortissement_z = 0.9;

  // Dans ce cas-l, on accroche bien au sol.
  k_amortissement_mouvement = 1;
};

PROPRIETE_FACETTE::~PROPRIETE_FACETTE ()
{
};

void
PROPRIETE_FACETTE::lecture_propriete_facette (BUFFER_NOEUD * BF,
					      buffer_noeud * bn)
{
};

/************************ ACTION_FACETTE **********************************************/

ACTION_FACETTE::ACTION_FACETTE ()
{
  // a priori, on va s'initialiser de la manire la plus minimale possible
  type_facette = TYPE_FACETTE_STANDARD;
};


ACTION_FACETTE::~ACTION_FACETTE ()
{
}

void
ACTION_FACETTE::actionne (LPCONTEXT_OBJET lpcontext)
{
  switch (consequence)
    {
    case lance_animation:
      lpcontext->lpaction_salle = &animation_salle;
      break;
    }
}

void
ACTION_FACETTE::applique_action_contact (LPCONTEXT_OBJET lpcontext)
{
  lpcontext->lpaction_salle = NULL;

  // Vrification des conditions initiales.
  if (type_facette == TYPE_FACETTE_STANDARD)
    return;
  if (!((par_objet == lpcontext->type)))
    return;
  if (!(par_action == contact))
    return;
  switch (type_facette)
    {
    case TYPE_FACETTE_ACTION:
      if (par_objet == lpcontext->type)
	actionne (lpcontext);
      break;
    }
}

void
ACTION_FACETTE::applique_action_proximite (LPCONTEXT_OBJET lpcontext, float d)
{
  lpcontext->lpaction_salle = NULL;
  if (type_facette == TYPE_FACETTE_STANDARD)
    return;
  if (par_objet != lpcontext->type)
    return;
  if ((par_action == activation) && (!(lpcontext->action)))
    return;
  if ((par_action == presence) && (d > distance))
    return;
  switch (type_facette)
    {
    case TYPE_FACETTE_ACTION:
      actionne (lpcontext);
      break;
    }
}

void
ACTION_FACETTE::lecture_action_facette (BUFFER_NOEUD * BF, buffer_noeud * bn)
{
  type_facette = TYPE_FACETTE_STANDARD;

  if (BF->get_nom (bn) == CONSTANTE_SCRIPT_FACETTESTANDARD)
    type_facette = TYPE_FACETTE_STANDARD;
  else if (BF->get_nom (bn) == CONSTANTE_SCRIPT_FACETTEACTION)
    {

      bn = BF->get_fils (bn);

      type_facette = TYPE_FACETTE_ACTION;

      par_action = (DECLENCHE_FACETTE_ACTION) - 1;
      par_objet = (DECLENCHE_FACETTE_PAR) - 1;
      consequence = (ACTION_FACETTE_NOM) - 1;

      buffer_noeud *bns;

      if (BF->trouve_suivant (bn, CONSTANTE_SCRIPT_DECLENCHECONTACT))
	par_action = contact;
      if ((bns =
	   BF->trouve_suivant (bn,
			       CONSTANTE_SCRIPT_DECLENCHEPRESENCE)) != NULL)
	{
	  par_action = presence;
	  distance = BF->get_float (bns);
	}

      if ((bns =
	   BF->trouve_suivant (bn,
			       CONSTANTE_SCRIPT_DECLENCHEACTIVATION)) != NULL)
	{
	  par_action = activation;
	  distance = BF->get_float (bns);
	}

      if (BF->trouve_suivant (bn, CONSTANTE_SCRIPT_PARMOBILE))
	par_objet = elementjeumobile;
      if (BF->trouve_suivant (bn, CONSTANTE_SCRIPT_PARJOUEUR))
	par_objet = joueur;
      if (BF->trouve_suivant (bn, CONSTANTE_SCRIPT_PARENNEMI))
	par_objet = ennemi;
      if (BF->trouve_suivant (bn, CONSTANTE_SCRIPT_PARINACTIF))
	par_objet = elementinactif;

      if ((bns =
	   BF->trouve_suivant (bn, CONSTANTE_SCRIPT_ACTIONPOUSSE)) != NULL)
	{
	  consequence = pousse;
	  force = VERTEX (BF, bns).get_point ();
	}
      if ((bns =
	   BF->trouve_suivant (bn, CONSTANTE_SCRIPT_ACTIONANIMATION)) != NULL)
	{
	  animation_salle.lecture_action (BF, BF->get_fils (bns));
	  consequence = lance_animation;
	}
      if ((bns = BF->trouve_suivant (bn, CONSTANTE_SCRIPT_ACTIONVIE)) != NULL)
	{
	  consequence = danger;
	  nb_pt_vie = BF->get_float (bns);
	}

      //printf("par action %i par consquence %i %i\n",par_action,par_objet,consequence);

    }
  else if (BF->get_nom (bn) == CONSTANTE_SCRIPT_FACETTEGRAVITE)
    {
      bn = BF->get_fils (bn);
    }
}

/************************ VERTEX **********************************************/
VERTEX::~VERTEX ()
{
}

VERTEX::VERTEX (float x, float y, float z, float tx, float ty)
{
  point = RWVECTOR (x, y, z);
  texcoord[0].TX = tx;
  texcoord[0].TY = ty;
  texcoord[0].coord = true;
  for (int i = 1; i != NOMBRE_TEXCOORD; i++)
    texcoord[i].coord = false;
};

VERTEX::VERTEX (float x, float y, float z)
{
  point = RWVECTOR (x, y, z);
  for (int i = 0; i != NOMBRE_TEXCOORD; i++)
    texcoord[i].coord = false;
};

VERTEX::VERTEX (BUFFER_NOEUD * BF, buffer_noeud * bn)
{
  float x, y, z;
  float tx, ty;
  int coordTex = 0;
  // Lecture des paramtres  partir d'un NoeudBuffer
  buffer_noeud *bns;
  bns = BF->trouve_suivant (bn, CONSTANTE_SCRIPT_X);
  if (bns != NULL)
    x = BF->get_float (bns);

  bns = BF->trouve_suivant (bn, CONSTANTE_SCRIPT_Y);
  if (bns != NULL)
    y = BF->get_float (bns);

  bns = BF->trouve_suivant (bn, CONSTANTE_SCRIPT_Z);
  if (bns != NULL)
    z = BF->get_float (bns);

  bns = BF->trouve_suivant (bn, CONSTANTE_SCRIPT_TX);
  if (bns != NULL)
    {
      tx = BF->get_float (bns);
      coordTex++;
    }

  bns = BF->trouve_suivant (bn, CONSTANTE_SCRIPT_TY);
  if (bns != NULL)
    {
      ty = BF->get_float (bns);
      coordTex++;
    }

  point = RWVECTOR (x, y, z);
  texcoord[0].TX = tx;
  texcoord[0].TY = ty;
  texcoord[0].coord = (coordTex == 2);
  for (int i = 1; i != NOMBRE_TEXCOORD; i++)
    texcoord[i].coord = false;
};

RWVECTOR
VERTEX::get_point ()
{
  return point;
};

void
VERTEX::trace_point ()
{
  for (int i = 0; i != NOMBRE_TEXCOORD; i++)
    if (texcoord[i].coord)
      glMultiTexCoord2fARB_local (GL_TEXTURE0_ARB + i, texcoord[i].TX,
				  texcoord[i].TY);
  VECTOR tvect = point.T_VECTOR ();
  glVertex3f (tvect.x, tvect.y, tvect.z);
}

VERTEX::VERTEX (RWVECTOR v, float tx, float ty)
{
  point = v;
  texcoord[0].TX = tx;
  texcoord[0].TY = ty;
  texcoord[0].coord = true;
  for (int i = 1; i != NOMBRE_TEXCOORD; i++)
    texcoord[i].coord = false;
};

VERTEX::VERTEX (RWVECTOR v)
{
  point = v;
  for (int i = 0; i != NOMBRE_TEXCOORD; i++)
    texcoord[i].coord = false;
};

bool
VERTEX::get_isinfocoord ()
{
  return texcoord[0].coord;
};

float
VERTEX::get_TX ()
{
  return texcoord[0].TX;
};

float
VERTEX::get_TY ()
{
  return texcoord[0].TY;
};

void
VERTEX::set_infocoord (float iX, float iY)
{
  texcoord[0].TX = iX;
  texcoord[0].TY = iY;
  texcoord[0].coord = true;
};

void
VERTEX::set_infocoord (float tx, float ty, int i)
{
  if ((i < NOMBRE_TEXCOORD) && (i >= 0))
    {
      texcoord[i].TX = tx;
      texcoord[i].TY = ty;
      texcoord[i].coord = true;
    };
};


void
VERTEX::set_decoupage_coord (float x, VERTEX & v1, VERTEX & v2)
{
  for (int i = 0; i != NOMBRE_TEXCOORD; i++)
    {
      if (v1.texcoord[i].coord)
	{
	  texcoord[i].coord = true;
	  texcoord[i].TX =
	    v1.texcoord[i].TX + x * (v2.texcoord[i].TX - v1.texcoord[i].TX);
	  texcoord[i].TY =
	    v1.texcoord[i].TY + x * (v2.texcoord[i].TY - v1.texcoord[i].TY);
	}
      else
	texcoord[i].coord = false;
    }
};

/************************ FACETTE **********************************************/

BOOL
FACETTE::interaction_base (RWVECTOR & a, RWVECTOR & b, float d)
{
  // calcul de l'intersection droite/plan
  if (action.type_facette != TYPE_FACETTE_STANDARD)
    return false;
  if (plan.COTE (a, d) && (!(plan.COTE (b, d))))
    {
      RWVECTOR impact;
      if (plan.intersection (a, b, d, &impact))
	{
	  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end ();
	       i++)
	    {
	      if (((*i)->plan.COTE (impact, 0.0)))
		{
		  return false;
		}
	    };
	  b = impact;
	  return true;
	}
    }
  return false;
};

BOOL
  FACETTE::interaction_exp (RWVECTOR & A, RWVECTOR & B, float &C,
			    RWVECTOR & D, RWVECTOR & P0, RWVECTOR & P1,
			    float &delta_t, float &rayon)
{
  float delta_t2 = delta_t;
  RWVECTOR Pi = P1;
  if (plan.intersection_expo (A, B, C, D, rayon, P0, Pi, delta_t2))
    {
      for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
	{
	  if (((*i)->plan.COTE (Pi, 0.0)))
	    {
	      return false;
	    };
	};
      P1 = Pi;
      delta_t = delta_t2;
      return true;
    };
  return false;
};

BOOL
  FACETTE::interaction (RWVECTOR a, RWVECTOR b, float d, RWVECTOR * impact,
			RWVECTOR * devie)
{
  // calcul de l'intersection droite/plan
//  printf("action.type_facette %i       %i %i\n",action.type_facette,plan.COTE(a,d),plan.COTE (b, d));
  if (action.type_facette != TYPE_FACETTE_STANDARD)
    return false;
  if (plan.COTE (a, d) && (!(plan.COTE (b, d))))
    {
      if (plan.intersection (a, b, d, impact))
	{
	  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end ();
	       i++)
	    {
	      if (((*i)->plan.COTE (*impact, 0.0)))
		{
		  return false;
		}
	    };
	  RWVECTOR ubis = plan.u * 0.0001;
	  *impact = (*impact) + ubis;
	  RWVECTOR p = plan.PROJECTION (b, d);
	  p = p + ubis;
	  *devie = p;
	  return true;
	}
    }
  return false;
};

void
FACETTE::calcul_plan ()
{
  RWVECTOR pa, pb, pc, pu, pab;
  LISTE_VERTEX::iterator t = liste.begin ();
  if (liste.size () < 3)
    return;
  pc = (*t)->get_point ();
  t = t + 1;
  pb = (*t)->get_point ();
  t = t + 1;
  pa = (*t)->get_point ();
  plan.calcul_PLAN (pa, pb, pc);
  for (t = liste.begin (); t != liste.end (); t = t + 1)
    {
      LISTE_VERTEX::iterator t2 = t;
      pb = (*t2)->get_point ();
      t2 = t2 + 1;
      if (t2 == liste.end ())
	t2 = liste.begin ();
      pa = (*t2)->get_point ();
      pab = pb - pa;
      pab = pab.normalise ();
      pu.produit_vectoriel (plan.u, pab);
      pc = pa + plan.u + (pu * (-1.0));
      (*t)->plan.calcul_PLAN (pa, pb, pc);
    }
  pente = plan.u.produit_scalaire (RWVECTOR (0, 0, 1));
};

void
FACETTE::calcul_BOX ()
{
  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      boite.MAJ ((*i)->get_point ());
    };
}

float
FACETTE::distance_boite (RWVECTOR & p)
{
  return boite.distance (p);
};

void
FACETTE::calcul_marche ()
{

}

void
FACETTE::calcul_parametres_solides ()
{
  calcul_plan ();
  calcul_BOX ();
  calcul_marche ();
  transfo_zu.genere_avec_u (plan.u);
};


BOITE
FACETTE::get_boite ()
{
  return boite;
}

FACETTE::FACETTE ()
{
};

FACETTE::FACETTE (BUFFER_NOEUD * BF, buffer_noeud * bn,
		  ETAT_MULTITEXTURE & em, float TexGenX, float TexGenY)
{

  texture = em;

  // Lecture des paramtres  partir d'un NoeudBuffer
  while (bn)
    {
      //printf("FACETTE -- %i\n",BF->get_nom (bn));
      if (BF->get_nom (bn) == CONSTANTE_SCRIPT_POINT)
	{
	  liste.insert (liste.end (),
			newVERTEX (VERTEX (BF, BF->get_fils (bn))));
	}
      else if (BF->get_nom (bn) == CONSTANTE_SCRIPT_FACETTESTANDARD)
	{
	  action.lecture_action_facette (BF, bn);
	}
      else if (BF->get_nom (bn) == CONSTANTE_SCRIPT_FACETTEACTION)
	{
	  action.lecture_action_facette (BF, bn);
	}
      else if (BF->get_nom (bn) == CONSTANTE_SCRIPT_FACETTEGRAVITE)
	{
	  action.lecture_action_facette (BF, bn);
	};
      bn = BF->trouve_suivant (bn);
    }
  //printf("%i\n",liste.size());
  if (liste.size () < 3)
    return;
  // Gnration automatique des coordonnes.
  LISTE_VERTEX::iterator i = liste.begin ();
  LPVERTEX a = *i++;
  LPVERTEX b = *i++;
  LPVERTEX c = *i++;
  gencoordtex.calcul_param (*a, *b, *c, TexGenX, TexGenY);
  int k = 0;

  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      //printf("%i Avant %i\n",k,(*i)->get_isinfocoord());
      if (!((*i)->get_isinfocoord ()))
	gencoordtex.autogencoord (**i);
      //printf("%i Apres %i %f %f \n",k,(*i)->get_isinfocoord(),(*i)->get_TX(),(*i)->get_TY());
      k++;
    };
};

void
FACETTE::trace_facette (ETAT_TEXTURE * etat_texture)
{
  etat_texture->active_etat (0, texture);
  glBegin (GL_POLYGON);
  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      (*i)->trace_point ();
    };
  glEnd ();
}

FACETTE::~FACETTE ()
{
  // On dtruit les points.
  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      deleteVERTEX (*i);
    };
  liste.clear();
};

void
FACETTE::applique_action_proximite (LPCONTEXT_OBJET lpcontext, RWVECTOR & P)
{
  action.applique_action_proximite (lpcontext, distance_boite (P));
};

RWVECTOR
FACETTE::get_u ()
{
  return plan.u;
}

MATRIX
FACETTE::get_transfo_zu ()
{
  return transfo_zu;
};

bool
  FACETTE::impact_sphere (RWVECTOR & point, float distance,
			  float distance_facette, FACETTE_IMPACT ** resultat)
{
  if (boite.distance (point) <= distance)
    {
      // Cration des points de la projection.
      RWVECTOR projection = plan.PROJECTION (point, distance_facette);
      // Distance entre ces 2 points.
      float dist_points = projection.distance (point);
//      printf(" %f %f ---%f %f %f ---- %f %f %f\n",distance,dist_points,
//                                                      point.x(),point.y(),point.z(),
//                                                      projection.x(),projection.y(),projection.z());
      if (distance > dist_points)
	{
	  float r = sqrt (distance * distance - dist_points * dist_points);
	  // 4 points  gnrer :
	  RWVECTOR P1 (-r, -r, distance_facette);
	  RWVECTOR P2 (-r, r, distance_facette);
	  RWVECTOR P3 (r, r, distance_facette);
	  RWVECTOR P4 (r, -r, distance_facette);
	  P1 = transfo_zu.transforme (P1) + projection;
	  P2 = transfo_zu.transforme (P2) + projection;
	  P3 = transfo_zu.transforme (P3) + projection;
	  P4 = transfo_zu.transforme (P4) + projection;
	  // Insertion des points dans une nouvelle facette...
	  FACETTE_IMPACT *f;

	  f = new FACETTE_IMPACT (plan, this);
	  VALEUR_LUMIERE vl (1, 1, 1, dist_points / distance);
	  f->set_couleur (vl);
	  LPVERTEX lpvertex;
	  lpvertex = newVERTEX (VERTEX (P1));
	  gencoordtex.autogencoord (*lpvertex);
	  lpvertex->set_infocoord (0.0, 0.0, 1);
	  f->ajoute_vertex (lpvertex);

	  lpvertex = newVERTEX (VERTEX (P2));
	  gencoordtex.autogencoord (*lpvertex);
	  lpvertex->set_infocoord (0.0, 1.0, 1);
	  f->ajoute_vertex (lpvertex);

	  lpvertex = newVERTEX (VERTEX (P3));
	  gencoordtex.autogencoord (*lpvertex);
	  lpvertex->set_infocoord (1.0, 1.0, 1);
	  f->ajoute_vertex (lpvertex);

	  lpvertex = newVERTEX (VERTEX (P4));
	  gencoordtex.autogencoord (*lpvertex);
	  lpvertex->set_infocoord (1.0, 0.0, 1);
	  f->ajoute_vertex (lpvertex);

	  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end ();
	       i++)
	    {
	      f->troncature ((*i)->plan);
	    };

	  *resultat = f;
	  return true;
	}
    }
  return false;
};

PROPRIETE_FACETTE *
FACETTE::get_propriete_facette ()
{
  return &propriete_facette;
};

/************************ FACETTE_IMPACT ****************************/

void
FACETTE_IMPACT::ajoute_vertex (LPVERTEX lpvertex)
{
  liste.insert (liste.end (), lpvertex);
  boite.MAJ (lpvertex->point);
};

FACETTE_IMPACT::FACETTE_IMPACT (PLAN p, LPFACETTE lpf):plan (p),
facette_source (lpf)
{
  // A detruire immdiatement si non fix !
  Tmax = 0;
  T = 0;
  couleur = false;
  couleur_princ.R = 1;
  couleur_princ.G = 1;
  couleur_princ.B = 1;
  couleur_princ.A = 0;
  couleur_lprinc.R = 0;
  couleur_lprinc.G = 0;
  couleur_lprinc.B = 0;
  couleur_lprinc.A = 0;
}

FACETTE_IMPACT::~FACETTE_IMPACT ()
{
  // Dtruire la liste de points...
  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      deleteVERTEX (*i);
    };
}

bool
FACETTE_IMPACT::test_ID (int tID)
{
  return (ID == tID);
}

void
FACETTE_IMPACT::set_nom (int tID)
{
  ID = tID;
}

void
FACETTE_IMPACT::set_info (MATRIX & m, GLuint desc)
{
  texture.texture = desc;
  text_decal = m;
}

void
FACETTE_IMPACT::set_info_desc (GLuint desc)
{
  texture.texture = desc;
}

void
FACETTE_IMPACT::trace_facette (ETAT_TEXTURE * etat_texture,
			       LPBLOCINFO lpblocinfo)
{
  glMatrixMode (GL_TEXTURE);
  glLoadIdentity ();
  glMatrixMode (GL_MODELVIEW);

  if (facette_source)
    {
      etat_texture->active_etat (0, facette_source->texture);
    };
  if (texture.texture)
    {
      etat_texture->active_etat (1, texture);
    };

  if (couleur)
    {
      VALEUR_LUMIERE coul = couleur_princ + couleur_lprinc;
      //printf("R %f G %f B %f A %f \n",coul.R,coul.G,coul.B,coul.A);
      lpblocinfo->lpetat_texture->set_color (1, coul);
    }
  glBegin (GL_POLYGON);
  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      (*i)->trace_point ();
    };
  glEnd ();
}

void
FACETTE_IMPACT::set_couleur (VALEUR_LUMIERE & vl)
{
  couleur_princ = vl;
  couleur = true;
}

void
FACETTE_IMPACT::set_couleur_RGB (VALEUR_LUMIERE & vl)
{
  couleur_princ.R = vl.R;
  couleur_princ.G = vl.G;
  couleur_princ.B = vl.B;
  couleur = true;
}

void
FACETTE_IMPACT::set_Tmax (float t)
{
  Tmax = t;
}

bool
FACETTE_IMPACT::set_T (float delta)
{
  T += delta;
  return (T > Tmax);
}

void
FACETTE_IMPACT::troncature (PLAN & p)
{
  LISTE_VERTEX liste2;

  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      LPVERTEX v1 = *i;
      LPVERTEX v2;
      if ((i + 1) == liste.end ())
	v2 = (*(liste.begin ()));
      else
	v2 = (*(i + 1));
      RWVECTOR p1 = v1->point;
      RWVECTOR p2 = v2->point;
      if (!(p.COTE (p1, 0)))
	{
	  liste2.insert (liste2.end (), newVERTEX (*v1));
	  if ((p.COTE (p2, 0)))
	    {
	      // Calcul d'intersection, etc...
	      RWVECTOR inter;
	      float k;
	      p.intersection (p1, p2, &inter, k);
	      // Dcoupage des textures en fonction de k.
	      LPVERTEX v3 = newVERTEX (VERTEX (inter, 0, 0));
	      liste2.insert (liste2.end (), v3);
	      v3->set_decoupage_coord (k, *v1, *v2);
	    }
	}
      else
	{
	  if (!(p.COTE (p2, 0)))
	    {
	      // Calcul d'intersection, etc...
	      RWVECTOR inter;
	      float k;
	      p.intersection (p1, p2, &inter, k);
	      LPVERTEX v3 = newVERTEX (VERTEX (inter, 0, 0));
	      v3->set_decoupage_coord (k, *v1, *v2);
	      liste2.insert (liste2.end (), v3);
	    }
	}
    };
  // Destruction de liste.



  for (LISTE_VERTEX::iterator i = liste.begin (); i != liste.end (); i++)
    {
      deleteVERTEX (*i);
    };
  liste.erase (liste.begin (), liste.end ());
  for (LISTE_VERTEX::iterator i = liste2.begin (); i != liste2.end (); i++)
    {
      liste.insert (liste.end (), *i);
    };
  liste2.erase (liste2.begin (), liste2.end ());

}

void
FACETTE_IMPACT::reset_couleur_lumiere ()
{
  couleur_lprinc.R = 0;
  couleur_lprinc.G = 0;
  couleur_lprinc.B = 0;
  couleur_lprinc.A = 0;

  couleur_princ.R = 0;
  couleur_princ.G = 0;
  couleur_princ.B = 0;
  couleur_princ.A = 0;

  couleur = true;
  // on en profite pour annuler la facette de base (car ce n'est pas une facette lumiere)
  facette_source = NULL;
};

void
FACETTE_IMPACT::add_couleur_lumiere (VALEUR_LUMIERE & vl)
{
  couleur_lprinc = couleur_lprinc + vl;
};

FACETTE_IMPACT::FACETTE_IMPACT (LPFACETTE orig)
{
  for (int i = 0; i != orig->liste.size (); i++)
    {
      ajoute_vertex (newVERTEX (*(orig->liste[i])));
    }
  Tmax = 0;
  T = 0;
  facette_source = orig;
  couleur = false;
  couleur_princ.R = 1;
  couleur_princ.G = 1;
  couleur_princ.B = 1;
  couleur_princ.A = 0;
  couleur_lprinc.R = 1;
  couleur_lprinc.G = 1;
  couleur_lprinc.B = 1;
  couleur_lprinc.A = 0;
};

RWVECTOR
FACETTE_IMPACT::centre ()
{
  return boite.centre ();
};

void
FACETTE::ajoute_vertex (LPVERTEX lpvertex)
{
  liste.insert (liste.end (), lpvertex);
  boite.MAJ (lpvertex->point);
};
