/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: facette.h,v 1.3 2006/12/21 18:18:14 atila Exp $

#ifndef FACETTE_FICHIER_INCLUDE
#define FACETTE_FICHIER_INCLUDE

using namespace std;

#include "mathsRWvsNDJ.h"
#include "action_salle.h"
#include "buffer_noeud.h"
#include "script_nom.h"
#include "def_texture.h"
#include <deque>
#include <GL/gl.h>
#include "context_objet.h"



class ACTION_FACETTE
{
  DECLENCHE_FACETTE_ACTION par_action;
  DECLENCHE_FACETTE_PAR par_objet;
  ACTION_FACETTE_NOM consequence;

  // pour l'enclencher
  float distance;
  float puissance;
  // si on l'enclenche, alors consequence :
  ACTION_SALLE animation_salle;
  float v1, v2;
  float nb_pt_vie;
  RWVECTOR force;
  float gravite;

public:
    TYPE_ACTION_FACETTE type_facette;

  void applique_action_contact (LPCONTEXT_OBJET lpcontext);
  void applique_action_proximite (LPCONTEXT_OBJET lpcontext, float distance);
  void actionne (LPCONTEXT_OBJET lpcontext);

    ACTION_FACETTE ();
   ~ACTION_FACETTE ();
  void lecture_action_facette (BUFFER_NOEUD * BF, buffer_noeud * bn);
};

class PROPRIETE_FACETTE
{

public:

  TYPE_MATIERE_FACETTE type_matiere_facette;
  bool visible, sensible_objet, sensible_balle, transparent, additif;

  // résistance de la facette.
  float k_facette;

  // coefficient d'adhérence
  float k_adherence;

  // coefficients d'amortissements du rebond selon X et Z...
  float k_amortissement_x, k_amortissement_z;

  // coefficient de ralentissement de l'impulsion initiale du mouvement continu (pas du saut !!).
  float k_amortissement_mouvement;

    PROPRIETE_FACETTE ();
   ~PROPRIETE_FACETTE ();
  void lecture_propriete_facette (BUFFER_NOEUD * BF, buffer_noeud * bn);
};

#define NOMBRE_TEXCOORD 4

typedef struct TEXCOORD
{
  float TX, TY;
  // Ce bool sert  savoir si les 2 coord de texture ont t dfinies.
  // Ca sert  dterminer si il faut calculer les coords de texture.
  bool coord;
} TEXCOORDCOMPLEX;

class VERTEX
{
  TEXCOORD texcoord[NOMBRE_TEXCOORD];

  RWVECTOR point;

  PLAN plan;

  friend class FACETTE;
  friend class FACETTE_IMPACT;
  friend class FACETTE_PORTAL;

public:  VERTEX (float, float, float, float, float);
    VERTEX (RWVECTOR, float, float);
    VERTEX (float, float, float);
    VERTEX (BUFFER_NOEUD * BF, buffer_noeud * bn);
    VERTEX (RWVECTOR v);
  void trace_point ();

  RWVECTOR get_point ();
  bool get_isinfocoord ();
  void set_infocoord (float, float);
  void set_infocoord (float, float, int i);
  float get_TX ();
  float get_TY ();
  void set_decoupage_coord (float x, VERTEX & v1, VERTEX & v2);

   ~VERTEX ();
};

// La classe de calcul automatique des coordonnes de texture.
// Comment ca marche ?
// On utilise au minimum trois points qui dterminent le plan de la facette. 
// Celui-ci est utilis pour gnrer une quation P.u+d pour les X et les Y, ou
// on essaie de stisfaire au mieux les dpendances ScaleTX et ScaleTY, ainsi que les
// informations spcifies dans a,b et c.
// autogencoord permet ensuite de gnrer automatiquement les coordonnes pour un VERTEX
// pass en paramtre.
class GENCOORDTEX
{
  PLAN Px, Py;
public:


    GENCOORDTEX ();
  void calcul_param (VERTEX & a, VERTEX & b, VERTEX & c, float ScaleTX,
		     float ScaleTY);
  void autogencoord (VERTEX & a);
   ~GENCOORDTEX ();

};

typedef VERTEX *LPVERTEX;

typedef vector < LPVERTEX > LISTE_VERTEX;

// Une facette spcifique pour une texture d'impact
// Gestion interne du descripteur de texture à appeler
// Des fonctions de trac, de gestion temporelle, etc...
class FACETTE_IMPACT
{
  LISTE_VERTEX liste;

  float T, Tmax;
  MATRIX text_decal;

  // Ceci porte typiquement l'identifiant d'un objet JEU_IMPACT.
  // Mais on peut l'utiliser pour autre chose !
  int ID;

  PLAN plan;
  BOITE boite;

  VALEUR_LUMIERE couleur_princ;
  bool couleur;

  VALEUR_LUMIERE couleur_lprinc;

  LPFACETTE facette_source;

  // L'tat de multitexture.
  ETAT_MULTITEXTURE texture;

public:

  void ajoute_vertex (LPVERTEX lpvertex);
  void troncature (PLAN & p);

  // 2 fonctions qui agissent sur le callback de la facette.
  bool test_ID (int tID);
  void set_nom (int tID);
  void set_info (MATRIX & m, GLuint desc);
  void set_info_desc (GLuint desc);

  // Les fonctions de gestion temporelle de l'impact.
  // avance_T renvoie true si T>Tmax;
  void set_Tmax (float t);
  bool set_T (float diff);

  void trace_facette (ETAT_TEXTURE * etat_texture, LPBLOCINFO lpblocinfo);

    FACETTE_IMPACT (PLAN p, LPFACETTE lpf);
    FACETTE_IMPACT (LPFACETTE);
   ~FACETTE_IMPACT ();

  void set_couleur (VALEUR_LUMIERE & vl);
  void set_couleur_RGB (VALEUR_LUMIERE & vl);

  void reset_couleur_lumiere ();
  void add_couleur_lumiere (VALEUR_LUMIERE & vl);

  RWVECTOR centre ();
};

class FACETTE
{
protected:
  ACTION_FACETTE action;

  BOITE boite;
  PLAN plan;
  LISTE_VERTEX liste;

  void calcul_plan ();
  void calcul_BOX ();
  void calcul_marche ();

  // Matrice de projection sur le plan, dpend de u.
  // Doit tre combine au point de projection pour 
  // avoir une matrice de transfo complte.
  MATRIX transfo_zu;

  PROPRIETE_FACETTE propriete_facette;

  // Le gnrateur de coordonnes de texture.
  GENCOORDTEX gencoordtex;

  // L'tat de multitexture.
  ETAT_MULTITEXTURE texture;

public:
  float pente;

  BOITE clip;
    FACETTE ();
    FACETTE (BUFFER_NOEUD * BF, buffer_noeud * bn, ETAT_MULTITEXTURE & em,
	     float TexGenX, float TexGenY);
  void ajoute_vertex (LPVERTEX lpvertex);
  void lecture_point (BUFFER_NOEUD * BF, buffer_noeud * bn);
  void trace_facette (ETAT_TEXTURE * etat_texture);

  // Point A et B, distance d, impact, point projeté après impact
  // Retour : contact ou pas contact
  BOOL interaction (RWVECTOR, RWVECTOR, float, RWVECTOR *, RWVECTOR *);
  BOOL interaction_exp (RWVECTOR & A, RWVECTOR & B, float &C, RWVECTOR & D,
			RWVECTOR & P0, RWVECTOR & P1,
			float &delta_t, float &rayon);


  // Point A et B, distance d, impact remplace b en cas de collision...
  // Retour : contact ou pas contact
  BOOL interaction_base (RWVECTOR &, RWVECTOR &, float);

  void applique_action_proximite (LPCONTEXT_OBJET lpcontext, RWVECTOR & P);

   ~FACETTE ();

  BOITE get_boite ();
  float distance_boite (RWVECTOR & p);
  RWVECTOR get_u ();
  MATRIX get_transfo_zu ();
  void calcul_parametres_solides ();

  // Pour calculer des ombrages, des impacts, etc...
  // renvoie si il y a eu création de facette.
  // résultat contient la FACETTE_IMPACT créée, avec les positions des points/textures calculés.
  // point est le centre de la sphére.
  // distance est le rayon de la sphère.
  // distance_facette est la distance à la facette...
  bool impact_sphere (RWVECTOR & point, float distance,
		      float distance_facette, FACETTE_IMPACT ** resultat);

  PROPRIETE_FACETTE *get_propriete_facette ();

  friend class FACETTE_IMPACT;
  friend class FACETTE_PORTAL;
};


#endif
