/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: jeu.cpp,v 1.5 2006/12/29 12:30:17 atila Exp $

#include "jeu.h"

#include "jeu_vivant.h"

#include "jeu_option.h"
#include "jeu_impact.h"
#include "jeu_spawnjoueur.h"
#include "jeu_nain.h"
#include "jeu_projectile.h"
#include "jeu_joueur.h"
#include "jeu_debris.h"

/**********************************/
/*******                         *****/
/*******       ELEMENTJEU            *****/
/*******                         *****/
/**********************************/

RWVECTOR
ELEMENTJEU::get_position ()
{
  return context_objet.contexte.position;
};

int
ELEMENTJEU::get_ID_connexion ()
{
  return 0;
};

bool
ELEMENTJEU::get_suicide ()
{
  return suicide;
};

void
ELEMENTJEU::tir_balle (LPBLOCINFO lpblocinfo, int type_arme, float portee,
		       float deviation, float angleZ, float angleX,
		       float coeff_puissance)
{
  tir_balle (lpblocinfo, type_arme, portee, deviation,
	     angleZ, angleX, RWVECTOR (), RWVECTOR (), 1, coeff_puissance);
};

void
ELEMENTJEU::tir_balle (LPBLOCINFO lpblocinfo, int type_arme,
		       float portee, float deviation,
		       float angleZ, float angleX,
		       RWVECTOR source_decal, RWVECTOR position_decal,
		       float canon, float coeff_puissance)
{
  RWVECTOR a, b, ai, bi;
  ai = RWVECTOR (canon, 0, 0);
  bi = RWVECTOR (portee, 0, 0);
  MATRIX tr, tri, trx, trz, trxr, trzr, tra, trb;
  tri.translation (context_objet.contexte.position + position_decal);
  float devX = (((float) rand ()) / (float) RAND_MAX - 0.5) * deviation;
  float devZ = (((float) rand ()) / (float) RAND_MAX - 0.5) * deviation;
  trzr.rotationz (angleZ + devZ);
  trxr.rotationy (angleX + devX);
  trz.rotationz (angleZ);
  trx.rotationy (angleX);
  tra = tri * trz * trx;
  trb = tri * trzr * trxr;

  a = tra.transforme (ai);
  b = trb.transforme (bi);
  // On peut tirer.
  // On tire en fonction de l'arme en cours.
  switch (type_arme)
    {
    case TYPE_ARME_GATLING:
    case TYPE_ARME_TROMBLON:
    case TYPE_ARME_MEGAGATLING:
      lpblocinfo->lpjeu->tir (lpblocinfo, a, b, type_arme,
			      (int) coeff_puissance, ID);
      break;
    case TYPE_ARME_FUSILPLASMA:
    case TYPE_ARME_LANCEROQUETTE:
    case TYPE_ARME_LANCEMINE:
      RWVECTOR vitesse = (b - a);
      vitesse = vitesse * (1.0 / vitesse.norme ()) * 10;
      TYPE_CORE_PROJECTILE type_projectile;
      switch (type_arme)
	{
	case TYPE_ARME_LANCEROQUETTE:
	  type_projectile = TYPE_PROJECTILE_MISSILE;
	  vitesse = vitesse * (1.0 / vitesse.norme ()) * 15;
	  break;
	case TYPE_ARME_LANCEMINE:
	  type_projectile = TYPE_PROJECTILE_GRENADE;
	  vitesse = vitesse * (1.0 / vitesse.norme ()) * 10;
	  break;
	case TYPE_ARME_FUSILPLASMA:
	  type_projectile = TYPE_PROJECTILE_PLASMA;
	  vitesse = vitesse * (1.0 / vitesse.norme ()) * 10;
	  break;
	}
      int id = -1;
      if (typeElementJeu () == TYPE_JOUEUR)
	id = ID;
      //vitesse=vitesse+context_objet.contexte.V0;
      lpblocinfo->lpjeu->tir_projectile (lpblocinfo, a, vitesse,
					 type_projectile, id);
      break;
    }
}

int
ELEMENTJEU::getID ()
{
  return ID;
}

void
ELEMENTJEU::setID (int i)
{
  ID = i;
}

void
ELEMENTJEU::interaction (LPBLOCINFO lpblocinfo, LPELEMENTJEU)
{
  // On ne fait rien ici...
};

void
ELEMENTJEU::interaction_standard (ELEMENTJEU * elem)
{
  // Uniquement si l'autre est un objet solide, rel.
  switch (elem->typeElementJeu ())
    {
    case TYPE_JOUEUR:
    case TYPE_NDJ:

      // On est repouss par l'objet.
      RWVECTOR dist =
	context_objet.contexte.position -
	elem->context_objet.contexte.position;
      float R = context_objet.rayon + elem->context_objet.rayon;
      R = R - dist.norme ();
      R = 1;
      float rapport = (elem->context_objet.masse) / context_objet.masse;
      R = R * rapport * rapport / 50;
      context_objet.v_pousse_objet =
	context_objet.v_pousse_objet + dist.normalise () * R;
      break;
    }
};


ELEMENTJEU::ELEMENTJEU ()
{
  lpanimation = NULL;
  context_objet.rayon = 0.1;
  context_objet.masse = 85;

  context_objet.contexte.position = RWVECTOR (0, 0, 0);
  context_objet.contexte.contact = false;
  context_objet.contexte.contact_mur_suit = false;
  context_objet.contexte.contact_mur_acroche = false;
  context_objet.contexte.v_fconstant = RWVECTOR (0, 0, 0);
  context_objet.contexte.fconstant = RWVECTOR (0, 0, 0);
  context_objet.contexte.v_avance = RWVECTOR (0, 0, 0);
  context_objet.contexte.v_objet_local = RWVECTOR (0, 0, 0);
  context_objet.contexte.v_impulsion = RWVECTOR (0, 0, 0);
  context_objet.contexte.V0 = RWVECTOR (0, 0, 0);
  context_objet.contexte.k = -0.01;
  context_objet.v_avance_max = 0.0;

  context_objet.v_avance = RWVECTOR (0, 0, 0);
  context_objet.v_avance_involontaire = RWVECTOR (0, 0, 0);
  context_objet.v_saute = RWVECTOR (0, 0, 0);

  context_objet.type = elementinactif;
  context_objet.delta_vie = 0;
  context_objet.temps_delta_vie = 0;
  context_objet.action = false;

  context_objet.amortissement_z = 1.0;
  context_objet.amortissement_xy = 1.0;
  context_objet.k_adherence = 0.9;
  context_objet.hauteur_marche = 0.0;
  context_objet.avance_sans_contact = false;
  context_objet.coeff_gravite = 1.0;

  suicide = false;
  // Premiere init : on doit creer les facettes, indpendemment du temps.
  fp_time = -1.0;
  refresh = true;
  interaction_flag = true;

  deplacement = false;
  derniere_position = context_objet.contexte.position;

};


void
ELEMENTJEU::teleporte (RWVECTOR cible)
{
  context_objet.contexte.position = cible;
};

void
ELEMENTJEU::LectureParamJeu (BUFFER_NOEUD * BF, buffer_noeud * bn,
			     LPBLOCINFO lpblocinfo)
{
  // Rien de special...
  while (bn)
    {
      if (BF->get_nom (bn) == CONSTANTE_SCRIPT_POSITION)
	{
	  context_objet.contexte.position =
	    VERTEX (BF, BF->get_fils (bn)).get_point ();
	}
      bn = BF->trouve_suivant (bn);
    }
};

void
ELEMENTJEU::nettoie_liste_facette_proche ()
{
  for (LISTE_LPFACETTE_PROCHE::iterator t = context_objet.liste.begin ();
       t != context_objet.liste.end (); t++)
    {
      delete (*t);
    }
  context_objet.liste.erase (context_objet.liste.begin (),
			     context_objet.liste.end ());
}

ELEMENTJEU::~ELEMENTJEU ()
{
  if (lpanimation)
    delete (lpanimation);
  lpanimation = NULL;
  nettoie_liste_facette_proche ();
}

void
ELEMENTJEU::lecture (LPBLOCINFO lpblocinfo, STOCK & stock,
		     int &position_stock)
{
  struct LOCAL_CALC_CONTEXT_OBJET *bloc;
  bloc = (LOCAL_CALC_CONTEXT_OBJET *) stock.get_posbuffer (position_stock);
  position_stock += sizeof (LOCAL_CALC_CONTEXT_OBJET);
  context_objet.contexte = *bloc;
};

void
ELEMENTJEU::ecriture (LPBLOCINFO lpblocinfo, STOCK & stock)
{
  // Remplissage sans interêt...
  stock.put_buffer ((char *) &context_objet.contexte,
		    sizeof (struct LOCAL_CALC_CONTEXT_OBJET));
};

void
ELEMENTJEU::gestion_globale (LPBLOCINFO lpblocinfo)
{
  derniere_position = context_objet.contexte.position;

  // Modifier v_avance si v_avance_involontaire est mis en place (pour viter de se forcer  rentrer dans les autres objets).
  if (!(context_objet.v_pousse_objet == RWVECTOR (0, 0, 0)))
    {

      RWVECTOR inv_norme = context_objet.v_pousse_objet.normalise ();
      float proj = inv_norme.produit_scalaire (context_objet.v_avance) - 0.1;
      if (proj < 0.0)
	context_objet.v_avance = context_objet.v_avance - (inv_norme * proj);

    }

  if (((fp_time > 1.0 / 5.0) && (deplacement)) || (fp_time == -1.0))
    {
      fp_time = 0;
      nettoie_liste_facette_proche ();
      lpblocinfo->lpniveau->ajoute_facette_proche (context_objet.liste, 5.0,
						   context_objet.contexte.
						   position);
    }
  fp_time += lpblocinfo->diff;

  // On peut faire une interpolation sur la position : ca rajoute en precision
  // A chaque rafraichissement.
  //  if ((lpblocinfo->serveur)||(typeElementJeu()==TYPE_JOUEUR))

  if (context_objet.rayon != 0.0)
    lpblocinfo->lpniveau->deplacement (&context_objet, lpblocinfo);

  deplacement = !(derniere_position == context_objet.contexte.position);

};

void
ELEMENTJEU::affichage (LPBLOCINFO lpblocinfo)
{
  if (lpanimation)
    {
      lpanimation->trace_animation (lpblocinfo);
    }
};

void
ELEMENTJEU::interaction_danger (LPBLOCINFO lpblocinfo,
				int IDsrc,
				int force,
				TYPECOUP typeCoup, RWVECTOR A, RWVECTOR B)
{
  if (!interaction_danger_possible (typeCoup))
    return;

  // Reculer de la valeur donne, diminuer la vie...
  switch (typeCoup)
    {
    case COUP_BALLE:
      {
	RWVECTOR AB = B - A;
	AB = AB.normalise ();
	context_objet.v_avance_involontaire = AB * force + RWVECTOR (0, 0, 0);
	lpblocinfo->lpjeu->creation_impact (B,
					    context_objet.contexte.V0 +
					    RWVECTOR (0, 0, -0.5),
					    IMPACT_CHAIR_BALLE, force,
					    lpblocinfo);
      }
      break;
    case COUP_EXPLOSION:
      {
	// On est dans la sphre d'impact.
	// Dcaler l'objet.
	// A= direction.
	// force=force_explosion.
	context_objet.v_avance_involontaire =
	  context_objet.v_avance_involontaire + A * (-force) + RWVECTOR (0, 0,
									 1);
      };
      break;
    }
};

bool
ELEMENTJEU::interaction_danger_possible (int typeCoup)
{
  return false;
};

bool
ELEMENTJEU::touche_tir (RWVECTOR a, RWVECTOR b, RWVECTOR & touche)
{
  return false;
};


/**********************************/
/*******                         *****/
/*******       JEU                  *****/
/*******                         *****/
/**********************************/

JEU::JEU ()
{
  max_id = 1;
  lpjoueur_courant = NULL;
  dernier_point_spawn = -1;
  refresh = true;
  min_time = 1.0 / 40.0;
  last_time = 0;
}

JEU::~JEU ()
{
  //printf("Nettoyage du jeu ... penser  enlever le while(1)\n");
  //while (1);
}

void
JEU::InsererElementJeu (LPELEMENTJEU lpelement)
{
  max_id++;
  lpelement->setID (max_id);
  elements.insert (elements.end (), lpelement);
};

void
JEU::LectureParamJeu (BUFFER_NOEUD * BF, buffer_noeud * bn,
		      LPBLOCINFO lpblocinfo)
{
  // Cration des lments.
  while (bn)
    {
      if (BF->get_nom (bn) == CONSTANTE_SCRIPT_OPTION)
	{
	  // Ajout d'une option.
	  JEU_OPTION *opt;
	  opt = new JEU_OPTION (lpblocinfo);
	  opt->LectureParamJeu (BF, BF->get_fils (bn), lpblocinfo);
	  InsererElementJeu (opt);
	}
      if (BF->get_nom (bn) == CONSTANTE_SCRIPT_SPAWNJOUEUR)
	{
	  // Ajout d'un point de spawn
	  JEU_SPAWNJOUEUR *opt;
	  opt = new JEU_SPAWNJOUEUR ();
	  opt->LectureParamJeu (BF, BF->get_fils (bn), lpblocinfo);
	  InsererElementJeu (opt);
	}
      if (BF->get_nom (bn) == CONSTANTE_SCRIPT_NDJ)
	{
	  // Ajout d'un mchant NDJ.
	  JEU_NDJ *ndj;
	  ndj = new JEU_NDJ (lpblocinfo);
	  InsererElementJeu (ndj);
	  ndj->LectureParamJeu (BF, BF->get_fils (bn), lpblocinfo);
	}
      bn = BF->trouve_suivant (bn);
    }
};

void
JEU::affichage (LPBLOCINFO lpblocinfo)
{
  for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
       elem != elements.end (); elem++)
    {
      //printf("AVANT AFFICHAGE %i %i\n",(*elem)->getID(),(*elem)-> typeElementJeu());
      (*elem)->affichage (lpblocinfo);
      //printf("APRES AFFICHAGE %i\n",(*elem)->getID());
    }
}

struct BLOC_BASE_ELEMENT
{
  char typage;
  unsigned int ID;
};

void
JEU::gestion_globale (LPBLOCINFO lpblocinfo)
{
  lpjoueur_courant = NULL;

  // Et maintenant, on gère le client/serveur.
  // D'abord le serveur.
  if (lpblocinfo->serveur)
    {
      bool joueur_local = false;
      // Suppression des joueurs en fonction des infos dans "RESEAU".
      for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
	   elem != elements.end (); elem++)
	{
	  // est-ce un joueur. Et si oui, est-il encore connecté ???
	  if ((*elem)->typeElementJeu () == TYPE_JOUEUR)
	    {
	      bool found = false;
	      int ID_connexion = (*elem)->get_ID_connexion ();
	      // ID_connexion=-1 alors c'est le joueur local dans un jeu CS_MELTED.
	      if (ID_connexion != -1)
		{
		  LPTABLEAU_JOUEUR lptab;
		  lptab = &(lpblocinfo->lpreseau->info_connexion);
		  for (int i = 0; i < lptab->size (); i++)
		    {
		      if ((*lptab)[i]->ID_connexion == ID_connexion)
			found = true;
		    }
		  if ((!found) && (ID_connexion != -1))
		    {
		      // On va gentiment le gicler...
		      delete (*elem);
		      elements.erase (elem, ++elem);
		      elem = elements.begin ();
		    }
		}
	      else
		{
		  joueur_local = true;
		}
	    }
	}
      // Cration des joueurs en fonction des infos dans "RESEAU".
      LPTABLEAU_JOUEUR lptab;
      lptab = &(lpblocinfo->lpreseau->info_connexion);
      for (int i = 0; i < lptab->size (); i++)
	{
	  bool found = false;
	  int ID = (*lptab)[i]->ID_connexion;
	  for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
	       elem != elements.end (); elem++)
	    {
	      // est-ce un joueur ???
	      if ((*elem)->typeElementJeu () == TYPE_JOUEUR)
		{
		  int ID_connexion = (*elem)->get_ID_connexion ();
		  if (ID == ID_connexion)
		    {
		      found = true;
		    }
		}
	    }
	  if (!found)
	    {
	      // On ajoute un joueur...
	      JEU_JOUEUR *jj = new JEU_JOUEUR (lpblocinfo);
	      jj->context_objet.contexte.position = PointSpawnJoueur ();
	      jj->donnee_serveur.ID_connexion = ID;
	      InsererElementJeu (jj);
	      //printf("SERVEUR: JEU: Ajout d'un joueur dans la liste \n");
	    }
	}
      // Creation du joueur local.
      if ((lpblocinfo->client) && (!joueur_local))
	{
	  JEU_JOUEUR *jj = new JEU_JOUEUR (lpblocinfo);
	  jj->context_objet.contexte.position = PointSpawnJoueur ();
	  jj->donnee_serveur.ID_connexion = -1;
	  InsererElementJeu (jj);
	  lpjoueur_courant = jj;
	}
      // Retrouver le joueur local.
      if (lpblocinfo->client)
	for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
	     elem != elements.end (); elem++)
	  {
	    // est-ce un joueur ???
	    if ((*elem)->typeElementJeu () == TYPE_JOUEUR)
	      {
		int ID_connexion = (*elem)->get_ID_connexion ();
		if (ID_connexion == -1)
		  {
		    lpjoueur_courant = (LPJEU_JOUEUR) * elem;
		  }
	      }
	  }
      //     if (lpjoueur_courant) printf("elements--- %i %i\n",elements.size(),lpjoueur_courant->donnee_serveur.ID_connexion);

      // Gestion des interactions entre les differents objets...

      for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
	   elem != elements.end (); elem++)
	if (((*elem)->context_objet.rayon > 0) && ((*elem)->interaction_flag)
	    && ((*elem)->deplacement))
	  {
	    TYPEELEMENTJEU t1 = (TYPEELEMENTJEU) (*elem)->typeElementJeu ();
	    if ((t1 == TYPE_JOUEUR) ||
		(t1 == TYPE_NDJ) || (t1 == TYPE_PROJECTILE))
	      {
		for (LISTE_ELEMENTJEU::iterator elem2 = elements.begin ();
		     elem2 != elements.end (); elem2++)
		  {
		    if (((*elem2)->context_objet.rayon > 0)
			&& ((*elem2)->interaction_flag))
		      {
			TYPEELEMENTJEU t2 =
			  (TYPEELEMENTJEU) (*elem2)->typeElementJeu ();
			if (((t2 == TYPE_OPTION) && (t1 == TYPE_JOUEUR))
			    || ((t2 == TYPE_PROJECTILE)
				&& (t1 != TYPE_PROJECTILE))
			    || ((t2 == TYPE_JOUEUR)) || ((t2 == TYPE_NDJ)))
			  {
			    RWVECTOR O1, O2;
			    float d1, d2, d;
			    O1 = (*elem)->context_objet.contexte.position;
			    O2 = (*elem2)->context_objet.contexte.position;
			    d1 = (*elem)->context_objet.rayon;
			    d2 = (*elem2)->context_objet.rayon;
			    d = (O1 - O2).norme ();
			    if (d - d1 - d2 < 0)
			      {
				(*elem)->interaction (lpblocinfo, (*elem2));
				(*elem2)->interaction (lpblocinfo, (*elem));
			      }
			  }
		      }
		  }
	      };
	  }

      //if (lpjoueur_courant) printf("elements--- %i %i\n",elements.size(),lpjoueur_courant->donnee_serveur.ID_connexion);

      // Cration des infos  transfrer.
      STOCK stock (64 * 1024);
      for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
	   elem != elements.end (); elem++)
	if ((*elem)->ID > 0)
	  {
	    // Ecriture de ID et du Type.
	    BLOC_BASE_ELEMENT bloc;
	    bloc.typage = (*elem)->typeElementJeu ();
	    bloc.ID = (*elem)->getID ();
	    stock.put_buffer ((char *) &bloc, sizeof (bloc));
	    (*elem)->ecriture (lpblocinfo, stock);
	  }
      int taille;
      char *buffer = NULL;
      stock.get_buffer (&buffer, &taille);
      lpblocinfo->lpreseau->put_UDP (CANAL_OBJET, taille, buffer);
      if (buffer)
	free (buffer);

    }
  if ((lpblocinfo->client) && (!lpblocinfo->serveur))
    {
      int position = 0;
      unsigned int taille;
      void *buffer = NULL;
      STOCK stock (64 * 1024);
      lpblocinfo->lpreseau->get_UDP (CANAL_OBJET, &taille, &buffer);
      stock.put_buffer ((char *) buffer, taille);
      if (buffer)
	{
	  // On reconstruit une liste...
	  LISTE_ELEMENTJEU elements2 = elements;
	  elements.clear ();
	  // remettre les elements locaux en place.
	  LISTE_ELEMENTJEU::iterator locaux = elements2.begin ();
	  while (locaux != elements2.end ())
	    {
	      if ((*locaux)->ID < 0)
		{
		  elements.insert (elements.end (), *locaux);
		  elements2.erase (locaux, locaux + 1);
		  locaux = elements2.begin ();
		}
	      else
		locaux++;
	    }
	  // prendre les lments 1 par 1.
	  // rcuprer le type des lments et leur ID.
	  while (position < stock.get_size ())
	    {
	      BLOC_BASE_ELEMENT *bloc;
	      bloc = (BLOC_BASE_ELEMENT *) stock.get_posbuffer (position);
	      position += sizeof (BLOC_BASE_ELEMENT);

	      // printf("Type %i ID %i position %i\n",bloc->typage,bloc->ID,position);

	      // Regarder si il est dans la liste.
	      ELEMENTJEU *elementjeu = NULL;
	      LISTE_ELEMENTJEU::iterator i;
	      i = elements2.begin ();
	      bool found = false;
	      while (i != elements2.end () && (!found))
		{
		  if ((*i)->getID () == bloc->ID)
		    found = true;
		  else
		    i++;
		}
	      if (found)
		{
		  (*i)->lecture (lpblocinfo, stock, position);
		  // Déplacement dans l'autre liste.
		  elements.insert (elements.begin (), *i);
		  elements2.erase (i, i + 1);
		}
	      else
		{
		  // Création de l'élémént.
		  switch (bloc->typage)
		    {
		    case TYPE_OPTION:
		      JEU_OPTION * opt;
		      opt = new JEU_OPTION (lpblocinfo);
		      opt->setID (bloc->ID);
		      opt->lecture (lpblocinfo, stock, position);
		      opt->init_standard (lpblocinfo);
		      elements.insert (elements.begin (), opt);
		      break;
		    case TYPE_JOUEUR:
		      JEU_JOUEUR * jj;
		      jj = new JEU_JOUEUR (lpblocinfo);
		      jj->setID (bloc->ID);
		      jj->lecture (lpblocinfo, stock, position);
		      elements.insert (elements.begin (), jj);
		      break;
		    case TYPE_SPAWNJOUEUR:
		      JEU_SPAWNJOUEUR * jsj;
		      jsj = new JEU_SPAWNJOUEUR ();
		      jsj->setID (bloc->ID);
		      jsj->lecture (lpblocinfo, stock, position);
		      elements.insert (elements.begin (), jsj);
		      break;
		    case TYPE_IMPACT:
		      JEU_IMPACT * ji;
		      ji = new JEU_IMPACT ();
		      ji->setID (bloc->ID);
		      ji->lecture (lpblocinfo, stock, position);
		      elements.insert (elements.begin (), ji);
		      break;
		    case TYPE_NDJ:
		      JEU_NDJ * ndj;
		      ndj = new JEU_NDJ (lpblocinfo);
		      ndj->setID (bloc->ID);
		      ndj->lecture (lpblocinfo, stock, position);
		      elements.insert (elements.begin (), ndj);
		      break;
		    case TYPE_PROJECTILE:
		      JEU_PROJECTILE * proj;
		      proj = new JEU_PROJECTILE (lpblocinfo);
		      proj->setID (bloc->ID);
		      proj->lecture (lpblocinfo, stock, position);
		      elements.insert (elements.begin (), proj);
		      break;
		    }
		}
	    }
	  for (LISTE_ELEMENTJEU::iterator i = elements2.begin ();
	       i != elements2.end (); i++)
	    {
	      if ((*i)->serverKill () && ((*i)->ID > 0))
		{
		  delete ((*i));
		}
	    };
	  if (buffer)
	    free (buffer);
	}
      int ID_connexion_joueur = lpblocinfo->lpreseau->ID_connexion_client;
      for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
	   elem != elements.end (); elem++)
	{
	  // est-ce un joueur ???
	  if ((*elem)->typeElementJeu () == TYPE_JOUEUR)
	    {
	      int ID_connexion = (*elem)->get_ID_connexion ();
	      if (ID_connexion_joueur == ID_connexion)
		{
		  lpjoueur_courant = (JEU_JOUEUR *) * elem;
		}
	    }
	}
    }
  // Calcul d'une valeur de refresh
  last_time += lpblocinfo->diff;
  refresh = false;
  if (last_time > min_time)
    {
      last_time = 0;
      refresh = true;
    };
  // Ne PAS fusionner les 2 blocs suivants : sinon problme
  // de rfrencement dans la liste.
  LISTE_ELEMENTJEU copie_elements = elements;
  for (LISTE_ELEMENTJEU::iterator elem = copie_elements.begin ();
       elem != copie_elements.end (); elem++)
    {
      // L'ordre des 2 lignes compte !
      (*elem)->gestion_globale (lpblocinfo);
      (*elem)->refresh = refresh;
    }
  for (LISTE_ELEMENTJEU::iterator elem = elements.begin ();
       elem != elements.end (); elem++)
    {
      if ((*elem)->get_suicide ())
	{
	  // invoquer la suppression de l'objet.
	  delete *elem;
	  elements.erase (elem, elem + 1);
	  elem = elements.begin ();
	}
    }

}

void
JEU::set_info_joueur (MOT m1, MOT m2)
{
  if (lpjoueur_courant)
    lpjoueur_courant->set_info (m1, m2);
}

void
JEU::texte (MOT m, double t)
{
  if (lpjoueur_courant)
    lpjoueur_courant->texte (m, t);
}


RWVECTOR
JEU::PointSpawnJoueur ()
{
  // Trouver un point de spawn acceptable -> Un point suivant dans la liste,
  // apres le dernier utilis.
  dernier_point_spawn = PointSpawnSuivant (dernier_point_spawn);

  if (dernier_point_spawn != -1)
    {
      RWVECTOR
	pos =
	(elements[dernier_point_spawn])->context_objet.contexte.position;
    }
  if (dernier_point_spawn == -1)
    dernier_point_spawn = PointSpawnSuivant (-1);
  if (dernier_point_spawn == -1)
    return RWVECTOR (0, 0, 0);
  return ((elements[dernier_point_spawn])->context_objet.contexte.position);
}

int
JEU::PointSpawnSuivant (int depart)
{
  for (int i = depart + 1; i < elements.size (); i++)
    {
      if ((elements[i])->typeElementJeu () == TYPE_SPAWNJOUEUR)
	return i;
    }
  return -1;
}

JEU_JOUEUR *
JEU::get_joueur ()
{
  return lpjoueur_courant;
};

void
JEU::tir (LPBLOCINFO lpblocinfo, RWVECTOR & a, RWVECTOR & b, int type_arme,
	  int force, int ID_tireur)
{
  RWVECTOR ainit = a;
  RWVECTOR binit = b;

  // En fonction du type d'arme :
  float puissance;
  TYPE_JEU_IMPACT type_jeu_impact;

  switch (type_arme)
    {
    case TYPE_ARME_FUSILPLASMA:
      puissance = 1.0;
      type_jeu_impact = IMPACT_MUR_PLASMA;
      break;
    default:
      puissance = 1.0;
      type_jeu_impact = IMPACT_MUR_BALLE;
    }

  // dterminer si on touche le monde OU un objet OU rien du tout; Puis le point d'impact si
  // il s'agit d'un impact sur un mur.

  LPFACETTE lpf;
  LPOBJET lpo;

  // Monde

  RWVECTOR a_monde = ainit;
  RWVECTOR b_monde = binit;
  bool impact_monde =
    lpblocinfo->lpniveau->impact (a_monde, b_monde, &lpf, &lpo);
  RWVECTOR abm = (b_monde - a_monde);
  float distance_monde = abm.norme ();
  b_monde = b_monde + abm * (0.1 / distance_monde);

  // Objets

  RWVECTOR AB = binit - ainit;
  float ab = AB.norme ();
  ab *= ab;
  RWVECTOR AB_reduit = AB * (1.0 / ab);
  bool impact_objet = false;
  float k_cur = 1.0;
  float distance_objet;
  RWVECTOR b_objet;
  int i_touche;
  for (int i = 0; i != elements.size (); i++)
    if ((elements[i])->getID () != ID_tireur)
      if ((elements[i])->interaction_danger_possible (COUP_BALLE))
	{
	  RWVECTOR C = (elements[i])->get_position ();
	  RWVECTOR CA = C - ainit;
	  float k = CA.produit_scalaire (AB_reduit);
	  RWVECTOR c_prime = a + AB * k;
	  if ((c_prime - C).norme () < (elements[i])->get_rayon ())
	    if ((k > 0) && (k < k_cur))
	      {
		if ((elements[i])->touche_tir (ainit, binit, c_prime))
		  {
		    impact_objet = true;
		    b_objet = c_prime;
		    distance_objet = (a - b_objet).norme ();
		    i_touche = i;
		  }
	      }
	};

  // Agir en fonction de la distance...
  if ((impact_monde) &&
      ((distance_monde < distance_objet) || (!impact_objet)))
    {
      a = a_monde;
      b = b_monde;
      // Crer un objet d'impact sur le point touch.
      creation_impact (a, b, type_jeu_impact, force, lpblocinfo);
    }
  if ((impact_objet) &&
      ((distance_monde > distance_objet) || (!impact_monde)))
    {
      // Envoyer un impact sur l'objet...
      (elements[i_touche])->interaction_danger (lpblocinfo,
						ID_tireur,
						force,
						COUP_BALLE, ainit, b_objet);
    };
};

int 
JEU::cible_la_plus_proche(LPBLOCINFO lpblocinfo, RWVECTOR position, 
			  float portee, float angleZ, float angleX,
			  int ID_tireur)
{
  RWVECTOR a, b, ai, bi;
  ai = RWVECTOR (0, 0, 0);
  bi = RWVECTOR (portee, 0, 0);
  MATRIX tr, tri, trx, trz, trxr, trzr, tra, trb;
  tri.translation (position);
  trz.rotationz (angleZ);
  trx.rotationy (angleX);
  tra = tri * trz * trx;
  
  a = tra.transforme (ai);
  b = tra.transforme (bi);

  LPFACETTE lpf;
  LPOBJET lpo;

  // Monde

  int objet_touche=0;
  RWVECTOR am,bm;
  am=a;
  bm=b;
  bool impact_monde =
    lpblocinfo->lpniveau->impact (am, bm, &lpf, &lpo);
  RWVECTOR abm = (bm - am);
  float distance_monde = abm.norme ();

  // Objets

  RWVECTOR AB = b - a;
  float ab = AB.norme ();
  ab *= ab;
  RWVECTOR AB_reduit = AB * (1.0 / ab);
  bool impact_objet = false;
  float k_cur = 1;
  RWVECTOR b_objet;
  float current_distance = distance_monde;
  for (int i = 0; i != elements.size (); i++)
    {
    if (((elements[i])->getID () != ID_tireur) && 
	(elements[i])->cible_potentielle_lettre() )
	{
	  RWVECTOR C = (elements[i])->get_position ();
	  RWVECTOR CA = C - a;
	  float k = 0;
	  k = CA.produit_scalaire (AB) / ab;
	  if ((k > 0) && (k < 1))
	    {
	      RWVECTOR c_prime = a + AB_reduit * k;
	      float ccp= (c_prime -  C ) . norme();
	      float acp= (c_prime -  a ) . norme();

	      //printf("ccp %f acp %f \n", ccp, acp);

	      //if (ccp < acp)
		{
		  if (ccp + acp < current_distance)
		    {
		      k_cur = k;
		      current_distance=ccp + acp;
		      b_objet = c_prime;
		      objet_touche = (elements[i])->getID ();
		    }
		}
	    }
	};
    }
  return objet_touche;
};

void
JEU::tir_projectile (LPBLOCINFO lpblocinfo, RWVECTOR & p, RWVECTOR & v,
		     int type_projectile, int ID_tireur)
{
  JEU_PROJECTILE *jeu_proj = NULL;
  switch (type_projectile)
    {
      {
    case TYPE_PROJECTILE_AUCUN:
	return;
	break;
    case TYPE_PROJECTILE_GRENADE:
    case TYPE_PROJECTILE_MISSILE:
    case TYPE_PROJECTILE_PLASMA:
	jeu_proj = new JEU_PROJECTILE (lpblocinfo);
	jeu_proj->context_objet.contexte.position = p;
	jeu_proj->context_objet.contexte.V0 = v;
	//jeu_proj->context_objet.contexte.v_objet=v;
	jeu_proj->set_type ((TYPE_CORE_PROJECTILE) type_projectile);
	jeu_proj->set_vitesse (v);
	jeu_proj->set_source (ID_tireur);
	InsererElementJeu (jeu_proj);
	//printf("Projectile %i\n",jeu_proj->ID);
	break;
      }
    }
};

int
JEU::get_joueur_proche (RWVECTOR position)
{
  int ID = -1;
  RWVECTOR p_joueur;
  for (int i = 0; i < elements.size (); i++)
    {
      if ((elements[i])->typeElementJeu () == TYPE_JOUEUR)
	{
	  JEU_JOUEUR *jj = (JEU_JOUEUR *) elements[i];
	  if (jj->bloc_vivant.vie != 0)
	    {
	      if (ID == -1)
		ID = (elements[i])->getID ();
	      else
		{
		  RWVECTOR p2_joueur = (elements[i])->get_position ();
		  if ((p_joueur - position).norme () >
		      (p2_joueur - position).norme ())
		    {
		      ID = (elements[i])->getID ();
		      p_joueur = p2_joueur;
		    };
		}
	    };
	}
    }
  return ID;
};

LPELEMENTJEU
JEU::get_element (int ID)
{
  for (int i = 0; i < elements.size (); i++)
    {
      if ((elements[i])->getID () == ID)
	return elements[i];
    }
  return NULL;
};

void
JEU::creation_impact (RWVECTOR a, RWVECTOR b,
		      TYPE_JEU_IMPACT type_jeu_impact,
		      float puissance, LPBLOCINFO lpblocinfo)
{
  JEU_IMPACT *ji = new JEU_IMPACT ();
  InsererElementJeu (ji);
  ji->context_objet.contexte.position = b;
  ji->init_serveur (type_jeu_impact, puissance, a, b, lpblocinfo);
}

void
JEU::creation_fumee (LPBLOCINFO lpblocinfo, RWVECTOR p, RWVECTOR v)
{
  JEU_IMPACT *ji = new JEU_IMPACT ();
  InsererElementJeu (ji);
  ji->ID = -ji->ID;
  ji->context_objet.contexte.position = p;
  ji->init_serveur (IMPACT_FUMEE, 0, p, v, lpblocinfo);
};

void
JEU::creation_debris (MOT & animation,
		      RWVECTOR position, RWVECTOR vitesse,
		      float rayon, float masse, float temps,
		      LPBLOCINFO lpblocinfo)
{
  JEU_DEBRIS *ji =
    new JEU_DEBRIS (animation, position, vitesse, rayon, masse, temps,
		    lpblocinfo);
  InsererElementJeu (ji);
};

void
JEU::explosion (LPBLOCINFO lpblocinfo, int ID, RWVECTOR a, RWVECTOR b,
		float force, float rayon)
{
  lpblocinfo->lpjeu->creation_impact (a, b, IMPACT_EXPLOSION, force,
				      lpblocinfo);
  // Il faut aussi pousser tous les lments proches, et leur enlever de la vie, ajouter des points.
  for (int i = 0; i < elements.size (); i++)
    {
      ELEMENTJEU & el = *(elements[i]);
      // Calculer la distance  l'explosion.
      RWVECTOR direction = a - el.context_objet.contexte.position;
      //float rayon_src=get_element(ID)->context_objet.rayon;
      float dist = direction.norme ();
      //printf("DSIT %i %f \n",i,dist);

      if (dist < rayon)
	{
	  // On est dans la sphre d'impact.
	  // Dcaler l'objet.
	  float ratio = force * (rayon - dist) / rayon;
	  //printf(" dist %f rayon %f ratio %f\n",dist,rayon,ratio);
	  el.interaction_danger (lpblocinfo, ID, (int) ratio, COUP_EXPLOSION,
				 direction.normalise (), RWVECTOR (0, 0, 0));
	};
    };
};
