/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*

Ce fichier contient les définitions principales pour tous les éléments du jeu.
Tous les objets l'utilisent : joueur,option,ndj, ...

*/
// $Id: jeu.h,v 1.3 2006/12/21 18:18:14 atila Exp $

#ifndef JEU_H_FICHIER
#define JEU_H_FICHIER

#include "context_objet.h"
#include "bloc_info.h"
#include "buffer_noeud.h"
#include "animation.h"
#include <deque>
#include "stdio.h"
#include "banque.h"
#include "stock.h"
#include "reseau.h"
#include "niveau.h"

typedef enum
{ TYPE_AUCUN,
  TYPE_OPTION,
  TYPE_JOUEUR,
  TYPE_SPAWNJOUEUR,
  TYPE_NDJ,
  TYPE_LETTRE,
  TYPE_IMPACT,
  TYPE_PROJECTILE,
  TYPE_DEBRIS
}
TYPEELEMENTJEU;

typedef enum
{ COUP_PHYSIQUE, COUP_BALLE, COUP_EXPLOSION } TYPECOUP;

class ELEMENTJEU
{
protected:

  // Le contexte général de l'objet : position, vitesse ...
  // Seule la vitesse et la position sont passés au client pour modification.
  CONTEXT_OBJET context_objet;

  // La position precedente pour savoir s'il s'est deplace.
  RWVECTOR derniere_position;
  // Savoir s'il s'est deplace.
  bool deplacement;

  // Un identifiant unique choisi par le serveur pour identifier l'objet.
  int ID;

  // L'animation associee a l'objet.
  LPANIMATION lpanimation;

  // Changer cette valeur a true pour invoquer la fin de l'objet...
  // Cette méthode n'est valable que sur le serveur, puisque le client regenere a partir du
  // serveur.
  bool suicide;

  // Un flag fixe par JEU.
  bool refresh;

  // Un flag pour savoir si il peut interagir avec les autres elements.
  // utilise conjointement avec le rayon.
  bool interaction_flag;

  // Un temps pour appeler la regeneration de la liste des facette proches.
  // Le temps min est fixe...
  float fp_time;
  void nettoie_liste_facette_proche ();
  // Une fonction de tri classique.
  // type_arme : pareil que dans JEU
  // portee : assez clair, non ?
  // deviation pour le calcul de l'erreur de tir.
  // Le tir part du centre de la cible,
  // angleX et angleZ font référence aux rotations respectives des objets selon les 2 axes
  void tir_balle (LPBLOCINFO lpblocinfo, int type_arme, float portee,
		  float deviation, float angleZ, float angleX,
		  float coeff_puissance);

  void tir_balle (LPBLOCINFO lpblocinfo, int type_arme, float portee,
		  float deviation, float angleZ, float angleX,
		  RWVECTOR source_decal, RWVECTOR position_decal, float canon,
		  float coeff_puissance);

public:

  int getID ();
  void setID (int i);
  virtual int get_ID_connexion ();

  RWVECTOR get_position ();

    ELEMENTJEU ();
   ~ELEMENTJEU ();

  virtual short int typeElementJeu ()
  {
    return TYPE_AUCUN;
  };

  virtual bool serverKill ()
  {
    return true;
  };

  // ct serveur;
  virtual void LectureParamJeu (BUFFER_NOEUD * BF, buffer_noeud * bn,
				LPBLOCINFO lpblocinfo);
  virtual void ecriture (LPBLOCINFO lpblocinfo, STOCK & stock);

  // côté client;
  virtual void affichage (LPBLOCINFO lpblocinfo);
  virtual void lecture (LPBLOCINFO lpblocinfo, STOCK & stock, int &position);

  virtual void gestion_globale (LPBLOCINFO lpblocinfo);
  friend class JEU;

  // Gestion de la rencontre avec un autre objet.
  virtual void interaction (LPBLOCINFO lpblocinfo, ELEMENTJEU *);

  // Rencontre avec un autre objet : fonction standard pour se repousser mutuellement.
  void interaction_standard (ELEMENTJEU *);

  // Gestion d'un impact gnr par un autre objet.
  // Infos passes : ID de l'objet source qui a gnr l'impact
  //                 force du tir/coup
  //                 type de coup/tir
  // Point A : source
  // Point B : point d'impact prsum...
  virtual void interaction_danger (LPBLOCINFO lpblocinfo,
				   int IDsrc,
				   int force,
				   TYPECOUP typeCoup, RWVECTOR A, RWVECTOR B);
  virtual bool interaction_danger_possible (int typeCoup);
  virtual bool touche_tir (RWVECTOR a, RWVECTOR b, RWVECTOR & touche);

  bool get_suicide ();

  float get_rayon ()
  {
    return context_objet.rayon;
  };

  void teleporte (RWVECTOR);

  virtual bool envoie_lettre(char lettre)
    {
      return false;
    }
  virtual bool cible_potentielle_lettre()
    {
      return false;
    }
};

typedef ELEMENTJEU *LPELEMENTJEU;

class JEU_JOUEUR;
typedef JEU_JOUEUR *LPJEU_JOUEUR;
#include "jeu_impact.h"

#define LISTE_ELEMENTJEU deque<LPELEMENTJEU>

// **************************************
// Ce bout est un apport de jeu_joueur.h
struct DONNEE_ARME
{
  bool possede;
  int munition;
};
#define NB_ARME 7
//***************************************

class JEU_JOUEUR;

class JEU
{
  int max_id;
  LISTE_ELEMENTJEU elements;
  JEU_JOUEUR *lpjoueur_courant;

  int dernier_point_spawn;
  RWVECTOR PointSpawnJoueur ();
  int PointSpawnSuivant (int);

  // Un bit pour fixer si un refresh des animations est necessaire.
  // Grace a ce flag, on peut determiner s'il est necessaire de rafraichir,
  // sachant qu'il manque peut-etre de la puissance de calcul ailleurs.
  bool refresh;
  // Des parametres associes.
  // min_time  : le temps minimum a attendre entre 2 refresh
  // last_time : dernier temps ou refresh etait actif.
  float min_time;
  float last_time;


public:

  // Fonctionnalits cot serveur.
  void InsererElementJeu (LPELEMENTJEU lpelement);
  void LectureParamJeu (BUFFER_NOEUD * BF, buffer_noeud * bn,
			LPBLOCINFO lpblocinfo);
  void ecriture (LPBLOCINFO lpblocinfo, STOCK);

  int get_joueur_proche (RWVECTOR position);
  LPELEMENTJEU get_element (int ID);

  // Fonctionnalits cot client.
  // Pour rcuprer les infos du joueur.
  void affichage (LPBLOCINFO lpblocinfo);
  JEU_JOUEUR *get_joueur ();

  // Arme active
  int get_munitions (DONNEE_ARME[NB_ARME]);
  void set_info_joueur (MOT, MOT);
  void texte (MOT, double);

  void gestion_globale (LPBLOCINFO lpblocinfo);

    JEU ();
   ~JEU ();

  void test_contact (LPBLOCINFO lpblocinfo, char *texte);

  // Une fonction de tir.
  // Cette fonction prend en charge l'interaction avec le monde ET les objets.
  void tir (LPBLOCINFO lpblocinfo,
	    RWVECTOR & a, RWVECTOR & b,
	    int type_arme, int force, int ID_tireur);
  void tir_projectile (LPBLOCINFO lpblocinfo, RWVECTOR & a, RWVECTOR & b,
		       int type_projectile, int ID_tireur);
  void creation_impact (RWVECTOR a, RWVECTOR b,
			TYPE_JEU_IMPACT type_jeu_impact, float puissance,
			LPBLOCINFO lpblocinfo);
  void creation_fumee (LPBLOCINFO lpblocinfo, RWVECTOR p, RWVECTOR v);

  // Dterminer quelle est la cible la plus proche pour le jeu au clavier.
  int cible_la_plus_proche(LPBLOCINFO lpblocinfo, RWVECTOR position, 
			   float portee, float angleZ, float angleX,
			   int ID_tireur);

  // gnrer une force d'explosion.
  // Attention ! Ceci gnre AUSSI un objet d'explosion.
  // elem est utilis pour compter les points, faire du "friendly-fire", etc...
  // Force=force au centre.
  // Rayon : rayon de l'impac. Dcroissance linaire du centre vers le rayon extrieur.
  void explosion (LPBLOCINFO lpblocinfo, int ID, RWVECTOR a, RWVECTOR b,
		  float force, float rayon);

  void creation_debris (MOT & animation,
			RWVECTOR position,
			RWVECTOR vitesse,
			float rayon, float masse, float temps,
			LPBLOCINFO lpblocinfo);

};


#endif
