/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: jeu_impact.h,v 1.3 2006/12/21 18:18:14 atila Exp $


/***************************************************
 *
 * Un ELEMENTJEU pour la gestion des impacts sur les murs,
 * les objets et des explosions.
 *
 * Tous ces objets sont 
 *  - "transparents" pour les autres, mais doivent tre 
 *    transmis au client.
 *  - de temps de vie trs limite (qques secondes)
 * C'est le client qui fait les calculs d'impacts pour 
 * les dessins sur les murs.
 *
 ****************************************************/


#ifndef JEU_IMPACT_FICHIER
#define JEU_IMPACT_FICHIER

///////////////////////////////////////////////////////////////
// Dfinition des diffrents types possibles d'impacts.
// Cela comprend le nommmage des impacts, mais aussi des constantes
// associes
///////////////////////////////////////////////////////////////

// ATTENTION : CETTE VALEUR EST EN ADEQUATION AVEC CE QUI SUIT !!!
#define NB_TYPE_JEU_IMPACT 5
typedef enum TYPE_JEU_IMPACT
{
  IMPACT_MUR_BALLE = 0,
  IMPACT_EXPLOSION = 1,
  IMPACT_CHAIR_BALLE = 2,
  IMPACT_MUR_PLASMA = 3,
  IMPACT_FUMEE = 4,
};
// Un tableau de valeurs associes  chaque TYPE_JEU_IMPACT
typedef struct TABLEAU_TYPAGE_JEU_IMPACT
{
  char *nom_animation;
  char *nom_animation_texture;
  float temps_animation;	// Le temps initial  dcompter.
  bool impact_sphere;		// Si il faut faire un halo sphrique sur le mur.
  float rayon;			// Rayon des impacts sur les murs.
  float z_speed;		// monte/descente automatique : chair descend, explosion monte.

  int interaction_mur;		// Si ca interagit avec le monde. (rebonds sur les murs des boyaux)
  float poids_objet, rayon_objet;	// Son poids dans ce cas.
} TABLEAU_TYPAGE_JEU_IMPACT;


#include "jeu.h"
#include "animation_texture.h"
#include "def_texture.h"


const TABLEAU_TYPAGE_JEU_IMPACT tableau_typage_jeu_impact[NB_TYPE_JEU_IMPACT]
  = {
  {"d_commun/d_objet3d/impact.3da",
   "d_commun/d_objet3d/impact.ani",
   1,
   true,
   0.1},

  {
   "d_commun/d_explosion/explosion.3da",
   "",
   1.4,
   false,
   0.1},

  {
   "d_commun/d_impact/impact_chair.3da",
   "",
   0.5,
   false,
   0.1},

  {
   "d_commun/d_objet3d/impact_plasma.3da",
   "d_commun/d_objet3d/impact_plasma.ani",
   1,
   true,
   0.3,
   },

  {
   "d_commun/d_explosion/trail.3da",
   "",
   1.0,
   false,
   0,
   }

};

// Le bloc qui est pass entre le serveur et le client.
typedef struct BLOC_JEU_IMPACT
{
  TYPE_JEU_IMPACT type_jeu_impact;
  // Puissance de l'impact.
  float puissance;
  // Temps d'apparition restant. A 0.0, on dtruit l'objet.
  float temps_restant;
  // Les 2 points de tir : debut,fin -> sert  retrouver l'objet touch
  //(pour la matrice de dplacement).
  // Sert aussi dans le cas de impact_chair  donner une position (a) et vitesse
  //  l'objet (b), qui est stocke
  RWVECTOR a, b;
};

class JEU_IMPACT:public ELEMENTJEU
{

  // Pass par le serveur aux clients.
  BLOC_JEU_IMPACT bloc_jeu_impact;

  // Cette variables est utilise pour savoir si on a dj initialis l'objet
  // par une premire lecture de ses paramtres.
  // Par la suite, il NE FAUT PAS resynch avec le serveur sous peine d'avoir
  // des anim saccades.
  // Avant 1ire init : false;
  // Aprs 1re init  : true;
  bool preinit;

  // pour avoir une rfrence sur l'objet cible.
  LPOBJET lpobjetreference;
  // Et le pont correspondant. Si lpobjetreference est dfini, alors ce point
  // est utilis pour recalculer la position d'impact  chaque tour.
  RWVECTOR point_impact;
  // La matrice de la normale  l'impact, utilise pour transformer l'axe z.
  MATRIX normale_impact;

  // Appel par le client ET le serveur pour positionner les bonnes valeurs
  // tires du tableau de rfrence.
  void init_standard (LPBLOCINFO lpblocinfo);

  // La liste des objets a appeler par callback.
  LPLISTE_OBJET liste_objet;

  // Quelques infos sur l'anim texture  balancer pour l'impact.
  LPANIMATION_TEXTURE lpanimation_texture;
  // Avec son timing associ...
  float temps_anim_texture;

public:

    virtual short int typeElementJeu ()
  {
    return TYPE_IMPACT;
  };
  virtual bool serverKill ();

  // Initialisation. Elle doit se poursuivre par un "init_serveur" ou un "lecture"
  // pour positionner les valeurs au bon endroit.
  JEU_IMPACT ();

  ~JEU_IMPACT ();
  void init_serveur (TYPE_JEU_IMPACT type_jeu_impact, float puissance,
		     RWVECTOR & a, RWVECTOR & b, LPBLOCINFO lpblocinfo);
  void init_debris ();
  void gestion_globale (LPBLOCINFO lpblocinfo);

  void lecture (LPBLOCINFO lpblocinfo, STOCK & stock, int &position_stock);
  void ecriture (LPBLOCINFO lpblocinfo, STOCK & stock);
};

#endif
