/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: jeu_joueur.h,v 1.4 2006/12/28 10:50:43 atila Exp $

#ifndef JEU_JOUEUR_FICHIER
#define JEU_JOUEUR_FICHIER

#include "jeu.h"
#include "jeu_vivant.h"

#define JAMBE_STATIQUE	0
#define JAMBE_AVANCE	1
#define JAMBE_SAUTE	2

#define TEMPS_MIN_AVANT_SAUT		0.5

#define FORCE_DEPLACEMENT_AVANT		5
#define FORCE_DEPLACEMENT_STRAFE	      3

#define TYPE_ARME_GATLING	            1
#define TYPE_ARME_TROMBLON	            2
#define TYPE_ARME_MEGAGATLING	            3
#define TYPE_ARME_LANCEMINE	            4
#define TYPE_ARME_LANCEROQUETTE	      5
#define TYPE_ARME_FUSILPLASMA	            6

#define MAX_OPTION                  5
#define OPTION_INVULNERABILITE      0
#define OPTION_PIGEON               1
#define OPTION_ANNIHILATRON         2
#define OPTION_RAPIDITE             3
#define OPTION_SUPERPUISSANCE       4

#define TEMPS_FIN_JEU        3.0

// Ces infos-l sont forces par le serveur.
struct DONNEE_SERVEUR
{
  DONNEE_ARME arme[NB_ARME];
  float donnee_option[MAX_OPTION];
  int ID_connexion;
  int armure;

  char etat_jambe;
  // Prends 3 valeurs 
  // JAMBE_STATIQUE = statique;
  // JAMBE_AVANCE = avance;
  // JAMBE_SAUTE = saute;
  MOT skin;
  MOT nom;

  // Temps avant le prochain tir. Il faut decompte_tir<0 pour pouvoir tirer.
  float decompte_tir;

  // Temps depuis le dernier tir. Ca permet au client de mettre  jour son 
  // affichage d'arme en csq.
  // Pour savoir si le serveur a fait tirer le perso, on fait la comparaison 
  // avec le dernier temps enregistr.
  float temps_dernier_tir;

  bool trouve_vie;
  bool trouve_super_vie;
  bool trouve_ammo;
  bool trouve_armure;
  bool trouve_arme;
  bool douleur_simple;
  bool douleur_dure;
  int arme_selectionnee;

  // Ahhhh.... voil le compteur que tout le monde aimerait exploser !
  int points;

  // Petite subtilit : le prcdent sert pour le total de points de dgts, 
  // le suivant pour le nombere de frags.
  int frags;

  // Petit click quand l'arme est vide, et qu'on essaie de tirer. Ca fait toujours plaisir !
  // Le son est activ quand le temps du dernier click  chang.
  float temps_click;

  RWVECTOR V0;

  bool gagne;

  // En mode calcul/clavier
  int ID_ennemi_selectionne;
};



// Ces infos-l sont forces par le client.
struct DONNEE_CLIENT
{
  char arme_selectionnee;

  float angle_corps;
  float angle_arme;
  float angle_jambes;

  float T_anim_jambe;
  float T_anim_corps;

  // Valeurs valables uniquement en mode de jeu "clavier/calcul"
  int lettre_chiffre;

};

struct CONTROLE_JOUEUR
{
  char arme_nouvelle;

  bool avance;
  bool recule;
  bool saute;
  bool strafe_gauche;
  bool strafe_droite;
  bool tir;
  bool action;

  char arme_selectionnee;

  float angle_corps;
  float angle_arme;
  float angle_jambes;

  float T_anim_jambe;
  float T_anim_corps;

  MOT texte;
  double dernier_texte;
  MOT skin;
  MOT nom;

  // En mode calcul/clavier
  char touche_active;

};

class JEU_JOUEUR:public JEU_VIVANT
{
  DONNEE_SERVEUR donnee_serveur;
  DONNEE_CLIENT donnee_client;
  CONTROLE_JOUEUR controle_joueur;

  // Toujours charg sur le client.
  // Mais on n'affiche pas le joueur qui joue !
  LPANIMATION lpanim_armes[NB_ARME];
  LPANIMATION lpanimation_joueur;

  // Calcul en local...
  float ratio_statique_avance;
  float ratio_sol_air;

  ////////////////
  // Valeurs de tir uniquement utile au client.

  // A comparer avec celui envoy dans la dernire frame.
  float temps_dernier_tir_serveur;

  // Pour modifier l'affichage du client.
  float temps_dernier_tir_client;

  // Pour comparer le temps du dernier bruit de click (quand plus de munitions).
  float temps_click;		// (pour comparer ct serveur)
  float temps_client_click;	// valeur pour l'animation avec l'arme. 
  // (on peut mettre des clicks diffrents par arme)

  // Encore pour le client... le dcalage d'affichage.
  RWVECTOR decal;
  RWVECTOR decal_rot;

  // celui-la parce qu'on vole ou autre.
  RWVECTOR decal_effet;

  // Effet visuel pour gain de vie
  float effet_vie;
  // Effet visuel pour perte de vie
  float effet_degat;
  // Effet visuel pour gain d'option (ammo, arme)
  float effet_option;
  // Effet visuel quand le joueur perd
  float effet_mort;

  int last_vie;

  int son_douleur;
  int son_mort;

  /** Compteur dcremente quand le jeu est termin. quand il atteint 0
   on rend la main avec la fonction 'get_fini' */
  float compteur_fin;

  // Valeurs valables uniquement en mode de jeu "clavier/calcul"
  vector<int> ID_ennemi_vue;

public:
  void ajoute_munition (int num, int qte);
  void ajoute_arme (int num);
  void ajoute_vie (int qte);
  void ajoute_armure (int qte);
  void ajoute_option (int typeoption, int qte);

  virtual short int typeElementJeu ()
  {
    return TYPE_JOUEUR;
  };

  void affichage (LPBLOCINFO lpblocinfo);

  void gestion_globale (LPBLOCINFO lpblocinfo);
  int get_ID_connexion ();

  void lecture (LPBLOCINFO lpblocinfo, STOCK & stock, int &position_stock);
  void ecriture (LPBLOCINFO lpblocinfo, STOCK & stock);

  JEU_JOUEUR (LPBLOCINFO lpblocinfo);
  ~JEU_JOUEUR ();

  void set_info (MOT, MOT);
  void texte (MOT, double);
  int get_vie ();
  int get_armure ();
  int get_points ();
  int get_frags ();

  float get_effet_degat ();
  float get_effet_vie ();
  float get_effet_option ();
  float get_effet_mort ();
  bool get_gagne ();
  void set_gagne ();

  bool get_fini();

  int get_munitions (DONNEE_ARME[NB_ARME]);
  void get_decal (RWVECTOR &, RWVECTOR &, RWVECTOR &);

  void ajoute_points (int points, bool frag);
  friend class JEU;

  RWVECTOR get_position ();
  float get_vuex_joueur ();
  float get_vuez_joueur ();
  float get_option (float tab_opt[MAX_OPTION]);
  
  int get_cible_lettre();
};

#endif
