/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "jeu_vivant.h"
#include "jeu_joueur.h"
#include "math.h"

void
JEU_VIVANT::gestion_globale (LPBLOCINFO lpblocinfo)
{
  if (bloc_vivant.vie < 0)
    bloc_vivant.vie = 0;
  ELEMENTJEU::gestion_globale (lpblocinfo);
}



bool
JEU_VIVANT::touche_tir (RWVECTOR a, RWVECTOR b, RWVECTOR & touche)
{
  if (bloc_vivant.vie <= 0)
    return false;
  if (lpanimation)
    {
      touche = b;
      LPOBJET imp;
      LPFACETTE fac;
      return lpanimation->impact (a, touche, &fac, &imp);
    }
};

void
JEU_VIVANT::LectureParamJeu (BUFFER_NOEUD * BF, buffer_noeud * bn,
			     LPBLOCINFO lpblocinfo)
{
  ELEMENTJEU::LectureParamJeu (BF, bn, lpblocinfo);

  // Chargement spécifique....
  // genre, type d'arme, etc...
}

void
JEU_VIVANT::lecture (LPBLOCINFO lpblocinfo, STOCK & stock,
		     int &position_stock)
{
  ELEMENTJEU::lecture (lpblocinfo, stock, position_stock);
  bloc_vivant =
    *((BLOC_VIVANT_SERVEUR *) stock.get_posbuffer (position_stock));
  position_stock += sizeof (struct BLOC_VIVANT_SERVEUR);
}

void
JEU_VIVANT::ecriture (LPBLOCINFO lpblocinfo, STOCK & stock)
{
  ELEMENTJEU::ecriture (lpblocinfo, stock);
  stock.put_buffer ((char *) &bloc_vivant,
		    sizeof (struct BLOC_VIVANT_SERVEUR));
}

JEU_VIVANT::JEU_VIVANT (LPBLOCINFO lpblocinfo)
{
  bloc_vivant.vie = 100;
  bloc_vivant.degat = 0;
  bloc_vivant.coeff_degat = 1.0;
}

JEU_VIVANT::~JEU_VIVANT ()
{
}

void
JEU_VIVANT::interaction (LPBLOCINFO lpblocinfo, ELEMENTJEU * elem)
{
  interaction_standard (elem);
};

void
JEU_VIVANT::decompte_vie (float v, int &points, bool & frag)
{
  int perte = (int) (v * bloc_vivant.coeff_degat);
  bloc_vivant.vie -= perte;
  points = perte;
  bloc_vivant.degat += perte;
  frag = (bloc_vivant.vie <= 0);
  //printf("Vie %i\n",bloc_vivant.vie);
}

bool
JEU_VIVANT::interaction_danger_possible (int typeCoup)
{
  return true;
};


void
JEU_VIVANT::interaction_danger (LPBLOCINFO lpblocinfo,
				int IDsrc,
				int force,
				TYPECOUP typeCoup, RWVECTOR A, RWVECTOR B)
{
  if (interaction_danger_possible (typeCoup))
    {
      // Reculer de la valeur donne,
      ELEMENTJEU::interaction_danger (lpblocinfo, IDsrc, force, typeCoup, A,
				      B);
      // Diminuer la vie.
      int points;
      bool frag;
      decompte_vie (force, points, frag);
      // Retrouver le tireur.
      LPELEMENTJEU lpelem = lpblocinfo->lpjeu->get_element (IDsrc);
      // Ajouter des points si c'est un joueur.
      if (lpelem->typeElementJeu () == TYPE_JOUEUR)
	{
	  LPJEU_JOUEUR lpjoueur = (LPJEU_JOUEUR) lpelem;
	  lpjoueur->ajoute_points (points, frag);
	};
      //printf("Touche %i\n",force);
    };
};
