/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef lumiere_fichier_const
#define lumiere_fichier_const


#include "bdd.h"
#include "buffer_noeud.h"


enum ZONE_LUMIERE
{
  LUMIERE_ABSENTE,
  LUMIERE_SCENE,
  LUMIERE_OBJET,
  LUMIERE_ANIMATION
};

// Type lumire.
// Pour l'instant, seules SPOT et Ambiante sont prises en compte.
// Pour le cone, la mise en place du calcul est diffr.
enum TYPE_LUMIERE
{
  LUMIERE_SANS,
  LUMIERE_SPOT,
  LUMIERE_AMBIANTE,
  LUMIERE_AMBIANTE_DIFFUSE,
  LUMIERE_CONE
};

// *************************************************
// Classe qui combine les 2 prcdentes dfinitions pour crer une dfinition de lumire pour un objet...
// *************************************************

class INFO_LUMIERE
{
  ZONE_LUMIERE zone_lumiere;
  TYPE_LUMIERE type_lumiere;

public:

  // Lecture dans objet...
    INFO_LUMIERE (BUFFER_NOEUD * BF, buffer_noeud * bn);

  // Initialisation...
    INFO_LUMIERE ();

  // Pour le fun...
   ~INFO_LUMIERE ();

  friend class OBJET;
};


// *************************************************
// Une classe qui stocke les paramtres de lumire.
// Ceux-ci sont ordonns pour plaire  OpenGL. 
// Les valeurs par dfaut sont 1,1,1,1
// La modulation du canal alpha sert  modifier l'intensit
// *************************************************

class VALEUR_LUMIERE
{
 public:

  // Sont stocks ici les paramtres dynamiques.
  float R, G, B, A;
  float intensite, rayon;

  VALEUR_LUMIERE operator* (float);
  VALEUR_LUMIERE operator+ (VALEUR_LUMIERE vl);

  // Lecture dans animation...
  VALEUR_LUMIERE (BUFFER_NOEUD * BF, buffer_noeud * bn);

  // Initialisation...
    VALEUR_LUMIERE ();
    VALEUR_LUMIERE (float, float, float, float, float, float);
    VALEUR_LUMIERE (float, float, float, float);
   ~VALEUR_LUMIERE ();

  // Pour des oprations de mix et de somme.
  void mix (VALEUR_LUMIERE & v1, VALEUR_LUMIERE & v2, float T1, float T2,
	    float T);
  void somme (VALEUR_LUMIERE & v, float poids);

  // Pour calculer les impacts lumineux et autres...
  friend class OBJET;
};



#endif
