/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: mathsRWvsNDJ.cpp,v 1.3 2006/12/21 18:18:14 atila Exp $

#include "mathsRWvsNDJ.h"
#include <string.h>

/************************ BOITE **********************************************/

BOITE::BOITE ()
{
  RAZ ();
  min.x = 0;
  min.y = 0;
  min.z = 0;
  max.x = 0;
  max.y = 0;
  max.z = 0;
  calc_maxdim = 0;
  calc_volume = 0;
  calc_centre = RWVECTOR (0, 0, 0);

}

BOITE::~BOITE ()
{
};

void
BOITE::RAZ ()
{
  utilise = false;
};

void
BOITE::MAJ (VECTOR point)
{
  if (utilise)
    {
      if (point.x < min.x)
	min.x = point.x;
      if (point.y < min.y)
	min.y = point.y;
      if (point.z < min.z)
	min.z = point.z;

      if (point.x > max.x)
	max.x = point.x;
      if (point.y > max.y)
	max.y = point.y;
      if (point.z > max.z)
	max.z = point.z;
    }
  else
    {
      min = point;
      max = point;
      utilise = true;
    };

  float vmax = 0;
  if ((max.x - min.x) > vmax)
    vmax = max.x - min.x;
  if ((max.y - min.y) > vmax)
    vmax = max.y - min.y;
  if ((max.z - min.z) > vmax)
    vmax = max.z - min.z;
  calc_maxdim = vmax;
  calc_volume = (max.x - min.x) * (max.y - min.y) * (max.z - min.z);
  calc_centre =
    RWVECTOR ((max.x + min.x) / 2, (max.y + min.y) / 2, (max.z + min.z) / 2);

};

void
BOITE::MAJ (RWVECTOR point)
{
  MAJ (point.T_VECTOR ());
};

void
BOITE::MAJ (BOITE b)
{
  MAJ (b.max);
  MAJ (b.min);
}

float
BOITE::distance (VECTOR point)
{
  float dx, dy, dz, d;
  dx = dy = dz = d = 0;

  if (point.x < min.x)
    dx = min.x - point.x;
  if (point.x > max.x)
    dx = point.x - max.x;

  if (point.y < min.y)
    dy = min.y - point.y;
  if (point.y > max.y)
    dy = point.y - max.y;

  if (point.z < min.z)
    dz = min.z - point.z;
  if (point.z > max.z)
    dz = point.z - max.z;

  d = dx;
  if (d < dy)
    d = dy;
  if (d < dz)
    d = dz;

  return d;
};

float
BOITE::distance (RWVECTOR point)
{
  return distance (point.T_VECTOR ());
};

float
BOITE::pseudo_distance_boite (BOITE & b, MATRIX & m)
{
  RWVECTOR v[8];
  for (int i = 0; i < 4; i++)
    {
      v[i].valeur.x = min.x;
      v[i + 4].valeur.x = max.x;
    }
  for (int i = 0; i < 8; i += 2)
    {
      v[i].valeur.y = min.y;
      v[i + 1].valeur.y = max.y;
    }
  for (int i = 0; i < 8; i += 4)
    {
      v[i].valeur.z = min.z;
      v[i + 1].valeur.z = min.z;

      v[i + 2].valeur.z = max.z;
      v[i + 3].valeur.z = max.z;
    }
  for (int i = 0; i < 8; i++)
    v[i] = m.transforme (v[i]);
  float d = b.distance (v[0]);
  for (int i = 0; i < 8; i++)
    {
      float d2 = b.distance (v[i]);
      if (d > d2)
	d = d2;
    }
  return d;
}

float
BOITE::distance (BOITE & b, MATRIX & m, MATRIX & mp, MATRIX & m2,
		 MATRIX & m2p)
{
  MATRIX mm = m2p * m;
  float d = pseudo_distance_boite (b, mm);
  mm = m2 * mp;
  float d2 = b.pseudo_distance_boite (*this, mm);
  if (d2 < d)
    return d2;
  else
    return d;
  return d;
};

bool
BOITE::intersection (RWVECTOR & A, RWVECTOR & B)
{
  if (!utilise)
    return false;

  if ((!distance (A.T_VECTOR ())) || (!distance (B.T_VECTOR ())))
    return true;
  VECTOR VA, VB;
  VA = A.T_VECTOR ();
  VB = B.T_VECTOR ();
  float Ax, Ay, Az;
  float Bx, By, Bz;

  Ax = VA.x;
  Ay = VA.y;
  Az = VA.z;

  Bx = VB.x;
  By = VB.y;
  Bz = VB.z;

  // Test avec les pseudo-plans de l'axe X
  if (pseudo_intersection (Ax - min.x, Ay, Az,
			   Bx - min.x, By, Bz, min.y, max.y, min.z, max.z))
    return true;
  if (pseudo_intersection (Ax - max.x, Ay, Az,
			   Bx - max.x, By, Bz, min.y, max.y, min.z, max.z))
    return true;

  // Test avec les pseudo-plans de l'axe Y
  if (pseudo_intersection (Ay - min.y, Ax, Az,
			   By - min.y, Bx, Bz, min.x, max.x, min.z, max.z))
    return true;

  if (pseudo_intersection (Ay - max.y, Ax, Az,
			   By - max.y, Bx, Bz, min.x, max.x, min.z, max.z))
    return true;

  // Test avec les pseudo-plans de l'axe Z
  if (pseudo_intersection (Az - min.z, Ay, Ax,
			   Bz - min.z, By, Bx, min.y, max.y, min.x, max.x))
    return true;

  if (pseudo_intersection (Az - max.z, Ay, Ax,
			   Bz - max.z, By, Bx, min.y, max.y, min.x, max.x))
    return true;

  return false;
};

bool
  BOITE::pseudo_intersection (float Ax, float Ay, float Az,
			      float Bx, float By, float Bz,
			      float Ymin, float Ymax, float Zmin, float Zmax)
{

  if (((Ax <= 0) && (Bx <= 0)) || ((Ax >= 0) && (Bx >= 0)))
    return false;
  float diff = Bx - Ax;
  if (diff == 0)
    return true;
  float k = -Ax / diff;
  float Py = Ay + k * (By - Ay);
  float Pz = Az + k * (Bz - Az);
  if ((Py >= Ymin) && (Py <= Ymax) && (Pz >= Zmin) && (Pz <= Zmax))
    return true;
  return false;
};

float
BOITE::volume ()
{
  return calc_volume;
}

float
BOITE::maxDim ()
{
  return calc_maxdim;
}


RWVECTOR
BOITE::centre ()
{
  return calc_centre;
}

void
BOITE::print ()
{
  printf ("%f %f %f -- %f %f %f\n", min.x, min.y, min.z, max.x, max.y, max.z);
};

/************************ RWVECTOR **********************************************/


RWVECTOR::RWVECTOR ()
{
  valeur.x = valeur.y = valeur.z = 0;
};

RWVECTOR::RWVECTOR (float x, float y, float z)
{
  valeur.x = x;
  valeur.y = y;
  valeur.z = z;
};

RWVECTOR::RWVECTOR (VECTOR param)
{
  valeur = param;
};

RWVECTOR::~RWVECTOR ()
{
};

VECTOR
RWVECTOR::T_VECTOR ()
{
  return valeur;
};

float
RWVECTOR::norme ()
{
  return sqrt (valeur.x * valeur.x + valeur.y * valeur.y +
	       valeur.z * valeur.z);
};

void
RWVECTOR::produit_vectoriel (RWVECTOR v1, RWVECTOR v2)
{
  valeur.x = v1.valeur.y * v2.valeur.z - v1.valeur.z * v2.valeur.y;
  valeur.y = v1.valeur.z * v2.valeur.x - v1.valeur.x * v2.valeur.z;
  valeur.z = v1.valeur.x * v2.valeur.y - v1.valeur.y * v2.valeur.x;
};

float
RWVECTOR::produit_scalaire (RWVECTOR v)
{
  return (v.valeur.x * valeur.x + v.valeur.y * valeur.y +
	  v.valeur.z * valeur.z);
};

RWVECTOR
RWVECTOR::normalise ()
{
  float norme_val = norme ();
  if (norme_val != 0.0)
    return (RWVECTOR (valeur) * (1 / norme_val));
  else
    return RWVECTOR (0, 0, 0);
};

float
RWVECTOR::distance (RWVECTOR p2)
{

  return ((RWVECTOR (valeur) - p2).norme ());
}

void
RWVECTOR::affiche ()
{
  printf ("X %f Y %f Z %f \n", T_VECTOR ().x, T_VECTOR ().y, T_VECTOR ().z);
};

bool
RWVECTOR::operator== (RWVECTOR test)
{
  VECTOR t = test.T_VECTOR ();
  VECTOR t2 = T_VECTOR ();
  if ((t.x == t2.x) && (t.y == t2.y) && (t.z == t2.z))
    return true;
  else
    return false;
};

RWVECTOR
RWVECTOR::operator+ (RWVECTOR b)
{
  return (RWVECTOR
	  (valeur.x + b.valeur.x, valeur.y + b.valeur.y,
	   valeur.z + b.valeur.z));
}

RWVECTOR
RWVECTOR::operator- (RWVECTOR b)
{
  return (RWVECTOR
	  (valeur.x - b.valeur.x, valeur.y - b.valeur.y,
	   valeur.z - b.valeur.z));
}

RWVECTOR
RWVECTOR::operator* (float x)
{
  return (RWVECTOR (valeur.x * x, valeur.y * x, valeur.z * x));
}

float
RWVECTOR::x ()
{
  return valeur.x;
}

float
RWVECTOR::y ()
{
  return valeur.y;
}

float
RWVECTOR::z ()
{
  return valeur.z;
}

void
RWVECTOR::calcul_exp (RWVECTOR & A, RWVECTOR & B, float &C, RWVECTOR & D,
		      float &delta_t)
{
  valeur = (A + B * exp (C * delta_t) + D * delta_t).valeur;
};

void
RWVECTOR::V0_exp (RWVECTOR & A, RWVECTOR & B, float &C, RWVECTOR & D)
{
  valeur = (D + B * C).valeur;
};

bool
RWVECTOR::estNul ()
{
  if ((x () == 0) && (y () == 0) && (z () == 0))
    return true;
  else
    return false;
};

/************************ PLAN **********************************************/

void
PLAN::calcul_PLAN (RWVECTOR pa, RWVECTOR pb, RWVECTOR pc, RWVECTOR cote)
{
  calcul_PLAN (pa, pb, pc);
  if (COTE (cote, 0.0))
    u = u * (-1.0);
};


void
PLAN::calcul_PLAN (RWVECTOR pa, RWVECTOR pb, RWVECTOR pc)
{
  // Et hop, 2 vecteurs d'un seul coup
  RWVECTOR ab = pa + (pb * (-1));
  RWVECTOR ac = pa + (pc * (-1));
  // a,b,c a la vole
  u.produit_vectoriel (ab, ac);
  u = u.normalise ();
  P = pa;
};

float
PLAN::CalcVal (RWVECTOR & a)
{
  return (u.produit_scalaire (a + P));
};


PLAN::PLAN ()
{
};

PLAN::~PLAN ()
{
};

BOOL
PLAN::COTE (RWVECTOR pp, float d)
{
  RWVECTOR p2 = P + (u * d);
  RWVECTOR pprime = pp + (p2 * (-1.0));
  return (pprime.produit_scalaire (u) > 0);
};

RWVECTOR
PLAN::PROJECTION (RWVECTOR p, float d)
{
  RWVECTOR result;
  float k;
  k = ((P - p).produit_scalaire (u)) + d;
  result = p + (u * (k));
  return result;
}

BOOL
PLAN::intersection (RWVECTOR a, RWVECTOR b, float d, RWVECTOR * c)
{
  if ((COTE (a, d) != COTE (b, d)) && (COTE (a, d)))
    {
      RWVECTOR ab = b - a;
      RWVECTOR abn = ab.normalise ();
      RWVECTOR num = (P - a) + (u * d);
      float x = u.produit_scalaire (num) / u.produit_scalaire (abn);
      *c = a + (abn * x);
      return true;
    };
  return false;
};


float
PLAN::COTE_expo (RWVECTOR pp, float d)
{
  RWVECTOR p2 = P + (u * d);
  RWVECTOR pprime = pp + (p2 * (-1.0));
  float distance = pprime.produit_scalaire (u);
  distance *= 10.0;
  if (distance > 0)
    distance = 1.0;
  if (distance < -1.0)
    distance = -1.0;
  return distance;
}

/*
  BOOL PLAN::intersection_expo(   RWVECTOR& A,RWVECTOR& B,float& C,RWVECTOR& D,
  float& d,
  RWVECTOR& P0,RWVECTOR& P1,RWVECTOR& Pi,
  float& t)
  {
  //        RWVECTOR V0=B*C+D;
  //  printf(" C0 %f C1 %f T %f\n",
  //         COTE_expo(P0,d),COTE_expo(P1,d),t
  //         );
  if (  (COTE_expo(P0,d)>COTE_expo(P1,d))
  &&
  (COTE_expo(P0,d)>-1.0)
  )
  {
  #define nb_pas 10
  float tfin=0;
  RWVECTOR Pfin;
  Pfin.calcul_exp(A,B,C,D,tfin);
  float COTEtfin=COTE_expo(Pfin,d);
  int icolle=0;
  for (int i=1;i<=nb_pas;i++)
  {
  RWVECTOR Pt;
  float tt=(t)*i/nb_pas;
  Pt.calcul_exp(A,B,C,D,tt);
  float COTEtt=COTE_expo(Pt,d);
  if ((COTEtt>=COTEtfin)||(COTEtt==1.0))
  {
  tfin=tt;
  Pfin=Pt;
  COTEtfin=COTEtt;
  icolle=i;
  };
  };
  //    printf("TEMPS FINAL --- tfin %f t %f \n",tfin,t);
  if (icolle!=nb_pas)
  {
  P1=Pfin;
  Pi=Pfin;
  t=tfin;
  return true;
  };
  };
  return false;
  };
*/

BOOL
  PLAN::intersection_expo (RWVECTOR & A, RWVECTOR & B, float &C, RWVECTOR & D,
			   float &d, RWVECTOR & P0, RWVECTOR & P1, float &t)
{
  RWVECTOR V0 = B * C + D;
  float zero = 0;
  P0.calcul_exp (A, B, C, D, zero);
  P1.calcul_exp (A, B, C, D, t);
  /*
     printf("    C0 %f C1 %f U %f %f %f\n"
     "    V0 %f %f %f\n"
     "    P0 %f %f %f P1 %f %f %f\n",

     COTE_expo(P0,d),COTE_expo(P1,d),
     u.x(),u.y(),u.z(),
     V0.x(),V0.y(),V0.z(),
     P0.x(),P0.y(),P0.z(),
     P1.x(),P1.y(),P1.z() );
   */
  if ((COTE_expo (P0, d) > COTE_expo (P1, d)) && (COTE_expo (P0, d) > -1.0))
    {
      RWVECTOR Pi;
      float t0 = 0;
      float t1 = t;
      Pi = P0;
      RWVECTOR old_P0;

      while (fabs (t1 - t0) >= MAX_DELTA_T)
	{
	  float delta_prime = (t0 + t1) / 2;
	  RWVECTOR old_P0;
	  old_P0.calcul_exp (A, B, C, D, t0);
	  Pi.calcul_exp (A, B, C, D, delta_prime);
	  if (COTE_expo (Pi, d) >= COTE_expo (old_P0, d))
	    t0 = delta_prime;
	  else
	    t1 = delta_prime;
	};

      //   printf("+*+*+ T0 %f T1 %f \nCoteTO %f CoteTi %f \n",
      //             t0,t1,
      //             COTE_expo(old_P0,d),
      //             COTE_expo(P1,d)
      //             );

      t = t0;
      P1.calcul_exp (A, B, C, D, t);
      return true;
    }
  return false;
}

BOOL
PLAN::intersection (RWVECTOR a, RWVECTOR b, RWVECTOR * c, float &x)
{
  if ((COTE (a, 0) != COTE (b, 0)))
    {
      RWVECTOR ab = b - a;
      RWVECTOR abn = ab.normalise ();
      RWVECTOR num = (P - a);
      x = u.produit_scalaire (num) / u.produit_scalaire (abn);
      *c = a + (abn * x);
      x /= ab.norme ();
      return true;
    };
  return false;
};

void
PLAN::print ()
{
  printf ("PLAN - %f %f %f - %f %f %f\n", u.x (), u.y (), u.z (), P.x (),
	  P.y (), P.z ());
};

/************************ MATRIX **********************************************/



MATRIX
MATRIX::special_OpenGL ()
{
  MATRIX m2;

  if (ID)
    {
      for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	  valeur[i][j] = 0;
      for (int i = 0; i < 4; i++)
	valeur[i][i] = 1;
    }
  //      for (int i=0;i<4;i++) for (int j=0;j<4;j++) m2.valeur[i][j]=0;
  //    for (int i=0;i<4;i++) m2.valeur[3-i][i]=1;
  //    return m2;
  //   };

  for (int i = 0; i < 4; i++)
    for (int j = 0; j < 4; j++)
      m2.valeur[i][j] = valeur[j][i];
  m2.ID = false;
  return m2;
};

MATRIX
MATRIX::inverse ()
{
  if (ID)
    return MATRIX ();
  MATRIX retour;
  float a, b, c, d, e, f, g, h, i, j, k, l;

  a = valeur[0][0];
  b = valeur[0][1];
  c = valeur[0][2];
  d = valeur[0][3];

  e = valeur[1][0];
  f = valeur[1][1];
  g = valeur[1][2];
  h = valeur[1][3];

  i = valeur[2][0];
  j = valeur[2][1];
  k = valeur[2][2];
  l = valeur[2][3];

  float fk_gj = f * k - g * j;
  float ek_gi = e * k - g * i;
  float ej_fi = e * j - f * i;
  float bk_cj = b * k - c * j;
  float bg_cf = b * g - c * f;
  float gl_hk = g * l - h * k;
  float fl_hj = f * l - h * j;
  float ak_ci = a * k - c * i;
  float ag_ce = a * g - c * e;
  float el_hi = e * l - h * i;
  float bi_aj = b * i - a * j;
  float af_be = a * f - b * e;

  float alpha = a * fk_gj - b * ek_gi + c * ej_fi;

  retour.valeur[0][0] = fk_gj / alpha;
  retour.valeur[0][1] = -bk_cj / alpha;
  retour.valeur[0][2] = bg_cf / alpha;
  retour.valeur[0][3] = -(b * gl_hk - c * fl_hj + d * fk_gj) / alpha;

  retour.valeur[1][0] = -ek_gi / alpha;
  retour.valeur[1][1] = ak_ci / alpha;
  retour.valeur[1][2] = -ag_ce / alpha;
  retour.valeur[1][3] = (a * gl_hk - c * el_hi + d * ek_gi) / alpha;

  retour.valeur[2][0] = ej_fi / alpha;
  retour.valeur[2][1] = bi_aj / alpha;
  retour.valeur[2][2] = af_be / alpha;
  retour.valeur[2][3] = -(a * fl_hj - b * el_hi + d * ej_fi) / alpha;

  retour.valeur[3][0] = 0;
  retour.valeur[3][1] = 0;
  retour.valeur[3][2] = 0;
  retour.valeur[3][3] = 1;

  retour.ID = false;
  return retour;
};

void
MATRIX::imprime_test ()
{
  for (int i = 0; i < 4; i++)
    {
      for (int j = 0; j < 4; j++)
	printf ("%f ", valeur[i][j]);
      printf ("\n");
    };
  printf ("------\n");
}

MATRIX::MATRIX ()
{
  ID = true;
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      valeur[i][j] = 0;
  for (int i = 0; i <= 3; i++)
    valeur[i][i] = 1;
};

MATRIX & MATRIX::operator= (MATRIX test)
{
  if (test.ID)
    {
      ID = true;
      return *this;
    };
  ID = false;
#ifdef LINUX
  bcopy (test.valeur, valeur, 16 * sizeof (float));
#else
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      valeur[i][j] = test.valeur[i][j];
#endif
  return *this;
};


MATRIX::MATRIX (RWVECTOR A, RWVECTOR B, RWVECTOR C)
{
  for (int i = 0; i <= 2; i++)
    {
      valeur[i][3] = 0;
      valeur[3][i] = 0;
    }
  valeur[3][3] = 1;

  valeur[0][0] = A.x ();
  valeur[0][1] = A.y ();
  valeur[0][2] = A.z ();

  valeur[1][0] = B.x ();
  valeur[1][1] = B.y ();
  valeur[1][2] = B.z ();

  valeur[2][0] = C.x ();
  valeur[2][1] = C.y ();
  valeur[2][2] = C.z ();
  ID = false;
};


MATRIX::~MATRIX ()
{
  // fait rien  priori;
};

MATRIX
MATRIX::operator* (MATRIX & m2)
{
  if (ID)
    return m2;
  if (m2.ID)
    return *this;
  MATRIX rendu;
  float val;
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      {
	val = 0;
	for (int k = 0; k <= 3; k++)
	  val += valeur[i][k] * m2.valeur[k][j];
	rendu.valeur[i][j] = val;
      };
  rendu.ID = false;
  return rendu;
};

void
MATRIX::translation (RWVECTOR trans)
{
  if (trans == RWVECTOR (0, 0, 0))
    return;
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      valeur[i][j] = 0;
  for (int i = 0; i <= 3; i++)
    valeur[i][i] = 1;
  valeur[0][3] = trans.valeur.x;
  valeur[1][3] = trans.valeur.y;
  valeur[2][3] = trans.valeur.z;
  ID = false;
};


void
MATRIX::rotationx (float angle)
{
  float cosinus, sinus;
  cosinus = cos (angle);
  sinus = sin (angle);
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      valeur[i][j] = 0;
  for (int i = 0; i <= 3; i++)
    valeur[i][i] = 1;
  valeur[1][1] = cosinus;
  valeur[2][2] = cosinus;
  valeur[1][2] = -sinus;
  valeur[2][1] = sinus;
  ID = false;
};

void
MATRIX::rotationy (float angle)
{
  float cosinus, sinus;
  cosinus = cos (angle);
  sinus = sin (angle);
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      valeur[i][j] = 0;
  for (int i = 0; i <= 3; i++)
    valeur[i][i] = 1;
  valeur[0][0] = cosinus;
  valeur[2][2] = cosinus;
  valeur[0][2] = sinus;
  valeur[2][0] = -sinus;
  ID = false;
};

void
MATRIX::rotationz (float angle)
{
  float cosinus, sinus;
  cosinus = cos (angle);
  sinus = sin (angle);
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      valeur[i][j] = 0;
  for (int i = 0; i <= 3; i++)
    valeur[i][i] = 1;
  valeur[0][0] = cosinus;
  valeur[1][1] = cosinus;
  valeur[1][0] = sinus;
  valeur[0][1] = -sinus;
  ID = false;
};

void
MATRIX::echelle (RWVECTOR facteur)
{
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      valeur[i][j] = 0;
  valeur[3][3] = 1;
  valeur[0][0] = facteur.valeur.x;
  valeur[1][1] = facteur.valeur.y;
  valeur[2][2] = facteur.valeur.z;
  ID = false;
};

RWVECTOR
MATRIX::transforme (RWVECTOR vecteur)
{
  if (ID)
    return vecteur;
  int i = 0;
  VECTOR retour;
  float *ptr_retour;
  float *ptr_vecteur = (float *) (&vecteur);
  ptr_retour = (float *) (&retour);
  for (i = 0; i <= 2; i++)
    ptr_retour[i] = 0;
  for (i = 0; i <= 3; i++)
    for (int j = 0; j <= 2; j++)
      ptr_retour[i] += valeur[i][j] * ptr_vecteur[j];
  for (i = 0; i <= 2; i++)
    ptr_retour[i] += valeur[i][3];
  return retour;
};

RWVECTOR
MATRIX::extrait_origine ()
{
  if (ID)
    return RWVECTOR (0, 0, 0);
  return transforme (RWVECTOR (0, 0, 0));
};

RWVECTOR
MATRIX::extrait_x ()
{
  if (ID)
    return RWVECTOR (1, 0, 0);
  RWVECTOR x (1, 0, 0);
  RWVECTOR zero = extrait_origine ();
  x = transforme (x);
  x = x - zero;
  return x;
};

RWVECTOR
MATRIX::extrait_y ()
{
  if (ID)
    return RWVECTOR (0, 1, 0);
  RWVECTOR y (0, 1, 0);
  RWVECTOR zero = extrait_origine ();
  y = transforme (y);
  y = y - zero;
  return y;
};

RWVECTOR
MATRIX::extrait_z ()
{
  if (ID)
    return RWVECTOR (0, 0, 1);
  RWVECTOR z (0, 0, 1);
  RWVECTOR zero = extrait_origine ();
  z = transforme (z);
  z = z - zero;
  return z;
};

MATRIX
MATRIX::multiplie (float f)
{
  MATRIX ret;
  if (ID)
    {
      for (int i = 0; i <= 3; i++)
	for (int j = 0; j <= 3; j++)
	  ret.valeur[i][j] = 0;
      for (int i = 0; i <= 3; i++)
	ret.valeur[i][i] = f;
    }
  else
    {
      for (int i = 0; i <= 3; i++)
	for (int j = 0; j <= 3; j++)
	  ret.valeur[i][j] = valeur[i][j] * f;
    };
  ret.ID = false;
  return ret;
};

MATRIX
MATRIX::addition (MATRIX arg)
{
  MATRIX ret;
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      ret.valeur[i][j] = arg.valeur[i][j] + valeur[i][j];
  ret.ID = false;
  return ret;
};

bool
MATRIX::operator== (MATRIX test)
{
  if ((test.ID) && (ID))
    return true;
  for (int i = 0; i <= 3; i++)
    for (int j = 0; j <= 3; j++)
      if (valeur[i][j] != test.valeur[i][j])
	return false;
  return true;
};

void
MATRIX::mix (float t, MATRIX & a, MATRIX & b)
{
  // Mix de chacun des lments avec un ratio t de A, 1-t de B
  if ((a.ID) && (b.ID))
    {
      ID = true;
      return;
    };
  float alpha = 1.0 - t;
  for (int i = 0; i < 4; i++)
    for (int j = 0; j < 4; j++)
      valeur[i][j] = t * a.valeur[i][j] + alpha * b.valeur[i][j];
  ID = false;
};

void
MATRIX::genere_avec_u (RWVECTOR & u)
{
  RWVECTOR u2, v2, w2, z2;
  u2 = u;
  u2 = u2.normalise ();

  z2 = RWVECTOR (0, 0, -1.0);
  if (z2 == u2)
    {
      valeur[0][0] = -1.0;
      valeur[1][1] = 1.0;
      valeur[2][2] = -1.0;
      ID = false;
      return;
    }
  z2 = RWVECTOR (0, 0, 1.0);
  if (z2 == u2)
    {
      ID = true;
      return;
    }
  w2.produit_vectoriel (u2, z2);
  w2 = w2.normalise ();
  v2.produit_vectoriel (u2, w2);
  v2 = v2.normalise ();

  VECTOR u3, v3, w3;
  u3 = u2.T_VECTOR ();
  v3 = v2.T_VECTOR ();
  w3 = w2.T_VECTOR ();

  valeur[0][0] = w3.x;
  valeur[1][0] = w3.y;
  valeur[2][0] = w3.z;

  valeur[0][1] = v3.x;
  valeur[1][1] = v3.y;
  valeur[2][1] = v3.z;

  valeur[0][2] = u3.x;
  valeur[1][2] = u3.y;
  valeur[2][2] = u3.z;
  ID = false;
};
