/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*  Unit de cration de classes mathmatiques uniquement pour la gestion 
	des collisions avec les murs.
	Incorpore principlement : 
		Classe RBVector,Matrix
		Classe Plan
		Classe BOITE
		Des classes transitoires parce que c'est comme a...
*/
// $Id: mathsRWvsNDJ.h,v 1.3 2006/12/21 18:18:14 atila Exp $


#ifndef _maths_
#define _maths_

#include <math.h>
#include <stdio.h>
#define BOOL bool

#define MAX_DELTA_T 	0.0001

struct VECTOR
{
  float x, y, z;
};

class RWVECTOR
{
  VECTOR valeur;

public:

    friend class BOITE;
  friend class MATRIX;
  friend class PLAN;

    RWVECTOR ();
    RWVECTOR (VECTOR);
    RWVECTOR (float, float, float);
   ~RWVECTOR ();
  VECTOR T_VECTOR ();		// assure une surcouche totale
  float norme ();		// bourrin et lent, mais bon, on commence  me connaitre
  void produit_vectoriel (RWVECTOR, RWVECTOR);
  float produit_scalaire (RWVECTOR);
  RWVECTOR normalise ();
  float distance (RWVECTOR p2);
  void affiche ();
  bool operator== (RWVECTOR test);
  RWVECTOR operator+ (RWVECTOR b);
  RWVECTOR operator- (RWVECTOR b);
  RWVECTOR operator* (float x);

  float x ();
  float y ();
  float z ();

  bool estNul ();
  void calcul_exp (RWVECTOR & A, RWVECTOR & B, float &C, RWVECTOR & D,
		   float &delta_t);
  void V0_exp (RWVECTOR & A, RWVECTOR & B, float &C, RWVECTOR & D);

};
#define LPRWVECTOR RWVECTOR*


class PLAN
{
public:
  RWVECTOR P, u;

  void calcul_PLAN (RWVECTOR, RWVECTOR, RWVECTOR, RWVECTOR);
  void calcul_PLAN (RWVECTOR, RWVECTOR, RWVECTOR);
    PLAN ();
   ~PLAN ();
  BOOL COTE (RWVECTOR, float);
  float CalcVal (RWVECTOR & a);
  float COTE_expo (RWVECTOR pp, float d);

  RWVECTOR PROJECTION (RWVECTOR, float);

  // Intersection linaire.
  BOOL intersection (RWVECTOR a, RWVECTOR b, float d, RWVECTOR * c);

  // Intersection avec une exponentielle.
  BOOL intersection_expo (RWVECTOR & A, RWVECTOR & B, float &C, RWVECTOR & D,
			  float &d, RWVECTOR & P0, RWVECTOR & P1, float &t);
  BOOL intersection (RWVECTOR a, RWVECTOR b, RWVECTOR * c, float &x);

  void print ();

};


class MATRIX;
#define LPMATRIX MATRIX*
class MATRIX
{
  float valeur[4][4];
  bool ID;
public:  MATRIX ();
    MATRIX (RWVECTOR, RWVECTOR, RWVECTOR);
  MATRIX operator* (MATRIX & m2);
  MATRIX inverse ();
  void translation (RWVECTOR trans);
  void rotationx (float angle);
  void rotationy (float angle);
  void rotationz (float angle);
  void echelle (RWVECTOR facteur);
  MATRIX multiplie (float f);
  MATRIX addition (MATRIX arg);
  RWVECTOR transforme (RWVECTOR vecteur);
  RWVECTOR extrait_origine ();
  RWVECTOR extrait_x ();
  RWVECTOR extrait_y ();
  RWVECTOR extrait_z ();
  bool operator== (MATRIX test);
    MATRIX & operator= (MATRIX test);
  void imprime_test ();
  void mix (float t, MATRIX & a, MATRIX & b);
   ~MATRIX ();
  void genere_avec_u (RWVECTOR & u);
  MATRIX special_OpenGL ();
};

class BOITE
{
  // Pour dterminer si il y a  intersection avec un pseudo-plan de la boite...
  // les Axyz,Bxyz pour les points A et B dans le repre relatif du plan
  // le plan est dans X=0, et ses limites sont les Ymin,max Zmin,max
  bool pseudo_intersection (float Ax, float Ay, float Az,
			    float Bx, float By, float Bz,
			    float Ymin, float Ymax, float Zmin, float Zmax);

  BOOL utilise;
  VECTOR min, max;
  float calc_volume;
  RWVECTOR calc_centre;
  float calc_maxdim;

  // Fonction intermdiaire pour la distance inter-boite.
  // Calcule la distance de chacun des points extrmes de b, transforms par m,  b.
  float pseudo_distance_boite (BOITE & b, MATRIX & m);

public:


    BOITE ();
   ~BOITE ();
  void RAZ ();
  void MAJ (VECTOR);
  void MAJ (RWVECTOR);
  void MAJ (BOITE);
  float distance (VECTOR);
  float distance (RWVECTOR);
  float distance (BOITE & b, MATRIX & m, MATRIX & mp, MATRIX & m2,
		  MATRIX & m2p);
  bool intersection (RWVECTOR & A, RWVECTOR & B);
  float volume ();
  float maxDim ();
  RWVECTOR centre ();
  void print ();
};

#endif
