/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef moteur3d_fichier_const
#define moteur3d_fichier_const

#include "lecture.h"
#include "bloc_info.h"
#include "vector"
#include "list"
#include "deque"
#include "mathsRWvsNDJ.h"
#include "animation_texture.h"


class OBJET;
class ANIMATION;
class FACETTE_PORTAL;
typedef FACETTE_PORTAL *LPFACETTE_PORTAL;
typedef OBJET *LPOBJET;
typedef vector < LPOBJET > LISTE_LPOBJET;
typedef ANIMATION *LPANIMATION;
typedef vector < LPANIMATION > LISTE_LPANIMATION;

typedef struct PORTAL
{
  // La facette qui va servir  dlimiter la warp zone.
  LPFACETTE facette;
  // La facette recopie.
  LPFACETTE_PORTAL facette_portal;
  // Son petit nom tel qu'il est rcupr.
  MOT nom_objet;
  // Le nom rsolu en objet.
  LPOBJET lpobjet;
  // Le nom resolu en numero dans le moteur 3D.
  int num;
} PORTAL;

// Liste de portals.
typedef deque < PORTAL > LISTE_PORTAL;

#include "def_3d.h"

// La classe FACETTE_PORTAL sert a definir un cone de visualisation.
// Si elle est vide, alors elle contient tout l'ecran.

class FACETTE_PORTAL:public FACETTE
{
  FACETTE *source;


  // Pour la troncature.
  void troncature (PLAN & p);

public:
    FACETTE_PORTAL (FACETTE *);
    FACETTE_PORTAL ();

  // Generer une facette portal de base, centree sur l'observateur et
  // qui contient tout l'ecran.
    FACETTE_PORTAL (MATRIX &);

  // On met a jour les coordonnees de la facette,en fonction de
  // - la position de l'objet qui contient le portal,
  //   passe par la matrice de l'objet.
  // - l'orientation courante de l'observateur
  void calcule_trame (MATRIX &, RWVECTOR &);

  // On restreint le cone de visualisation a l'aide de la commande
  // OpenGL (glClipPlane).
  void trace_cone_visualisation ();

  // Celle-ci sert a ne donner aucun ordre de contrainte
  // de visualisation
  // utile pour les objets dans lesquels on se trouve
  static void cone_plein ();

  // Se tronquer par une autre facette_portal.
  void tronquer (LPFACETTE_PORTAL);

  // Tester son existence.
  bool existe ();

  // additionner a une autre facette portal.
  // Le resultat et stocke dans this.
  void ajoute (LPFACETTE_PORTAL);

  // Realiser une copie de la facette.
  FACETTE_PORTAL *copie ();

  // Determiner si elle est visible par l'observateur.
  bool sens (RWVECTOR & position);

  // Mettre  jour les plans de scission en fonction de la position de
  // l'observateur.
  void MAJ_plan (RWVECTOR & position);
   ~FACETTE_PORTAL ();

  void print ();
};

/*
// Cette structure sert a stocker les objets qui seront
// utilises pour les listes d'inclusions.
// C'est cet objet qui est passe au moteur3D par
// des objets ou des animations.
typedef struct LISTE_INCLUSION
{
      RWVECTOR centre;
      float rayon;
      LISTE_LPOBJET liste_objet;
} LISTE_INCLUSION;
    */
typedef struct OBJET_PORTAL
{
  LPOBJET objet;
  // Liste des facettes generees.
  LISTE_PORTAL liste_portal;
  MOT nom_animation;
  MOT nom_objet;
  int objet_num;
  // La facette portal qui le contient.
  LPFACETTE_PORTAL facette_contenant;
} OBJET_PORTAL;

typedef vector < OBJET_PORTAL > LISTE_OBJET_PORTAL;

typedef vector < LPOBJET > LISTE_LPOBJET;

/*
struct OBJET_INDIVIDUEL
{
  RWVECTOR centre;
  float rayon;
  LISTE_INDICE_PORTAL indice_portal;
  LPOBJET lpobjet;
};

typedef vector<OBJET_INDIVIDUEL> LISTE_OBJET_INDIVIDUEL;

typedef struct ANIMATION_TRACE
{
  bool ensemble;

  // Liste generique si ensemble=true;
  RWVECTOR centre;
  float rayon;
  LISTE_INDICE_PORTAL indice_portal;
  float refreshFrame;

  // Liste d'objets individuels
  LISTE_OBJET_INDIVIDUEL liste_objet_individuel;
  
  LISTE_LPOBJET liste_objet_lumiere_animation;
  LISTE_LPOBJET liste_objet_lumiere_objet;
} ANIMATION_TRACE;

typedef vector<ANIMATION_TRACE> LISTE_ANIMATION_TRACE;
 */
// La liste de rangement de l'affichage.
// On range les objets a aficher.
// 1er niveau : les portals tels quels+les objets qu'il contiennent
// niveau suivants: Les objets qui apparitiennent a un
//                  croisement de portals.
class LISTE_AFFICHAGE
{
  LPFACETTE_PORTAL lpfacette_affichage;
  LISTE_LPOBJET a_tracer;
  // Sous-liste correspondant au croisement avec l'indice courant.
  // La sous-liste doit-etre d'une taille egale au nombre de portal.
    vector < LISTE_AFFICHAGE > sous_liste_affichage;
  int nb_portal;

public:

    LISTE_AFFICHAGE ();
   ~LISTE_AFFICHAGE ();

  // Nettoyage et definition du nombre de portal, pour creation des ss-listes.
  void clear (int);
  // insertion d'une liste d'objet dans la liste
  // La facette portal doit etre copiee, puis etendue par tous les niveaux superieurs, avant
  // d'etre inseree dans un nouvel LISTE_AFFICHAGE (si besoin il y a d'en creer un)
  void insert_liste_objet (LISTE_LPOBJET & liste_objet,
			   LISTE_INDICE_PORTAL & liste_indice);
  void insert_portal (LPOBJET lpobjet,
		      int indice, LPFACETTE_PORTAL lpfacette_portal);

  // une fonction de trace final.
  void trace (LPBLOCINFO lpblocinfo);
};

// Le refresh-rate pour les lumieres.
#define MAX_TEMPS_LUMIERE     0.05

class MOTEUR			// Yeah... Ca dpote tout....
{
  unsigned int desc_image_lumiere;

  LISTE_LPOBJET liste_objet_lumiere_scene;
  LISTE_LPANIMATION liste_animation;

  float temps_lumiere;

  int frame_lumiere;

  LISTE_OBJET_PORTAL liste_objet_portal;

  // Methode interne pour mettre a jour recursivement
  // en cascade les portals.
  void update_facette_portal (OBJET_PORTAL &);

  LPFACETTE facette_base;

  // Ou les objets se trouvent, et avec quelles facettes portal les tracer.
  // Le niveau 0 correspond a l'ecran (clipping ecran).
  LISTE_AFFICHAGE liste_affichage;

  // Insertion d'un indice dans une liste, et dans le sens croissant.
  void insertion_indice (LISTE_INDICE_PORTAL &, int);
  void trouve_liste_portal (LISTE_INDICE_PORTAL & liste, float rayon,
			    RWVECTOR centre);

  GLuint dessin_degat, dessin_mort, dessin_option, dessin_vie, dessin_gagne;

  LISTE_LPOBJET objet_horsmoteur;
  bool mode_clip;

  MOT cible;
  int desc_image_cible;

  // Trouver la liste des portals impliques.
  RWVECTOR centre;
  float rayon;
  LISTE_INDICE_PORTAL indice_portal;
  float refreshFrame;


public:

  // La matrice observateur. On multiplie chaque objet
  // pour connaitre sa position dans le repre observateur. 
    MATRIX positionnement_absolu;

  // Matrice d'orientation relatif. Utilise pour les
  // objets parallle  la camra, 
  // et pour l'arme.
  MATRIX positionnement_relatif;
  RWVECTOR position_camera, position_camera_standard;

  MATRIX positionnement_relatif_inverse, positionnement_absolu_inverse;
  MATRIX positionnement_camera;

  // Appelees par le module principal.
  void init_affichage (LPBLOCINFO lpblocinfo,
		       RWVECTOR position,
		       float Oz, float Ox, float hauteur_vue);
  void init_affichage (LPBLOCINFO lpblocinfo,
		       RWVECTOR position,
		       RWVECTOR decal, RWVECTOR rot_decal,
		       float Oz, float Ox, float hauteur_vue);
  void fin_affiche (LPBLOCINFO lpblocinfo);
  void swap_end ();

  // Appelee par animation
  void nouvelle_animation (LPANIMATION lpa);
  // Bien desallouer proprement, partout ou ca apparait.
  void fin_animation (LPANIMATION lpa);

    MOTEUR (LPBLOCINFO lpblocinfo);
   ~MOTEUR ();

  void insert_portal (LPOBJET, MOT &, MOT &, LISTE_PORTAL &);
  void solve_portal ();
  void effet (LPBLOCINFO lpblocinfo, bool gagne, float effet_vie,
	      float effet_option, float effet_degat, float effet_mort);

  void switch_mode_clip (bool);

  void affiche_cible (MOT & fichier_cible, LPBLOCINFO lpblocinfo,
		      float decal_x = 0.0, float decal_y = 0.1);
  void polygone_ecran (LPBLOCINFO lpblocinfo, float alpha, int dessin,
		       float coeff = 0.5, float depx = 0.0, float depy = 0.0);
};

typedef MOTEUR *LPMOTEUR;

#endif
