/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// $Id: niveau.cpp,v 1.3 2006/12/21 18:18:14 atila Exp $

#include "niveau.h"
#include "script.h"
#include "snd_mixer.h"

NIVEAU::NIVEAU ()
{
};


NIVEAU::~NIVEAU ()
{
  // Destruction des salles.
  for (LISTE_SALLE::iterator salle = liste_salle.begin ();
       salle != liste_salle.end (); salle++)
    {
      delete (*salle);
    };
};

bool
NIVEAU::renvoie_intersection_lineaire (RWVECTOR & P0, RWVECTOR & P1,
				       float rayon, LPFACETTE & lpfacette,
				       LPFACETTE_PROCHE & lpf,
				       RWVECTOR & normale,
				       RWVECTOR & pcourant,
				       LISTE_LPFACETTE_PROCHE & liste)
{
  // Trouver l'intersection la plus proche.
  bool intersection = false;
  lpf = NULL;
  lpfacette = NULL;
  // printf("TEST BOUCLE -----------------------------------------------\n");

  for (LISTE_LPFACETTE_PROCHE::iterator t = liste.begin (); t != liste.end ();
       t++)
    {
      LPOBJET lpobj = (*t)->lpobjet;
      MATRIX & mat = lpobj->repere_global;
      MATRIX & mat_inv = lpobj->repere_global_inverse;
      RWVECTOR point_zero = mat_inv.transforme (RWVECTOR (0, 0, 0));
      MATRIX translate;
      translate.translation (point_zero);
      MATRIX mat_statique = mat * translate;
      LISTE_FACETTE *liste2;
      liste2 = &((*t)->liste_facette);
      RWVECTOR P0bis, P1bis, P2bis;
      P0bis = mat_inv.transforme (P0);
      P1bis = mat_inv.transforme (P1);
      for (LISTE_FACETTE::iterator t2 = liste2->begin ();
	   t2 != liste2->end (); t2++)
	{
	  RWVECTOR inter, project;
	  if ((*t2)->interaction (P0bis, P1bis, rayon, &P2bis, &project))
	    {
	      intersection = true;
	      P1 = mat.transforme (P2bis);
	      P1bis = P2bis;
	      pcourant = P2bis;
	      lpfacette = *t2;
	      lpf = *t;
	      RWVECTOR norm = lpfacette->get_u ();
	      normale = mat_statique.transforme (norm);
	      /*
	         printf("intersection \nP0 %f %f %f \nP1 %f %f %f \n N0 %f %f %f NP %f %f %f\n",
	         P0.x(),P0.y(),P0.z(),P1.x(),P1.y(),P1.z(),
	         norm.x(),norm.y(),norm.z(),
	         normale.x(),normale.y(),normale.z()
	         );   */
	    };
	}
    };
  return intersection;
};


/******************************************************************
 *
 * Dplacement nouvelle gnration...
 *
 ******************************************************************/
void
NIVEAU::ajoute_facette_proche (LISTE_LPFACETTE_PROCHE & liste, float taille,
			       RWVECTOR position)
{
  for (LISTE_SALLE::iterator salle = liste_salle.begin ();
       salle != liste_salle.end (); salle++)
    {
      (*salle)->ajoute_facettes_proches (position, 3.0, &liste);
    };
}

void
avance_contexte (LOCAL_CALC_CONTEXT_OBJET & lcco, RWVECTOR P, float delta_t)
{
  if (delta_t != 0)
    {
      RWVECTOR diff_p = P - lcco.position;
      diff_p = diff_p * (1.0 / delta_t);
      lcco.V0 = diff_p;
    }
  lcco.position = P;

  // avancer les forces constantes
  lcco.v_impulsion = lcco.v_impulsion + lcco.fconstant * delta_t;

  // avancer l'impulsion.
  lcco.v_impulsion = lcco.v_impulsion * (exp (lcco.k * delta_t));

  // La marche a pied
  if (lcco.k_avance == 0)
    {
      lcco.v_avance = lcco.v_avance * (exp (lcco.k * delta_t));
    }
}

RWVECTOR
test_avance (LOCAL_CALC_CONTEXT_OBJET & lcco, float delta_t)
{
  RWVECTOR impulsion =
    lcco.v_impulsion * ((exp (delta_t * lcco.k) - 1.0) / lcco.k);
  RWVECTOR avance;
  if (lcco.k_avance == 0)
    {
      avance = lcco.v_avance * ((exp (delta_t * lcco.k) - 1.0) / lcco.k);
    }
  else
    {
      avance = lcco.v_avance * delta_t;
    }
  RWVECTOR P =
    lcco.fconstant * (delta_t * delta_t / 2) +
    lcco.position + lcco.v_decor_local * (delta_t) + impulsion + avance;
  return P;
}

void
projection_vecteur (RWVECTOR & v, RWVECTOR n, float coeff)
{
  float ordV = v.produit_scalaire (n);
  if (ordV < 0)
    v = v + n * (-ordV * coeff);
}

void
NIVEAU::interaction_mur (LOCAL_CALC_CONTEXT_OBJET & lcco, LPOBJET lpobj,
			 LPFACETTE lpfacette, CONTEXT_OBJET & co,
			 RWVECTOR & n, float diff)
{
  float coeff = co.amortissement_z;
  projection_vecteur (lcco.v_fconstant, n, coeff);
  projection_vecteur (lcco.fconstant, n, coeff);
  projection_vecteur (lcco.v_avance, n, coeff);
  projection_vecteur (lcco.v_objet_local, n, coeff);
  projection_vecteur (lcco.v_decor_local, n, coeff);
  projection_vecteur (lcco.v_impulsion, n, coeff);
  lcco.contact = true;
  if (coeff == 1.0)
    {
      float prod = n.produit_scalaire (RWVECTOR (0, 0, 1.0));
      if (prod > 0.7)
	{
	  lcco.contact_mur_acroche = true;
	  lcco.contact_mur_suit = true;
	  // on rajoute l'effet de pousse par l'objet.
	  MATRIX & lrgi = lpobj->lrepere_global_inverse;
	  MATRIX & rgi = lpobj->repere_global;
	  RWVECTOR vitesse_locale =
	    lcco.position - rgi.transforme (lrgi.transforme (lcco.position));
	  co.v_pousse_decor = vitesse_locale * (-1.0 / diff);
	}
    }
};

void
NIVEAU::deplacement (LPCONTEXT_OBJET lpcontext, LPBLOCINFO lpblocinfo)
{
  LISTE_LPFACETTE_PROCHE & liste = lpcontext->liste;
  CONTEXT_OBJET & co = *lpcontext;
  LOCAL_CALC_CONTEXT_OBJET & lcco = co.contexte;
  RWVECTOR pos = lcco.position;

  bool was_contact = lcco.contact;
  bool was_contact_mur_suit = lcco.contact_mur_suit;
  bool was_contact_mur_acroche = lcco.contact_mur_acroche;
  lcco.contact_mur_suit = false;
  lcco.contact_mur_acroche = false;
  lcco.contact = false;
  if ((!was_contact_mur_acroche) && (!co.avance_sans_contact))
    {
      co.v_avance = RWVECTOR (0, 0, 0);
    }
  if ((!was_contact_mur_suit) && (!co.avance_sans_contact))
    {
      co.v_saute = RWVECTOR (0, 0, 0);
    }

  RWVECTOR P1 = lcco.position;

  // On dplace l'objet si il est pouss par un bloc du niveau...
  for (LISTE_LPFACETTE_PROCHE::iterator t = liste.begin (); t != liste.end ();
       t++)
    {
      RWVECTOR P12, P22, P3, P4;
      LPOBJET lpobj = (*t)->lpobjet;
      lpobj->calcul_matrices_inverse ();
      MATRIX & lrgi = lpobj->lrepere_global_inverse;
      MATRIX & rgi = lpobj->repere_global_inverse;
      P12 = lrgi.transforme (P1);
      P22 = rgi.transforme (P1);
      LISTE_FACETTE *liste2;
      liste2 = &((*t)->liste_facette);
      for (LISTE_FACETTE::iterator t2 = liste2->begin ();
	   t2 != liste2->end (); t2++)
	{
	  if ((*t2)->interaction (P12, P22, co.rayon, &P3, &P4))
	    {
	      RWVECTOR trans = P22 - P12;
	      P1 = lpobj->repere_global.transforme (P12);
	      P12 = lpobj->lrepere_global_inverse.transforme (P1);
	      P22 = lpobj->repere_global_inverse.transforme (P1);
	      lcco.contact = true;
	      lcco.position = P1;
	      // reinititialisation du dep par objet.
	      co.v_pousse_decor = RWVECTOR ();
	    }
	}
    }

  // Toutes les actions de proximite...
  for (LISTE_LPFACETTE_PROCHE::iterator t = liste.begin (); t != liste.end ();
       t++)
    {
      RWVECTOR P12, P22, P3, P4;
      LPOBJET lpobj = (*t)->lpobjet;
      P12 = lpobj->lrepere_global_inverse.transforme (P1);
      P22 = lpobj->repere_global_inverse.transforme (P1);
      LISTE_FACETTE *liste2;
      liste2 = &((*t)->liste_facette);
      for (LISTE_FACETTE::iterator t2 = liste2->begin ();
	   t2 != liste2->end (); t2++)
	{
	  (*t2)->applique_action_proximite (&co, P22);
	  if (co.lpaction_salle)
	    {
	      LPSALLE lps = (LPSALLE) ((*t)->lpanim);
	      lps->applique_action (co.lpaction_salle);
	      co.lpaction_salle = false;
	    }
	}
    }

  float delta_t = lpblocinfo->diff;
  lcco.v_decor_local = co.v_pousse_decor;

  // V_avance vaut la valeur entree.
  if (co.v_avance == RWVECTOR (0, 0, 0))
    {
      lcco.k_avance = 0;
    }
  else
    {
      lcco.v_avance = co.v_avance;
      lcco.k_avance = 1;
    }
  // V impulsion cumule les modifs.
  lcco.v_impulsion = lcco.v_impulsion + co.v_avance_involontaire + co.v_saute;
  if (!was_contact)
    {
      lcco.k = -0.1;
    }
  else
    {
      lcco.k = -15.0;
    }

  co.v_avance = RWVECTOR (0, 0, 0);
  co.v_saute = RWVECTOR (0, 0, 0);
  co.v_avance_involontaire = RWVECTOR (0, 0, 0);
  co.v_pousse_objet = RWVECTOR (0, 0, 0);
  co.v_pousse_decor = RWVECTOR (0, 0, 0);

  // Calcul des parametres A et B.
  lcco.fconstant =
    RWVECTOR (0, 0, -10.0 * co.masse / 85.0 * co.coeff_gravite);

  while (delta_t > 0.0)
    {
      RWVECTOR P_copie = lcco.position;
      RWVECTOR P_prime = test_avance (lcco, delta_t);
      RWVECTOR P_prime2 = P_prime;
      RWVECTOR normale, pobjet;
      LPFACETTE lpfacette;
      LPFACETTE_PROCHE lpfacette_proche;
      float adv = 0;
      if (renvoie_intersection_lineaire (P_copie,
					 P_prime2,
					 co.rayon,
					 lpfacette,
					 lpfacette_proche,
					 normale, pobjet, co.liste))
	{
	  // Regarder si c'est une marche qu'on peut monter.
	  bool marche = false;
	  if (co.hauteur_marche > 0.0)
	    {
	      float pente =
		fabs (normale.produit_scalaire (lcco.fconstant.normalise ()));
	      // On a donc la nouvelle normale, on vrifie qu'elle forme une marche.
	      if (pente < 0.1)
		{
		  // On ne monte QUE si c'est possible de l'avoir d'une seule enjambe.
		  // (sinon, on risque de pouvoir escalader une montagne)
		  RWVECTOR pos = lcco.position;
		  RWVECTOR dir = P_prime - pos;
		  float hauteur = co.hauteur_marche;
		  if (test_marche
		      (pos, dir, co.rayon, hauteur, lpfacette, liste))
		    {
		      marche = true;
		      float vitesse = dir.norme () / delta_t;
		      float temps = hauteur / vitesse;
		      if (temps > delta_t)
			temps = delta_t;
		      float hauteur_reelle = vitesse * temps;
		      delta_t -= temps;
		      lcco.position =
			lcco.position + RWVECTOR (0, 0, hauteur_reelle);
		      lcco.contact = true;
		      lcco.contact_mur_acroche = true;
		      lcco.contact_mur_suit = true;
		    };
		}
	    }
	  if (!marche)
	    {
	      float D2 = (lcco.position - P_prime).norme ();
	      float D = (lcco.position - P_prime2).norme ();
	      float delta_t2 = D / D2 * delta_t;
	      avance_contexte (lcco, P_prime2, delta_t2);
	      interaction_mur (lcco, lpfacette_proche->lpobjet, lpfacette, co,
			       normale, lpblocinfo->diff);
	      delta_t = delta_t - delta_t2;
	    }
	}
      else
	{
	  avance_contexte (lcco, P_prime, delta_t);
	  delta_t = 0;
	}
    };
}

bool
NIVEAU::test_marche (RWVECTOR position, RWVECTOR direction, float rayon,
		     float &hauteur_max, LPFACETTE lpfacette,
		     LISTE_LPFACETTE_PROCHE & liste)
{
  RWVECTOR positionT = position;
  RWVECTOR positionT2 = position + RWVECTOR (0, 0, hauteur_max);
  LPFACETTE_PROCHE lpfacette_proche;
  LPFACETTE lpfacetteT;
  RWVECTOR normale, pcourant;
  bool intersection = renvoie_intersection_lineaire (positionT,
						     positionT2,
						     rayon,
						     lpfacetteT,
						     lpfacette_proche,
						     normale, pcourant,
						     liste);
  // position2 est la hauteur max a laquelle on peut monter.
  // on recherche si on peut avancer, et de combien.
  bool trouve = false;
  RWVECTOR hauteur = (positionT2 - positionT);
  for (int i = 1; i != 10; i++)
    {
      float coeff = float (i) / 10.0;
      RWVECTOR position3 = position + hauteur * coeff;
      RWVECTOR position4 = position3 + direction;
      LPFACETTE lpfacette2;
      bool inter = renvoie_intersection_lineaire (position3,
						  position4,
						  rayon,
						  lpfacette2,
						  lpfacette_proche,
						  normale, pcourant,
						  liste);
      if ((!inter) || (lpfacette2 != lpfacette))
	{
	  trouve = true;
	  hauteur_max = (hauteur * coeff).z ();
	  break;
	};
    };
  return trouve;
}

void
NIVEAU::charge_niveau (MOT * nom_fichier, LPBLOCINFO lpblocinfo)
{
  BUFFER_NOEUD BF (lpblocinfo->lpbdd, nom_fichier);
  buffer_noeud *bf, *bf2, *bf3, *bf4;
  bf = BF.ROOT ();
  if (!bf)
    printf ("Niveau non-trouv !!! %s\n", nom_fichier->info);
  while (bf)
    {
      if ((BF.get_nom (bf) == CONSTANTE_SCRIPT_MUSIQUE)
	  && (lpblocinfo->client))
	{
	  musique = BF.get_MOT (bf);
	  musique_handler = lpblocinfo->lpmixer->lance_son (musique, 1);
	  lpblocinfo->lpmixer->set_volume (musique_handler, 128);
	}
      else if (BF.get_nom (bf) == CONSTANTE_SCRIPT_SALLE)
	{
	  MOT nom_salle, nom_animation;
	  bf2 = BF.get_fils (bf);
	  bf3 = BF.trouve_suivant (bf2, CONSTANTE_SCRIPT_FICHIER_SALLE);
	  bf4 = BF.trouve_suivant (bf2, CONSTANTE_SCRIPT_FICHIER_ANIMATION);

	  if ((bf3) && (bf4))
	    {
	      LPSALLE nouv = new SALLE ();
	      //nouv->charge_salle (&nom_salle, lpblocinfo);
	      //nouv->charge_animation (&nom_animation, lpblocinfo);
	      if (bf3->type_noeud == TEXTE)
		{
		  nom_salle = BF.get_MOT (bf3);
		  nouv->charge_salle (&nom_salle, lpblocinfo);
		}
	      if (bf3->type_noeud == FILS)
		{
		  bf3 = BF.get_fils (bf3);
		  nouv->charge_salle (&BF, bf3, lpblocinfo);
		}

	      if (bf4->type_noeud == TEXTE)
		{
		  nom_animation = BF.get_MOT (bf4);
		  nouv->charge_animation (&nom_animation, lpblocinfo);
		}
	      if (bf4->type_noeud == FILS)
		{
		  bf4 = BF.get_fils (bf4);
		  nouv->charge_animation (&BF, bf4, lpblocinfo);
		}
	      liste_salle.insert (liste_salle.begin (), nouv);
	    }
	  else
	    printf ("WARNING : Fichier de niveau mal-format !!!\n");
	}

      if (lpblocinfo->serveur)
	if (BF.get_nom (bf) == CONSTANTE_SCRIPT_JEU)
	  {
	    lpblocinfo->lpjeu->LectureParamJeu (&BF, BF.get_fils (bf),
						lpblocinfo);
	  }
      bf = BF.trouve_suivant (bf);
    }
  if (lpblocinfo->client)
    {
      for (LISTE_SALLE::iterator salle = liste_salle.begin ();
	   salle != liste_salle.end (); salle++)
	{
	  (*salle)->connecte_moteur3d (lpblocinfo);
	};
      lpblocinfo->lpmoteur->solve_portal ();
    }
};

NIVEAU::NIVEAU (MOT * nom_fichier, LPBLOCINFO lpblocinfo)
{
  charge_niveau (nom_fichier, lpblocinfo);
}

void
NIVEAU::affichage (LPBLOCINFO lpblocinfo)
{
  for (LISTE_SALLE::iterator salle = liste_salle.begin ();
       salle != liste_salle.end (); salle++)
    {
      (*salle)->trace_animation (lpblocinfo);
    };
};

void
NIVEAU::gestion_globale (LPBLOCINFO lpblocinfo)	// signale aux salles de s'animer
{
  for (LISTE_SALLE::iterator salle = liste_salle.begin ();
       salle != liste_salle.end (); salle++)
    {
      (*salle)->gestion_globale (lpblocinfo);
    };

  // Si on est serveur, on ecrit.
  if (lpblocinfo->serveur)
    {
      STOCK stock (64 * 1024);
      for (LISTE_SALLE::iterator salle = liste_salle.begin ();
	   salle != liste_salle.end (); salle++)
	{
	  (*salle)->ecriture (lpblocinfo, stock);
	};
      int taille;
      char *buffer = NULL;
      stock.get_buffer (&buffer, &taille);
      lpblocinfo->lpreseau->put_UDP (CANAL_NIVEAU, taille, buffer);
      if (buffer)
	free (buffer);
    }
  if ((lpblocinfo->client) && (!lpblocinfo->serveur))
    // Si on est client et qu'il y a qqch, on lit le bloc.
    {
      int position = 0;
      unsigned int taille;
      void *buffer;
      STOCK stock (64 * 1024);
      lpblocinfo->lpreseau->get_UDP (CANAL_NIVEAU, &taille, &buffer);
      stock.put_buffer ((char *) buffer, taille);
      if (buffer)
	{
	  int position = 0;
	  for (LISTE_SALLE::iterator salle = liste_salle.begin ();
	       salle != liste_salle.end (); salle++)
	    {
	      (*salle)->lecture (lpblocinfo, stock, position);
	    };
	  free (buffer);
	}
    }
  if (lpblocinfo->client)
    {
      if (lpblocinfo->lpmixer->fin_son (musique_handler))
	{
	  musique_handler = lpblocinfo->lpmixer->lance_son (musique, 1);
	  lpblocinfo->lpmixer->set_volume (musique_handler, 128);
	}
    }
};

void
NIVEAU::donne_info_niveau (MOT * nom, LPBLOCINFO lpblocinfo,
			   INFO_NIVEAU * info)
{
  BUFFER_NOEUD BF (lpblocinfo->lpbdd, nom);
  buffer_noeud *bf, *bf2, *bf3;
  bf = BF.ROOT ();
  if (!bf)
    printf ("Fichier niveau info non-trouv !!! %s\n", nom->info);
  while (bf)
    {
      if (BF.get_nom (bf) == CONSTANTE_SCRIPT_INFO_NIVEAU)
	{
	  MOT nom_salle, nom_animation;
	  bf2 = BF.get_fils (bf);
	  bf3 = BF.trouve_suivant (bf2, CONSTANTE_SCRIPT_NOM_NIVEAU);
	  if (bf3)
	    info->nom_niveau = BF.get_MOT (bf3);
	  bf3 = BF.trouve_suivant (bf2, CONSTANTE_SCRIPT_NOM_SCREENSHOT);
	  if (bf3)
	    info->screenshot = BF.get_MOT (bf3);

	  bf3 = bf2;

	  while (bf3)
	    {
	      bf3 = BF.trouve_suivant (bf3, CONSTANTE_SCRIPT_INFO_NIVEAU);
	      if (bf3)
		info->info.insert (info->info.end (), BF.get_MOT (bf3));
	      bf3 = BF.trouve_suivant (bf3);
	    }

	}
      bf = BF.trouve_suivant (bf);
    }
}

bool
  NIVEAU::impact (RWVECTOR & a, RWVECTOR & b, LPFACETTE * touche,
		  LPOBJET * lpo)
{
  bool retour = false;
  *touche = NULL;
  *lpo = NULL;
  for (LISTE_SALLE::iterator salle = liste_salle.begin ();
       salle != liste_salle.end (); salle++)
    {
      if ((*salle)->impact (a, b, touche, lpo))
	retour = true;
    };
  return retour;
}

void
NIVEAU::impact_sphere (RWVECTOR & point, float distance,
		       float distance_facette, int ID, LPLISTE_OBJET & liste,
		       float temps)
{
  for (LISTE_SALLE::iterator salle = liste_salle.begin ();
       salle != liste_salle.end (); salle++)
    {
      (*salle)->impact_sphere (point, distance, distance_facette, ID, liste,
			       temps);
    };
}
