/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//#ident "$Id: snd_mixer.cpp,v 1.5 2006/12/28 10:50:43 atila Exp $"
//#include "confdefs.h" // rzr si on active la marco DONT_WANT_SOUND //!+rzr
#include "snd_mixer.h"
#include "SDL/SDL.h"

#ifdef DONT_WANT_SOUND		//!+rzr
#define SDL_MixAudio
#define SDL_OpenAudio
#define SDL_LockAudio()
#define SDL_UnlockAudio()
#endif //!-rzr

#ifndef DONT_WANT_SOUND
#include "SDL/SDL_sound.h"
#endif

#define NUM_SOUNDS 1024

void mixaudio (void *unused, Uint8 * stream, int len);

struct sample
{
  Uint8 *data;
  Uint32 dpos;
  Uint32 dlen;
  int volume;
} sounds[NUM_SOUNDS];

void
mixaudio (void *unused, Uint8 * stream, int len)
{
  int i;
  Uint32 amount;

  for (i = 0; i < NUM_SOUNDS; ++i)
    {
      if ((sounds[i].dpos == sounds[i].dlen) || (sounds[i].data == NULL))
	{
	  sounds[i].data = NULL;
	}
      else
	{
	  amount = (sounds[i].dlen - sounds[i].dpos);
	  if (amount > len)
	    {
	      amount = len;
	    }

	  SDL_MixAudio (stream, &sounds[i].data[sounds[i].dpos], amount,
			SDL_MIX_MAXVOLUME * (255 - sounds[i].volume) / 255);

	  sounds[i].dpos += amount;
	}
    }
}

int
play_sound (Uint8 * data, int len)
{
  int index;

  /* Look for an empty (or finished) sound slot */
  for (index = 0; index < NUM_SOUNDS; ++index)
    {
      if ((sounds[index].dpos == sounds[index].dlen)
	  || (sounds[index].data == NULL))
	{
	  break;
	}
    }
  if (index == NUM_SOUNDS)
    return -1;

  SDL_LockAudio ();
  sounds[index].data = data;
  sounds[index].dlen = len;
  sounds[index].dpos = 0;
  sounds[index].dpos = 128;
  SDL_UnlockAudio ();
  return index;
}

/*****************************************
SOUND_BUFFER
******************************************/


SOUND_BUFFER::SOUND_BUFFER (MOT & fic, LPBDD lpbdd)
{
  fichier = fic;
  // Recuperation des donnees.
  char *buffer = NULL;
  taille = 0;
  lpbdd->extrait_fichier_buffer (&fic, (char **) &buffer, &taille);
  // Decompression;
#ifndef DONT_WANT_SOUND		//!+rzr
  Sound_AudioInfo *desire =
    (Sound_AudioInfo *) malloc (sizeof (Sound_AudioInfo));
  desire->format = AUDIO_S16;
  desire->rate = 22050;
  desire->channels = 2;
#endif
  const char *error;
  SDL_RWops *rw = SDL_RWFromMem (buffer, taille);
  MOT format ("WAV");
  if (fic.info[fic.taille] == '3')
    format = MOT ("MP3");
  if (fic.info[fic.taille] == 'g')
    format = MOT ("OGG");
#ifndef DONT_WANT_SOUND		//!+rzr
  Sound_Sample *sample =
    Sound_NewSample (rw, format.info, desire, 1024 * 1024);
  free(desire);
  Sound_DecodeAll (sample);
  data = (char *) malloc (sample->buffer_size);
  memcpy (data, sample->buffer, sample->buffer_size);
  taille = sample->buffer_size;
  Sound_FreeSample (sample);
  free(buffer);
#endif
};

SOUND_BUFFER::~SOUND_BUFFER ()
{
  if (data)
    free (data);
};

bool
SOUND_BUFFER::operator== (MOT & test)
{
  return fichier == test;
};

char *
SOUND_BUFFER::get_ptr (int &t)
{
  t = taille;
  return data;
}

/*****************************************
SOUND_CONTROL
******************************************/

SOUND_CONTROL::SOUND_CONTROL (int i, SOUND_BUFFER * snd_buff)
{
  controle_position = false;
  sound_number = i;
  int taille = 0;
  sound_buffer = snd_buff;
  char *buffer = sound_buffer->get_ptr (taille);
  channel = play_sound ((Uint8 *) buffer, taille);
};

SOUND_CONTROL::~SOUND_CONTROL ()
{
  termine_son ();
};

bool
SOUND_CONTROL::operator== (int i)
{
  return i == sound_number;
};

bool
SOUND_CONTROL::fin ()
{
  if (channel == -1)
    return false;
  bool ret = false;
  SDL_LockAudio ();
  if (sounds[channel].data)
    ret = true;
  SDL_UnlockAudio ();
  return ret;
};

int
SOUND_CONTROL::setVolume (int volume)
{
  if (channel == -1)
    return false;
  SDL_LockAudio ();
  if (sounds[channel].data)
    sounds[channel].volume = volume;
  SDL_UnlockAudio ();
};

int
SOUND_CONTROL::gestion (RWVECTOR position_courante)
{
  // mettre  jour le volume si il dpend de la position courante.
  if (controle_position)
    {
      float distance = (position_courante - position).norme ();
      float volume = 1000.0 / (distance);
      if (volume > 255.0)
	volume = 255.0;
      volume = 255.0 - volume;
      setVolume ((int) volume);
    };
};

void
SOUND_CONTROL::set_position (RWVECTOR position_actuelle)
{
  controle_position = true;
  position = position_actuelle;
};

void
SOUND_CONTROL::termine_son ()
{
  if (channel == -1)
    return;
  SDL_LockAudio ();
  sounds[channel].data = NULL;
  SDL_UnlockAudio ();
};


/*****************************************
SOUND_BUFFER
******************************************/

SOUND_BUFFER *
MIXER::trouve_buffer_son (MOT & son)
{
  int trouve = -1;
  //printf("Recherche\n");
  //printf("Recherche %i\n",sound_buffer_list.size());
  for (int i = 0; i != sound_buffer_list.size (); i++)
    {
      if ((*sound_buffer_list[i]) == son)
	return sound_buffer_list[i];
    }
  SOUND_BUFFER *snd = new SOUND_BUFFER (son, lpbdd);
  sound_buffer_list.insert (sound_buffer_list.begin (), snd);
  return snd;
};

int
MIXER::lance_son (MOT & son, int volume)
{
  if (son.info[0] == 0)
    return -1;
  //printf("Joue %s\n",son.info);
  SOUND_BUFFER *snd = trouve_buffer_son (son);
  compteur++;
  SOUND_CONTROL *snd_ctrl = new SOUND_CONTROL (compteur, snd);
  sound_control_list.insert (sound_control_list.begin (), snd_ctrl);
  return compteur;
};

SOUND_CONTROL *
MIXER::trouve_sound_control (int son)
{
#ifndef DONT_WANT_SOUND		//!+rzr
  //printf("taille liste son %i\n",sound_control_list.size ());
  for (int i = 0; i != sound_control_list.size (); i++)
    {
      if ((*(sound_control_list[i])) == son)
	return sound_control_list[i];
    }
#endif
  return NULL;

};

int
MIXER::controle_son (int son, int volume)
{
#ifndef DONT_WANT_SOUND		//!+rzr
  SOUND_CONTROL *snd = trouve_sound_control (son);
  if (snd)
    {
      snd->setVolume (volume);
      return true;
    };
#endif
  return false;
};

int
MIXER::controle_position_son (int son, RWVECTOR position)
{
#ifndef DONT_WANT_SOUND		//!+rzr
  SOUND_CONTROL *snd = trouve_sound_control (son);
  if (snd)
    {
      snd->set_position (position);
      return true;
    };
#endif
  return false;
};

bool
MIXER::fin_son (int son)
{
#ifndef DONT_WANT_SOUND		//!+rzr
  SOUND_CONTROL *snd = trouve_sound_control (son);
  if (snd)
    {
      return false;
    };
#endif
  return true;
};

MIXER::MIXER (LPBDD bdd)
{
  compteur = 1;
  lpbdd = bdd;
};

MIXER::~MIXER ()
{
  deque < SOUND_CONTROL * >::iterator i = sound_control_list.begin ();
  while (i != sound_control_list.end ())
    {
      SOUND_CONTROL *a = *i;
      delete a;
      sound_control_list.erase (i, i + 1);
      i = sound_control_list.begin ();
    }
  for (deque < SOUND_BUFFER * >::iterator j = sound_buffer_list.begin (); 
       j != sound_buffer_list.end () ; 
       j++)
    {
      delete *j;
    };
};


void
MIXER::gestion ()
{
  deque < SOUND_CONTROL * >::iterator i = sound_control_list.begin ();
  while (i != sound_control_list.end ())
    {
      if (!((*i)->fin ()))
	{
	  SOUND_CONTROL *a = *i;
	  delete a;
	  sound_control_list.erase (i, i + 1);
	  i = sound_control_list.begin ();
	}
      else
	i = i + 1;
    }
};

void
MIXER::gestion (RWVECTOR position)
{
  gestion ();
  deque < SOUND_CONTROL * >::iterator i = sound_control_list.begin ();
  while (i != sound_control_list.end ())
    {
      (*i)->gestion (position);
      i = i + 1;
    }

};

int
MIXER::set_volume (int son, int volume)
{
  SOUND_CONTROL *snd = trouve_sound_control (son);
  if (snd)
    {
      snd->setVolume (volume);
    };
};

void
MIXER::termine_son (int son)
{
  SOUND_CONTROL *snd = trouve_sound_control (son);
  if (snd)
    {
      snd->termine_son ();
    };
};

void init_son()
{
#ifndef DONT_WANT_SOUND		//!+rzr
  Sound_Init ();
  SDL_AudioSpec fmt;
  fmt.freq = 22050;
  fmt.format = AUDIO_S16;
  fmt.channels = 2;
  fmt.samples = 512;	/*Une bonne valeur pour les jeux */
  fmt.callback = mixaudio;
  fmt.userdata = NULL;
  printf ("Init audio\n");
  if (SDL_OpenAudio (&fmt, NULL) < 0)
    {
      fprintf (stderr, "Impossible d'accder  l'audio: %s\n",
	       SDL_GetError ());
      exit (1);
	}
  for (int index = 0; index < NUM_SOUNDS; ++index)
    sounds[index].data = NULL;
  const Sound_DecoderInfo **i = Sound_AvailableDecoders ();
  for (; *i != NULL; i++)
    {
      const char **ext = (*i)->extensions;
	  for (; *ext != NULL; ext++)
	    printf ("[%s]", *ext);
	  printf ("- [%s].\n", (*i)->description);
    }
  SDL_PauseAudio (0);
#endif
}
